<?php
/**
 * Bittorrent seeder
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2007 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.8
 */

define('TIMEOUT',120);
define('MIN_UNCHOKE_TIME',8);
define('MAX_PROCESSED_MESSAGES_PER_LOOP',8);
require(INCLUDE_DIR.'transferFunctions.php');

/**
 * @desc Extract needed vars from general.ini config, and release all others from memory
 *
 */
function bittorrentSeederSetEnv(){
	$_ENV['weezoGeneral']=cfMGetVar('weezoGeneral');
	$var['directLinkDownloadSpeedLimit']=@$_ENV['weezoGeneral']['directLinkDownloadSpeedLimit'];
	$var['bittorrentPort']=@$_ENV['weezoGeneral']['bittorrentPort'];
	$var['bittorrentEnabled']=@$_ENV['weezoGeneral']['bittorrentEnabled'];
	unset($_ENV['weezoGeneral']);
	$_ENV['weezoGeneral']=$var;
	unset($var);
}

/**
 * @desc debug to output
 *
 * @param string $message
 */
function out($message,$debugLevel=LOG_DBG){
	static $debugThreshold=-1;
	if($debugThreshold<0) $debugThreshold=(isset($_GET['debugThreshold']))?$_GET['debugThreshold']:LOG_DET;
	static $excl;

	if($debugLevel>$debugThreshold) return;
	//if(time()-cfMGetVar('checkingSeeder')<100) return;

	// Remove non-printable chars
	$out='';
	for($i=0;$i<strlen($message);$i++) {
		$c=substr($message,$i,1); $kc=ord($c);
		if($kc<32) $out.='('.$kc.')'; else $out.=$c;
	}

	// Echo or sent do debugSingle
	if(cfMGetVar('debug') || isset($_GET['debug'])) {echo $out.'<br>'; flush();}
	//return cfDebugSingle($out);
}

/**
 * @desc read transfers into $tranfers array
 *			reload only if necessary
 */
function btLoadTransfers(){
	global $transfers;
	global $weezoTransfersLastUpdate;

	if($weezoTransfersLastUpdate==cfMGetVar('weezoTransfersLastUpdate')) return;

	$transfers=tReadTransfersFiles();
	$weezoTransfersLastUpdate=cfMGetVar('weezoTransfersLastUpdate');
}

/**
 * @desc Update and save transfer state
 *
 * @param string $key: $connections array key
 * @param integer $state: new requested state
 * @param integer $writtenBytes: nb of written bytes since last update
 * @return integer: set state
 */
function btUpdateTransfer($key, $state, $writtenBytes=0){
	global $transfers;
	global $connections;
	$needSave=false;

	btLoadTransfers();

	// If transfer not found, close connection
	if(!isset($transfers[$connections[$key]['tid']])){
		unset($connections['key']);
		return null;
	}
	$tr=$transfers[$connections[$key]['tid']];
	$prevState=$tr->status;
	if($prevState==4 || $prevState==6 || $prevState==8 || $prevState==9) return $prevState;

	if($state!=$prevState){
		$tr->lastUpdated=time();
		$tr->status=$state;
		$needSave=true;
	}
	if($writtenBytes) {
		$tr->sentDataLength+=$writtenBytes;
		$tr->updateProgress($tr->sentDataLength/$tr->size);
		//out('update progress '.($tr->sentDataLength/$tr->size));
		$needSave=true;
	}
	if($needSave && $prevState!=$state) $tr->commitToFile($transfers,true);
	elseif($needSave && $prevState==$state) $tr->commitToFile($transfers,'time');
	$transfers[$connections[$key]['tid']]=$tr;
	return $state;
}

/**
 * @desc Display string in binary and/or decimal formats
 *
 * @param string $str
 * @param boolean $returnBits : show bits formated string
 * @param boolean $returnBytes : show decimal formated string
 * @return string
 */
function stringToBits($str, $returnBits=true, $returnBytes=true){
	$output='';
	if($returnBits){
		for($i=0;$i<strlen($str);$i++){
			$bits=8;
			$a=ord($str[$i]);
			for($offset=pow(2,$bits-1);$offset>=1;$offset=$offset/2) $output.=(($a & $offset)?'1':'0');
			$output.= ' ';
		}
		if($returnBytes) $output.= '<br>';
	}

	if($returnBytes){
		for($i=0;$i<strlen($str);$i++){
			$output.= ord($str[$i]).' ';
		}
	}
	return $output;
}

/**
 * @desc return a bittorrent formated message
 *
 * @param integer $id : id (type) of message
 * @param string $payload : message's payload
 * @return string
 */
function btMessage($id,$payload=''){
	if($id===null) return chr(0).chr(0).chr(0).chr(0);
	return pack('N',strlen($payload)+1).chr($id).$payload;
}

/**
 * @desc generate bitfield message for complete torrent
 * @param integer $pieces : number of pieces in torrent
 *
 * @return string bittorrent formated message
 */
function btMessageBitfield($pieces){
	return btMessage(5, str_pad('',floor($pieces/8),chr(255)).(($pieces%8)?chr((255)>>(8-$pieces%8)<<(8-$pieces%8)):''));
}

/**
 * @desc Label of bittorrent message (used for debugging)
 *
 * @param integer $id : bittorrent id of message
 * @return string description
 */
function btIdLabel($id){
	switch ($id){
		case 0:
			return 'choke';
		case 1:
			return 'unchoke';
		case 2:
			return 'interested';
		case 3:
			return 'not interested';
		case 4:
			return 'have';
		case 5:
			return 'bitfield';
		case 6:
			return 'request';
		case 7:
			return 'piece';
		case 8:
			return 'cancel';
		default:
			return 'unknown id ('.$id.')';
	}
}

/**
 * @desc Format bittorrent message for visual debugging
 *
 * @param integer $id : bittorrent id of message
 * @param string $payload : bittorrent message payload
 * @return string
 */
function btFormatedPayload($id, $payload){
	switch ($id){
		case 0:// choke
		case 1:// unchoke
		case 2:// interested
		case 3:// not interested
			if(strlen($payload)) return 'ERROR : no payload expected ! : '.$payload;
			return 'no payload';
		case 4: // have : 32 bit integer, index of piece
			if(strlen($payload)!=4) return 'ERROR : incorrect payload size, 4 bytes expected ! : '.$payload;
			list($len)=array_values(unpack('N',$payload));
			return $payload.' => Piece n'.$len;
		case 5: // bitfield
			return stringToBits($payload,true,false);
		case 6: // request
			if(strlen($payload)!=12) return 'ERROR : incorrect payload size, 3*4 bytes expected for REQUEST message ! : , '.strlen($payload).' provided :'.$payload;
			list($index,$begin,$length)=array_values(unpack('N3l',$payload));
			return 'Piece index : '.$index.', begin : '.$begin.', length : '.$length;
		case 7: // piece
			if(strlen($payload)<8) return 'ERROR : incorrect payload size, >= 2*4 bytes expected for PIECE message ! : '.$payload;
			list($index,$begin)=array_values(unpack('N2l',substr($payload,0,8)));
			return 'Piece index : '.$index.', begin : '.$begin.', DATA ('.(strlen($payload)-8).') Bytes';
		case 8:
			if(strlen($payload)!=12) return 'ERROR : incorrect payload size, 3*4 bytes expected for CANCEL message! : '.$payload;
			list($index,$begin,$length)=array_values(unpack('N3l',$payload));
			return 'Cancel index : '.$index.', begin : '.$begin.', length : '.$length;
		default:
			return 'Unknown format : '.$payload;
	}
}

/**
 * @desc process incoming bittorrent message
 *
 * @param string $key : key of connection in $connections array
 * @param integer $id : bittorrent id of message
 * @param string $payload : bittorrent message payload
 * @return null if connection has been shutdown
 */
function btProcessMessage($key, $id, $payload){
	global $connections;
	static $debug=0;
	if($id==4) {
		$debug++;
		out ($debug.' PIECES RECEIVED');
	}
	switch ($id){
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			return false;

		case 6: // request
			if(strlen($payload)!=12) return false;

			// Parse payload
			list($index,$begin,$length)=array_values(unpack('N3l',$payload));

			// Update speed limitation if needed
			if(@$connections[$key]['transferLimitationType']=='global'){
				if(cfGGetVar('directLinkDownloadSpeedLimit')){
					if(!$connections[$key]['transferLimitation']) $connections[$key]['transferLimitation']=new transferLimitation(1024*cfGGetVar('directLinkDownloadSpeedLimit'));
					else $connections[$key]['transferLimitation']->setSpeedLimit(1024*cfGGetVar('directLinkDownloadSpeedLimit'));
				}
				else $connections[$key]['transferLimitation']=false;
			}

			// Check speed limitation
			if($connections[$key]['transferLimitation'] && !@$connections[$key]['transferLimitation']->isReady()){
				out('CHOKE client',LOG_DET);
				// return btMessage(8, pack('N',$index).pack('N',$begin).pack('N',$length)); // cancel request
				// Clear pending messages and in-buffer
				$connections[$key]['choked']=time();
				$connections[$key]['inBuffer']='';
				$connections[$key]['inMessages']=array();
				// CHOKE request
				return btMessage(0);
			}


			// Load requested piece in memory (if needed)
			if($index!=$connections[$key]['inMemoryPiece']){

				$pl=$connections[$key]['pieceLength'];
				// Erase previous cached data
				$connections[$key]['inMemoryPieceData']='';
				$start=$pl*$index;
				$read=0;
				$readStarted=false;

				// Browse files
				foreach ($connections[$key]['filesOffsets'] as $sk=>$fo) {
					$offset=0;
					// Find first file
					if(!$readStarted && $fo['end']>=$start && $fo['start']!=$fo['end']){
						$readStarted=true;
						$offset=$start-$fo['start'];
						out('Start sending data from file '.$fo['name'].', from byte:'.$offset,LOG_DET);
					}
					// Read
					if($readStarted){
						$toRead=min((1+$fo['end']-$fo['start'])-$offset,$pl-$read);
						if(0){
							if($handle=fopen($fo['name'],'rb')){
								fseek($handle,$offset,SEEK_SET);
								$connections[$key]['inMemoryPieceData'].=fread($handle,$toRead);
								fclose($handle);
							}
						}
						else{
							$readData=cfFread($fo['name'],$offset,$toRead);
							if(strlen($readData)==0){
								out('Error : cannot open source file : '.$fo['name'],LOG_INF);
								socket_close($connections[$key]['cnx']);
								unset($connections[$key]);
								return null;
							}
							out('Read '.strlen($readData).'/'.$toRead.' at:'.($offset).' MB from '.$fo['name']);
							$connections[$key]['inMemoryPieceData'].=$readData;
						}
						out('add '.$toRead.' bytes from '.$fo['name']);
						$read+=$toRead;
						// All data red
						if($read==$pl) break;
					}
				}
				$connections[$key]['inMemoryPiece']=$index;
			}
			// Update transfer (running, data sent)
			if(btUpdateTransfer($key, 3, strlen(substr($connections[$key]['inMemoryPieceData'],$begin,$length)))===null) return null;

			// Return data
			return btMessage(7,pack('N',$index).pack('N',$begin).substr($connections[$key]['inMemoryPieceData'],$begin,$length));

		case 7: // receive piece : unsupported
			out('Receive piece (unsupported, dropped)');
			return false;
		case 8: // cancel
			if(strlen($payload)!=12) return false;
			// Transform cancel message into matching request message to find it into messages list
			$searched=chr(8).$payload;
			// browse in-messages for canceled request
			foreach ($connections[$key]['inMessages'] as $msgKey=>$msg) if($msg==$searched){
				unset($connections[$key]['inMessages'][$msgKey]);
				out('Request successfully canceled',LOG_INF);
				return false;
			}
			out('Request couldn\'t be canceled',LOG_INF);
			return false;
		default:
			out('Drop unknown message id='.$id,LOG_INF);
			return false;
	}
}

/**
 * @desc Write connection outgoing buffer
 *
 * @param string $key : id of connection to use, in $connections array
 * @param string $data : data to be queued to buffer
 * @desc return remaining bytes in buffer, or false if socket has been closed
 */
function btSocket_write($key, $data=''){
	global $connections;

	// Transfer speed limitation, add data length to transfer limitation
	if($connections[$key]['transferLimitation']) $connections[$key]['transferLimitation']->addData(strlen($data),false,false);

	// Queue data in buffer
	$connections[$key]['outBuffer'].=$data;
	// If buffer empty, exit
	if(!strlen($connections[$key]['outBuffer'])) return 0;
	socket_clear_error($connections[$key]['cnx']);
	// Write
	$connections[$key]['outBuffer']=substr($connections[$key]['outBuffer'],@socket_write($connections[$key]['cnx'],$connections[$key]['outBuffer']));
	if(strlen($connections[$key]['outBuffer'])) out('Send '.strlen($connections[$key]['outBuffer']).' bytes to client');

	// Detect closed connection
	if(socket_last_error($connections[$key]['cnx'])==10053){
		socket_close($connections[$key]['cnx']);
		// Set transfer as paused
		btUpdateTransfer($key, 5, 0);
		out('Socket closed on write error',LOG_DET);
		unset($connections[$key]);
		return NULL;
	}
	return strlen($connections[$key]['outBuffer']);
}

/**
 * @process data received from connection
 *
 * @param string: $key : connection key
 */
function btProcessInBuffer($key){
	global $transfers;
	global $connections;
	global $seederId;

	// Empty buffer => exit
	if(!$connections[$key]['inBuffer']) return;
	while(true){
		$buff=$connections[$key]['inBuffer'];

		/**
		 * Bittorrent handshake
		 */
		if(strlen($buff)>=48 && ord($buff[0])==19 && strtolower(substr($buff,1,19))=='bittorrent protocol'){
			out('Bittorrent handshake detected',LOG_DET);
			$connections[$key]['type']='bittorrent';

			// Update connection
			if(strlen($buff)>=68){
				$message=substr($buff,0,68);
				$connections[$key]['inBuffer']=substr($buff,68);
				$torrentIdHash=str_replace('-','%2D',str_replace('.','%2E',str_replace('_','%5F',strtoupper(rawurlencode(substr($message,28,20))))));
				$clientIdHash=substr($message,48,20);
			}
			else{
				$message=substr($buff,0,48);
				$torrentIdHash=str_replace('-','%2D',str_replace('.','%2E',str_replace('_','%5F',strtoupper(rawurlencode(substr($message,28,20))))));
				$clientIdHash=false;
			}

			btLoadTransfers();

			// Seek matching torrent in tranfers
			$found=false;
			foreach ($transfers as $trId=>$tr) {
				if(strtolower(rawurldecode($torrentIdHash))==strtolower(rawurldecode($tr->infoHash))){
					if($tr->status!=6 && $tr->status!=8 && $tr->status!=9){
						$connections[$key]['tid']=$trId;
						$found=true;
						out('Requested torrent found',LOG_INF);
						out($tr->detail(),LOG_INF);
						break;
					}
				}
			}

			// If not found in transfers, hang-up
			if(!$found){
				out('Requested torrent not found',LOG_ERROR);
				socket_close($connections[$key]['cnx']);
				unset($connections[$key]);
				return false;
			}

			// Load transfer data into torrent
			$connections[$key]['pieceLength']=$tr->pieceLength;
			$filesOffsets=explode('|',$tr->filesOffsets);
			$startOffset=0;
			foreach (explode('|',$tr->filesOffsets) as $fo) if(strlen($fo)){
				list($completeFilename,$endOffset)=explode('?',$fo);
				$connections[$key]['filesOffsets'][]=array('name'=>$completeFilename,'start'=>$startOffset,'end'=>$endOffset-1);
				out($completeFilename.' File Offset:'.$startOffset.'->'.$endOffset,LOG_INF);
				$startOffset=$endOffset;
			}


			/**
			 * Transfer speed limitation
			 */

			// No user Id: use global direct link speed limitation
			if(!$tr->userId) {
				$connections[$key]['transferLimitationType']='global';
				$speedLimit=cfGGetVar('directLinkDownloadSpeedLimit');
			}
			// Passed user ID => search for speed limit for this user
			else{
				$connections[$key]['transferLimitationType']='user';
				$users=cfMGetVar('weezoUsers');
				if(isset($users[$tr->userId]) && isset($users[$tr->userId]['downloadSpeedLimit']))
					$speedLimit=$users[$tr->userId]['downloadSpeedLimit'];
				unset($users);
			}

			if(@$speedLimit){
				$connections[$key]['transferLimitation']=new transferLimitation(1024*$speedLimit);
				out('SET LIMITATION '.$speedLimit.' KB/s',LOG_INF);
			}

			// Handshake response
			$handShakeResponse=substr($message,0,20).str_pad('',8,chr(0)).substr($message,28,20).str_pad($seederId,20,chr(0));
			if(!$connections[$key]['handshaked']){
				if(btSocket_write($key,$handShakeResponse)===null) return;
				out('>>>> handshake response:'.$handShakeResponse);
				$connections[$key]['handshaked']=true;
			}

			// Update transfer state (connecting)
			btUpdateTransfer($key, 7, 0);

			// Partial handshake received : don't send anything else than handshake
			if(!$clientIdHash) return;

			// Send bitfield
			if(btSocket_write($key,btMessageBitfield($tr->pieces))===null) return;
			out('>>>> bitfield : '.btMessageBitfield($tr->pieces));

			// Unchoke client
			if(btSocket_write($key,btMessage(1))===null) return;
			out('>>>> unchoke : '.btMessage(1));

			// Set myself as not interested
			//if(btSocket_write($key,btMessage(3))===null) return;
			//out(socket_last_error($connections[$key]['cnx']).' '.socket_strerror(socket_last_error($connections[$key]['cnx'])));
			//out('>>>> not interested : '.btMessage(3));

			continue;
		}


		/**
		 * Bittorrent message
		 */
		if((true || $connections[$key]['type']=='bittorrent') && strlen($buff)>=4){
			list($len)=array_values(unpack('N',substr($buff,0,4)));
			$buff=substr($buff,4);
			if($len>strlen($buff)) break; // message length < announced message length :

			// Non-empty message
			if($len){
				$id=ord(substr($buff,0,1));
				$payload=substr($buff,1,$len-1);
				out('Add to msg list: '.$id.'('.btIdLabel($id).') = "'.btFormatedPayload($id,$payload).'" ('.(strlen($payload)).')');

				// If "cancel" message, proceed now (remove matching piece request from messages stack)
				if($id==8) btProcessMessage($key,$id,$payload);
				// else, add message to message stack
				else $connections[$key]['inMessages'][]=chr($id).$payload;
			}
			// Empty message = keep alive
			else {
				$id=null;
				$payload=false;
				out('BT message OK : keep alive');
			}

			// Remove processed data from in-buffer
			$connections[$key]['inBuffer']=substr($connections[$key]['inBuffer'],4+$len);

			continue;
		}
		break;
	}
}

/**
 * @process bittorrent messages stack
 *
 * @param string: $key : connection key
 * @return connection : processed connection
 */
function btProcessMessages($key){
	global $connections;
	$processed=0;
	while ($processed<MAX_PROCESSED_MESSAGES_PER_LOOP && count($connections[$key]['inMessages'])){
		$message=array_shift($connections[$key]['inMessages']);
		$id=ord(substr($message,0,1));
		$payload=substr($message,1);
		out('Buff: '.count($connections[$key]['inMessages']).' msgs; Process '.$id.'=>'.btFormatedPayload($id,$payload));
		// Process message
		if($response=btProcessMessage($key,$id,$payload)) {
			out('>>>> '.btFormatedPayload(ord(substr($response,4,1)),substr($response,5)));
			if(btSocket_write($key,$response)===null) return false;
		}
		// If connection has been shutdown, exit
		if($response===null) return false;
	}
}

if($_SERVER['REMOTE_ADDR']!='127.0.0.1' || !isset($_GET['action'])) exit;
echo '<head><title>'.$_GET['action'].'</title></head><body>';


/**
 * Array of bittorrent clients connections
 */
$connections=array();
$transfers=array();


wSession_write_close(false); unset($_SESSION);// Close session (if any)
ob_implicit_flush(); set_time_limit(0); ignore_user_abort(true);

// Start thread logging even if not enabled, as thread debug activation is manual and cannot be triggered before bittorrent seeder starts
cfThreadsDebugInit();

$weezoTransfersLastUpdate=-1;

// Release unneeded env vars
bittorrentSeederSetEnv();

// Stop server
if($_GET['action']=='stop' || $_GET['action']=='restart'){
	if(isset($_GET['bittorrentPort'])) $port=$_GET['bittorrentPort']; else $port=cfGGetVar('bittorrentPort');
	echo 'Send stop request on port '.$port.'<br>';
	if($handle=@fsockopen('127.0.0.1',$port,$errno,$errstr,4)) {
		fwrite($handle,'stopBittorrentSeeder');
		$data=''; while (!feof($handle)) $data.=fread($handle,1024);
		fclose($handle);
		if(isset($_GET['app'])) echo 'OK'; else echo $data;
	}
	else echo 'NOK';
	echo '<br>';
	if($_GET['action']=='restart') sleep(1);
}

if((@$_GET['action']!='start' && @$_GET['action']!='restart') || (!cfGGetVar('bittorrentEnabled') && !isset($_GET['debug']))) die('End of command');



// SERVER SCRIPT

// Create socket
if(!$serverSock = socket_create(AF_INET,SOCK_STREAM,SOL_TCP)) die('BITTORRENT (STOPPED)<br>Error creating socket');
if(!@socket_bind($serverSock,'0.0.0.0',cfGGetVar('bittorrentPort'))) die('BITTORRENT (STOPPED)<br>Port '.cfGGetVar('bittorrentPort').' already used');
;
socket_listen($serverSock); socket_set_nonblock($serverSock);

if($_GET['action']!=='restart') echo '<head><title>BITTORRENT</title></head><body>'; else echo 'Bittorrent Seeder started...<br>';
flush();

out("----------------------------------------------------------------------------------------",LOG_INF);
out("Server started on port ".cfGGetVar('bittorrentPort'),LOG_INF);
out('Debug activated: ' );
cfLog('BitTorrent Seeder started',2);

// Set seeder ID
if($regInfo=cfRegistrationData()) $seederId=substr('-WZ0000-'.$regInfo['regName'],0,20); else $seederId='-WZ0000-unRegistered';


$i=0;
$shutdown=false;

//set_socket_blocking($serverSock,0);


/****************************************************************************************************************
 * MAIN LOOP
 ***************************************************************************************************************/
while(!$shutdown) {
	/**
	 * New connection
	 */
	if($c = @socket_accept($serverSock)) {
		if(!@socket_getpeername ($c, $addr, $port)) $addr=false;
		$connections[]=array(
			'tid'=>false, // Connection Id, matches with tranfer Id
			'cnx'=>$c, // Connection socket
			'remoteIp'=>$addr, // Client's IP
			'lastPing'=>time(), // Last message received from client, timestamp
			'type'=>null, // Connection type (='bittorrent' if handshake detected)
			'inBuffer'=>'', // Current unprocessed buffer (received from client)
			'inMessages'=>array(), // Current unprocessed messages (received from client)
			'outBuffer'=>'', // Current unprocessed buffer (received from client)
			'pieceLength'=>0, // Size of pieces, in bytes
			'filesOffsets'=>array(), // Array of files name,start offset, end offset
			'inMemoryPiece'=>-1, // n of piece loaded into memory
			'inMemoryPieceData'=>false, // data of piece loaded into memory
			'handshaked'=>false, // True if seeder has returned handshake
			'choked'=>false, // True if seeder is choked (overspeed)
			'transferLimitation'=>false, // Transfer speed limitation object
			'transferLimitationType'=>false // Transfer speed limitation type : 'global' to use cfGGetVar('directLinkDownloadSpeedLimit'), 'user' to use user's value
			);
		out('******** New Connection ********* from '.$addr,LOG_INF);
	}

	/**
	 * Process opened connections
	 */
	foreach ($connections as $key=>$connection){
		$buff='';
		$cnx=$connection['cnx'];
		socket_clear_error($cnx);

		// Try to read data
		try{$buff=@socket_read($cnx,1024);} catch(Exception $e){out('Socket read error:'.$e->getMessage(),LOG_INF);};
		//out('read '.strlen($buff).' bytes',LOG_DET );

		// Socket closed or timeout detection
		if(socket_last_error($cnx)==10053 || ($buff=='' && socket_last_error($cnx)==0) || (time()-$connection['lastPing'] > TIMEOUT)) {
			if(time()-$connection['lastPing'] > TIMEOUT) out("timeout disconnected",LOG_INF); else out('read disconnected (last error:'.socket_last_error($cnx).')',LOG_INF);
			socket_shutdown($cnx,2);

			btLoadTransfers();

			// Update transfer state (paused)
			if(isset($transfers[$connections[$key]['tid']])) {
				if($transfers[$connections[$key]['tid']]->status!=4 && $transfers[$connections[$key]['tid']]->status!=6 && $transfers[$connections[$key]['tid']]->status!=8 && $transfers[$connections[$key]['tid']]->status!=9) btUpdateTransfer($key, 5, 0);
			}
			unset($connections[$key]);
			continue;
		}

		/**
		 * Data received from client
		 */
		if(strlen($buff)){
			// Port pinging by "DNS" server
			if(cfCmpLeft($buff,'GET /ping.php')){
				@socket_write($cnx,'<ping>ok</ping>');
				unset($connections[$key]);
				socket_shutdown($cnx,2); break;
			}

			// Server Shutdown command
			if($buff=='stopBittorrentSeeder' && $connection['remoteIp']=='127.0.0.1'){
				@socket_write($cnx,'<b style="color:white;background:#'.rand(0,999).'">Shutdown command received</b> '.time());
				socket_shutdown($cnx,2); $shutdown=true; break;
			}

			// Server ping command
			if($buff=='pingBittorrentSeeder' && $connection['remoteIp']=='127.0.0.1'){
				@socket_write($cnx,'ok');
				unset($connections[$key]);
				socket_shutdown($cnx,2); break;
			}

			// Debug output received message
			out('<<<< '.$buff);

			// Update lastPing
			$connections[$key]['lastPing']=time();

			// Completed torrent message
//			if(stripos($buff,'torrentcompleted')!==false) cfDbg('!!!');
			if(strlen($buff)==36 && substr($buff,0,16)=='torrentCompleted'){
				out('TRANSFER COMPLETED MESSAGE RECEIVED',LOG_INF);

				$infoHash=str_replace('-','%2D',str_replace('.','%2E',str_replace('_','%5F',rawurlencode(substr($buff,16)))));
				// Load transfers
				btLoadTransfers();

				// And search for the one matching the hash
				foreach($transfers as $trId=>$tr) {
					if($infoHash==$tr->infoHash){

						out('-------TRANSFER COMPLETED---------',LOG_INF);
						// Set as finished (if single download, set as completed, else reset to 0 (created) state)
						$transfers[$trId]->status=4;
						// Completed transfers. Check if bittorrent download is linked with a directLink or a publishDownload
						// and if so, process differently (check limitations)
						if($tr->linkedTransferToken){
							require_once(INCLUDE_DIR.'explorerFunctions.php');
							$tokenList=efTokensRead();

							$tid=$tr->linkedTransferToken;

							if($token=@$tokenList[$tid]){
								// Increment number of downloads for this directLink or publishDownload
								$tokenList[$tid]['limitNb']--;
								efTokensWrite($tokenList);

								// If token is still valid, reset bittorrent transfer instead of setting it completed
								if(efTokenIsValid($tokenList[$tid])){
									$state=0;
									$transfers[$trId]->trackerIdSet=false;
									$transfers[$trId]->sentDataLength=0;
									$transfers[$trId]->updateProgress(0);
								}
								// Else, remove from list
								else{
									$tr->clear(true);
								}
							}
							unset($tokenList);
						}
						$tr->commitToFile($transfers,true);
						break;
					}
				}



				continue;
			}
		}

		/**
		 * Process buffers and messages stacks
		 */

		// Unchoke client if needed
		if($connections[$key]['choked'] && (time() - $connections[$key]['choked'] > MIN_UNCHOKE_TIME) && @$connections[$key]['transferLimitation']->isReady()) {
			$connections[$key]['choked']=false;
			out('Unchoke client');
			if(btSocket_write($key,btMessage(1))===null) continue;
		}

		// Concatenate data to connection buffer if client is not choked
		if(!$connections[$key]['choked']) $connections[$key]['inBuffer'].=$buff;

		// Process bittorrent in-buffer
		btProcessInBuffer($key);

		// Process pending in-messages
		btProcessMessages($key);

		// Send not-yet sent out-buffer (if connection not closed)
		if(isset($connections[$key])) btSocket_write($key);

		// Keep alive
		//socket_write($cnx,btMessage(null));
	}
	usleep(100000);
	$i++;

	// Reload general config every 5sec (usefull for max transfer rate and maybe other params...)
	if($i%50===0) {
		bittorrentSeederSetEnv();
		out('mem:'.memory_get_usage().' speed limit:'.$_ENV['weezoGeneral']['directLinkDownloadSpeedLimit']);

		// Debug: update memory consumption and last activity time
		cfThreadsDebugUpdate();
	}
}
socket_close($serverSock);
out("************************");
out("Server shutdown");
?>
<script type="text/javascript">
document.title="BITTORRENT (STOPPED)";
</script>