<?php
/**
 * Display (and if requested resample) an image located out of document root
 *
 * parameters are passed with GET method :
 * file : file name (including full path), url encoded
 * (optional) w, h : width and height of image
 *
 * script should be called like : <img src="/ext/image.php?file=c%3foo.jpg&w=32&h=32"> or through cfExtImage function
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.1.0
 */

// Set to true to remove Content-Type header (debug)
$debugRemoveContentHeader=0;

/**
 * @desc Generate and sent image and headers to ouptut
 *
 * @param string $completeFilename
 * @param array $uri: parsed URI
 */
function imageGenerate($completeFilename,$uri){
	global $debugRemoveContentHeader;

	// Image rotation
	if(isset($uri['rot']) && is_numeric($uri['rot'])) $rotation=$uri['rot']; else $rotation=0;

	// Resample quality (get resource-level values before closing session)
	$resampleQuality=cfHGetVar('imageResampleQuality',DEFAULT_RESIZE_METHOD);

	// JPG quality (get resource-level values before closing session)
	$JPGQuality=cfHGetVar('thumbnailJPGQuality',DEFAULT_JPG_QUALITY);


	// Stretch small images (get resource-level values before closing session)
	$stretchSmallImages=cfRGetVar('stretchSmallImages');

	//close session
	$speedLimit=cfUGetVar('downloadSpeedLimit');
	wSession_write_close(); unset($_SESSION);
	set_time_limit(0);

	// Headers
	header('Keep-Alive: timeout=15, max=2000');
	header('Proxy-Connection: Keep-Alive');

	// No cache
	if(isset($uri['cache']) && $uri['cache']=='0' || $debugRemoveContentHeader) WHeaders::noCache();

	// Cache
	else WHeaders::cache($completeFilename);

	if(!isset($uri['w'])) $uri['w']=0; if(!isset($uri['h'])) $uri['h']=0;

	require(INCLUDE_DIR.'mime_type.php');

	/**
	 * File's icon
	 */
	if(isset($uri['icon'])){
		if(!$debugRemoveContentHeader) header('Content-Type: image/png');
		require_once(INCLUDE_DIR.'explorerFunctions.php');

		// Extract icon from file
		if(!($ico=efExtractIcon($completeFilename,false,((isset($uri['w']))?$uri['w']:16),array('icoPngCheck'=>1)))) {
			require_once(INCLUDE_DIR.'outputFunctions.php');
			$ico=file_get_contents(cfAppDocRoot().efIcon($completeFilename,false));
		}
		die($ico);
	}

	/**
	 * Audio files: extract cover
	 */
	if(substr(mimeType($completeFilename),0,5)=='audio'){
		imageExtractAudioCover($completeFilename,((isset($uri['w']))?$uri['w']:0),((isset($uri['h']))?$uri['h']:0),!isset($uri['noDC']));
	}

	/**
	 * Images
	 */
	if(substr(mimeType($completeFilename),0,5)=='image'){

		// if resize requested, generate resized image and send it to output
		if($rotation || $uri['w'] || $uri['h'] || (isset($uri['sq']) && $uri['sq'])){
			if(!$debugRemoveContentHeader) header('Content-Type: image/jpeg');

			if($uri['w'] || $uri['h'])
				$res=cfCreateResizedJPG($completeFilename, 0, 0, 'return', $uri['w'], $uri['h'], $resampleQuality, $JPGQuality, $rotation, (isset($uri['sq']) && $uri['sq']), $stretchSmallImages);
			else
				$res=cfCreateResizedJPG($completeFilename, 0, 0, 'return', 0, 0, $resampleQuality, $JPGQuality, $rotation, (isset($uri['sq']) && $uri['sq']), $stretchSmallImages);
			// Stream result to output
			if($res) cfStreamString($res, $speedLimit);
		}
		//If no resize and no rotation requested, directly send image to output
		else{
			$ext=strtolower(substr($completeFilename,strrpos($completeFilename,'.')+1));
			// browser supported extension : stream
			if($ext=='jpg' || $ext=='gif' || $ext=='jpeg' || $ext=='png' || $ext=='itc' || $ext=='itc2') {
				if(!$debugRemoveContentHeader){
					if($ext=='jpg' || $ext=='jpeg') header('Content-Type: image/jpeg');
					if($ext=='png' || $ext=='itc' || $ext=='itc2') header('Content-Type: image/png');
					if($ext=='gif') header('Content-Type: image/gif');
					if($ext=='ico') header('Content-Type: image/x-icon');
				}

				// Stream file to output
				if($ext=='itc2' || ($ext=='itc' && WEnv::host()->isOSX()))
					cfStreamString(substr(file_get_contents($completeFilename),492),$speedLimit);
				elseif($ext=='itc')
					cfStreamString(substr(file_get_contents($completeFilename),500),$speedLimit);
				else
					cfStreamFile($completeFilename,false,$speedLimit);
				$res=true;
			}
			// unsupported: (try to) convert to jpg
			else {
				if(!$debugRemoveContentHeader) header('Content-Type:image/jpeg');
				$res=cfCreateResizedJPG($completeFilename,0,0,"",0,0,$resampleQuality, $JPGQuality, 0,false, $stretchSmallImages);
				// Stream result to output
				if($res) cfStreamString($res,$speedLimit);
			}
		}
		// If image couldn't  be generated, return empty image
		if(!$res) echo file_get_contents(cfAppDocRoot().'/gfx/dot.jpg');

		// Update debug buffer
		if(cfMGetVar('debugAsync'))	cfDebugUpdateBuffer('DONE '.basename($completeFilename).' '.(($uri['w'] || $uri['h'])?' resampled:'.$uri['w'].'x'.$uri['h']:''));
		exit;
	}

	/**
	 * Other files
	 */
	@unlink(cfAppDataDir().'/thumbnail.tmp.jpg');
	// Generate resized image with createJPG
	if(!cfGGetVar('disableCreateJPG') && cfStreamProc('"'.cfAppBinDir().'/createJPG.exe" "'.$completeFilename.'" "'.cfAppDataDir().'/thumbnail.tmp.jpg" '.(($uri['w'])?$uri['w']:'160').' '.(($uri['h'])?$uri['h']:'120').' all no 100 '.$JPGQuality,array('stdout'=>'return'))!='NOK'
		&& file_exists(cfAppDataDir().'/thumbnail.tmp.jpg')){
		if(!$debugRemoveContentHeader) header('Content-Type: image/jpeg');
		echo file_get_contents(cfAppDataDir().'/thumbnail.tmp.jpg');
		@unlink(cfAppDataDir().'/thumbnail.tmp.jpg');
	}
	else{

		if(!$debugRemoveContentHeader) header('Content-Type: image/jpeg');
		imagejpeg(imagecreatetruecolor((($uri['w'])?$uri['w']:'160'),(($uri['h'])?$uri['h']:'120')));
	}
}

/**
 * @desc Check published image access rights and send to output
 *
 * @param array $uri: parsed URI
 */
function imagePublished($uri){
	$id=$uri['dlToken'];

	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$tokenList=efTokensRead();
	if(!isset($tokenList[$id]) || $tokenList[$id]['type']!=='publishImage') logError('invalid published video id ('.$id.')');
	$token=$tokenList[$id];

	// Check file's existence
	if(!is_file($token['filename'])) logError('perempted published image ('.$token['filename'].')');

	// Increase number of downloads and save
	$tokenList[$id]['limitNb']--;
	efTokensWrite($tokenList);

	// Full-size image
	if(isset($uri['w']) && isset($uri['h']) && $uri['w']==0 && $uri['h']==0 && @$token['fullsizeAllowed']) imageGenerate($token['filename'],array('w'=>0,'h'=>0));

	// Resized image
	imageGenerate($token['filename'],array('w'=>$token['w'],'h'=>$token['h']));
}

/**
 * @desc Send a resized version of an image located into /gfx dir or subdir
 *
 * @param array $uri: parsed URI
 * 			see cfFilteredImage for parameters
 */
function imageResizedGFX($uri){
	// Check if browser is requesting a cached image, and if so, just return not-modified-file header
	checkCache();

	$completeFilename=$uri['file'];
	$completeFilename=cfAppDocRoot().str_replace('..','',$completeFilename);
	$ext=cfFileExtension($completeFilename);
	if(!is_file($completeFilename)) logError('extImage: NX '.$completeFilename.' doesnt exist');

	$outputFormat=((@$uri['format']==='png' || ($ext=='png' && @$uri['format']!=='gif'))?'png':'gif');

	header('Last-Modified: '.gmdate("D, d M Y H:i:s",@filemtime($completeFilename)).' GMT');
	header('Etag: "'.md5($_SERVER['REQUEST_URI']).'"');

	if($ext=='png' || $ext=='gif'){
		WHeaders::contentType($outputFormat);

		if($ext=='png') $sourceImage=@imagecreatefrompng($completeFilename);
		else $sourceImage=@imagecreatefromgif($completeFilename);

		if(!$sourceImage) return false;
		$sourceWidth=imagesx($sourceImage); $sourceHeight=imagesy($sourceImage);

		// Set default dimensions if not passed
		if(!$uri['w']) $uri['w']=$sourceWidth;
		if(!$uri['h']) $uri['h']=$sourceHeight;

		$destImage = imagecreatetruecolor($uri['w'], $uri['h']);

		if(!@$uri['noAlpha']){
			imagecolortransparent($destImage, imagecolorallocatealpha($destImage, 0, 0, 0, 127));
			imagealphablending($destImage, false);
			imagesavealpha($destImage, true);
		}

		imagecopyresampled($destImage, $sourceImage, 0, 0, 0, 0, $uri['w'], $uri['h'], $sourceWidth, $sourceHeight);


		/**
		 * Image processing
		 */

		// Alpha blending
		if(isset($uri['alpha']) || isset($uri['desat']) || isset($uri['cr'])){

			if(!isset($uri['alpha'])) $uri['alpha']=255;
			$desat=min(max(@$uri['desat']/100,0),1);
			if(isset($uri['cr'])){
				$cr=$uri['cr'];
				$cg=$uri['cg'];
				$cb=$uri['cb'];
				$cp=$uri['cp']/100;
			}

			for($x=0;$x<$uri['w'];$x++){
				for($y=0;$y<$uri['h'];$y++){
					$c = imagecolorsforindex($destImage, imagecolorat($destImage, $x, $y));
					$r=$c['red'];
					$g=$c['green'];
					$b=$c['blue'];
					$a=$c['alpha'];
					$a=127-($uri['alpha']/255)*(127-$a);
					// desaturation
					if($desat){
						$av=($r+$g+$b)/3;
						$r=$r+$desat*($av-$r);
						$g=$g+$desat*($av-$g);
						$b=$b+$desat*($av-$b);
					}
					// coulouring
					if(isset($cp)){
						$r=$cp*$cr+(1-$cp)*$r;
						$g=$cp*$cg+(1-$cp)*$g;
						$b=$cp*$cb+(1-$cp)*$b;
					}
					$col=imagecolorallocatealpha($destImage,$r,$g,$b,$a);
					imagesetpixel($destImage, $x, $y, $col);
				}
			}
			if($outputFormat=='png') imagepng($destImage); else imagegif($destImage);
			exit;
		}

        imagedestroy($sourceImage);
		if($outputFormat=='png') imagepng($destImage); imagegif($destImage);
	}
	else{
		WHeaders::contentType('jpeg');
		cfCreateResizedJPG($completeFilename, 0, 0, '', $uri['w'], $uri['h'], 2, 75, 0, false, true);
	}
	exit;
}

/**
 * @desc Generate an image with error message, and log error
 *
 * @param string $logCaption : error message
 */
function logError($logCaption, $type='default'){
	global $uri;
	global $debugRemoveContentHeader;


	cfLog($logCaption, LOG_DEBUG);
	cfDebugUpdateBuffer('ERROR: '.$logCaption);

	$winDir=isset($_SERVER['SystemRoot'])?str_replace('\\','/',$_SERVER['SystemRoot']):'c:/windows';

	// Send headers with no-cache
	if(!$debugRemoveContentHeader) WHeaders::noCache();
	if(!$debugRemoveContentHeader) WHeaders::contentType("Content-type: image/png");
	$capt=str_replace('extImage: ','',$logCaption);
	$capt=substr($capt,0,floor(strlen($capt)/3))."\n".substr($capt,floor(strlen($capt)/3),floor(strlen($capt)/3))."\n".substr($capt,-floor(strlen($capt)/3));

	// Generate black image with a ? watermark
	$w=(isset($uri['w']) && $uri['w'])?$uri['w']:(int)(5*strlen($capt)/3);
	$h=(isset($uri['h']) && $uri['h'])?$uri['h']:(int)(5*strlen($capt)/3);
	$im = imagecreatetruecolor($w, $h);
	imagefilledrectangle($im, 0, 0, $w, $h, imagecolorallocate($im, 0, 0, 0));
	$tw=floor(3*min($w,$h)/4);
	$th=$tw; $tw*=0.75;
	$se = imagecreatetruecolor($tw,$th);
	// Display a "?" (first test imagettftext function as gd might have been compiled without ttf support)
	if(function_exists('imagettftext')) {
		if($type=='forbidden') imagettftext($se, $th, 0, 0, $th-$th/8, imagecolorallocate($im, 45, 45, 45), $winDir.'/fonts/arial.ttf', 'x');
		else imagettftext($se, $th, 0, $tw*0/16, $th, imagecolorallocate($im, 45, 45, 45), $winDir.'/fonts/arial.ttf', '?');
	}
	imagecopy($im,$se,floor(($w-$tw)/2),floor(($h-$th)/2),0,0,$tw,$th);

	imagepng($im);
	imagedestroy($im);

	exit;
}

/**
 * @desc return not-modified-file header
 *
 */
function return304(){
	header('HTTP/1.x 304 Not Modified');
	header('X-Powered-By:');
	header("Date: " . gmdate("D, d M Y H:i:s") . " GMT");
	header('Etag: "'.md5($_SERVER['REQUEST_URI']).'"');
	header('Content-Length');
	header('Content-Type');
	exit;
}

/**
 * @desc Check if browser is requesting a cached image, and if so, just return not-modified-file header
 *
 * @param string $completeFilename: filename including path
 */
function checkCache($completeFilename=false){
	global $uri;
	if(isset($uri['cache']) && $uri['cache']=='0') return false;

	$headers=getallheaders();
	if(!isset($headers['If-Modified-Since']) && !isset($headers['If-None-Match'])) return false;
	if(isset($headers['If-None-Match']) && $headers['If-None-Match']!='"'.md5($_SERVER['REQUEST_URI']).'"') return false;
	if(isset($headers['If-Modified-Since']) && $completeFilename){
		if(!($date=strtotime($headers['If-Modified-Since']))) return304();
		if($date<@filemtime($completeFilename)) return false;
	}
	return304();
}

/**
 * @desc Extract and send audio file's cover to output
 *
 * @param string $completeFilename
 * @param integer $width: ouput width
 * @param integer $height: ouput height
 * @param bool $directoryCover: false not to use directory cover if no embeded cover found
 */
function imageExtractAudioCover(&$completeFilename,$width=0,$height=0,$directoryCover=true){
	global $debugRemoveContentHeader;

	$ext=cfFileExtension($completeFilename);

	/**
	 * MP3 ID3 image
	 */
	if($ext=='mp3'){
		require(INCLUDE_DIR.'miscFunctions.php');

		$cover=new id3V2Ext($completeFilename);
		// ID3 embeded image: generate and send to output
		if($cover->hasImage()) $cover->generate($width,$height);
	}

	/**
	 * WMA embedded image
	 */
	elseif($ext=='wma' || $ext=='wmv' || $ext=='asf'){
		// Get embedded image
		require(INCLUDE_DIR.'php-reader/ASF.php');
		$asf=@new ASF($completeFilename);
		if(isset($asf->header->extendedContentDescription->descriptors['WM/Picture'])){
			$v=$asf->header->extendedContentDescription->descriptors['WM/Picture'];

			// Dirty parsing of WM/Picture data
			$pos=0;
			while($v[$pos]!='i' && $pos<50 && $pos<strlen($v))	$pos++;
			if($v[$pos]=='i'){
				$epos=$pos;
				while(substr($v,$pos,2)!="\0\0" && $pos<50)	$pos++;
				$mimeType=str_replace("\0",'',substr($v,$epos,$pos-$epos));
				while($v[$pos]=="\0" && $pos<strlen($v)) $pos++;
				// Resized image
				if($width && $height){
					if($in=@imagecreatefromstring(substr($v,$pos))){
						if(!$debugRemoveContentHeader) WHeaders::contentType('jpg');
						$out=imagecreatetruecolor($width,$height);
						fastimagecopyresampled($out,$in,0,0,0,0,$width,$height,imagesx($in),imagesy($in),3);
						imagejpeg($out,null,70);
						imagedestroy($in);
						imagedestroy($out);
						exit;
					}
				}
				// Original image
				else{
					if(!$debugRemoveContentHeader) header('Content-Type: '.$mimeType);
					echo substr($v,$pos);
					exit;
				}
			}
		}
	}

	// ISO14496 format
	elseif($ext=='3gp'||$ext=='3gpp'||$ext=='avc'||$ext=='dcf'||$ext=='m21'||$ext=='m4a'||$ext=='m4b'||$ext=='m4p'||$ext=='m4v'||$ext=='maf'||$ext=='mj2'||$ext=='mjp'||$ext=='mov'||$ext=='mp4'||$ext=='odf'||$ext=='sdv'||$ext=='qt'||$ext=='aac'){
		require_once(INCLUDE_DIR.'/php-reader/ISO14496.php');
		$isom=new ISO14496($completeFilename,array("base" => "moov.udta.meta.ilst"));

		if (isset($isom->moov->udta->meta->ilst)) {
			$ilst = $isom->moov->udta->meta->ilst;
			if (isset($ilst->covr)) {
				if($width && $height){
					if(!$debugRemoveContentHeader) header('Content-type: image/jpeg');
					if(($pos=strpos($ilst->covr,''))===false) // JPEG
						$pos=strpos($ilst->covr,'PNG'); // PNG
					if($pos!==false && ($in=imagecreatefromstring(substr($ilst->covr,$pos)))){
						$out=imagecreatetruecolor($width,$height);
						fastimagecopyresampled($out,$in,0,0,0,0,$width,$height,imagesx($in),imagesy($in),3);
						imagejpeg($out,null,70);
						imagedestroy($in);
						imagedestroy($out);
						exit;
					}
				}
				else{
					if ($ilst->covr->data->hasFlag(ISO14496_Box_DATA::JPEG)) {if(!$debugRemoveContentHeader) header("Content-Type: image/jpeg");}
					elseif ($ilst->covr->data->hasFlag(ISO14496_Box_DATA::PNG)){if(!$debugRemoveContentHeader) header("Content-Type: image/png");}
					echo $ilst->covr->data->value;
				}
			}
		}
	}
	// Else, look for best image in folder
	if ($directoryCover){
		require_once(INCLUDE_DIR.'explorerFunctions.php');
		$completeFilename=efGetCoverFromFolder($completeFilename, $score);
	}
	else {
		if(!$debugRemoveContentHeader) header('Content-type: image/jpeg');
		cfCreateResizedJPG(cfAppDocRoot().'/gfx/cd.png',80,80,'',$width,$height);
		exit;
	}
}

// Decode and parse URI
$uri=cfParseExtURI();

// DEBUG
if(cfMGetVar('debugAsync'))	cfDebugSetBuffer('image',$uri,'...');

// Published image
if(isset($uri['dlToken'])) imagePublished($uri);

// Simple resized gfx
if(isset($uri['file']) && (substr($uri['file'],0,5)=='/gfx/' || substr($uri['file'],0,5)=='/res/') && isset($uri['w']) && isset($uri['h'])) imageResizedGFX($uri);


/**
 * Regular redirected images below this point
 */
wSession_start_nowrite();
header('P3P: policyref="/w3c/p3p.xml",CP="NON COR CURa OUR NOR NAV"');

// Verify that user logged in
if (!isset($_SESSION['userLogged'])) logError('extImage: UL '.$_SERVER['REQUEST_URI'].' unlogged access attempt');

// Set active resource
if(isset($uri['resId'])&& is_numeric($uri['resId']) && isset($_SESSION['res'][$uri['resId']]) && is_array($_SESSION['res'][$uri['resId']])) $_SESSION['activeResourceId']=$uri['resId'];

// Check that a file is passed
if(!isset($uri['file'])) logError('extImage: NI No passed image');
$completeFilename=$uri['file'];


// Shared list
if(cfSharedMode('list') && strpos($completeFilename,'*resourceDataDirBasePath*')===false) $completeFilename=cfSharedCompleteFilename($completeFilename);

// Shared folder
else{
	// Don't debug-log images from resource data dir
	if(strpos($completeFilename,'*resourceDataDirBasePath*')!==false) {
		$_ENV['cfStreamFileNoLog']=1;
		$completeFilename=str_replace('*resourceDataDirBasePath*',cfAppResourceDir(),$completeFilename);
	}
	else $completeFilename=str_replace('*resourceBasePath*',cfRGetVar('path'),$completeFilename);
}

// Check if browser is requesting a cached image, and if so, just return not-modified-file header
checkCache($completeFilename);

/**
 * Check access rights
 */

// Resource specific file source and rights function : bypass common file's existence and access rights
// and (may) replace given file name by real file name
if(cfRGetVar('extAccessFunction')) {
	$extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));

	if(!($completeFilename=$extAccessFunction($completeFilename))) logError('extImage: DA '.$_SERVER['REQUEST_URI'].' denied access');
}
// Administration, allow exe viewing
elseif (isset($uri['icon']) && cfUGetVar('administrator') && cfRGetVar('type')=='administration'){}
else{
	// If file doesn't exist but is located into resource data directory and is an image and thumbnails allowed for this resource, generate
	// (might happend if a problem occurs with createJPG)
	if(!file_exists($completeFilename) && cfFileExtension($completeFilename)=='jpg' &&
		cfIsSubDir(cfRGetVar('resourceDataDir'),$completeFilename) && cfRGetVar('resourceDataDirAccessAllowed') && cfRGetVar('path')){
			$tmw=cfRGetVar('thumbnailsSize');
			if(cfRGetVar('photoThumbnailsNeeded') && is_numeric($tmw)){
				//substr(basename($completeFilename),0,strlen($tmw))==$tmw
				$sourceCompleteFilename=cfJoinPathFile(cfRGetVar('path'),substr(dirname($completeFilename),strlen(cfRGetVar('resourceDataDir'))));
				$sourceCompleteFilename.='/'.substr(cfFileWithoutExtension(basename($completeFilename)),strlen($tmw));
				// Verify that source file access is allowed
				if(cfFileRights($sourceCompleteFilename,'download')){
					// Everything is OK, thumbnail can be generated
					cfCreateResizedJPG($sourceCompleteFilename,0,0,$completeFilename,$tmw,$tmw);
				}
			}
	}
	if(!cfFileRights($completeFilename,'download')) logError('extImage: DA '.$completeFilename.' denied access','forbidden');

	//Verifies file's existence
	if(!is_file($completeFilename)) logError('extImage: NX '.$completeFilename.' doesnt exist');
}



// Generate and sent image and headers to ouptut
imageGenerate($completeFilename,$uri)
?>