<?php
require_once(INCLUDE_DIR.'treeViewClass.php');
require_once(INCLUDE_DIR.'explorerFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

class TreeFileBrowser{
	private $tree;
	function __construct(){
		$this->tree=new treeViewFolder('fileBrowser');
		$this->tree->setLocalBrowser();
		$this->tree->showFiles();
		$this->tree->setSortBy('name');
		$this->tree->setSortOrder('asc');
		if(cfIsAsync()) $this->tree=treeFolderLoad('fileBrowser',true);

		if(!cfIsAsync()) $this->tree->setCurrentDirectory('computerRoot');

		$this->tree->processCommands();
	}

	function show(){
		require_once(INCLUDE_DIR.'outputFunctions.php');

		cfInsertHEAD(false);
		echo cfScriptLink('winClient.js');
		echo '</head>';
		?>
<body>
<?php
outInsertStandardComForm();
?>
<script type="text/javascript">
var treeViews={};
</script>
<div class="frame1" id="fileBrowserFrame">
<?php
	echo $this->tree->HTML();
?>
</div>
</body>
<?php
	}
}


/**
 * @desc No-treeView local file browser (no file right check), with directory, file or files and directories selection
 *
 */
class FileBrowser{
	private $baseDirectory;
	private $currentDirectory;
	private $extensionFilter=false;
	private $fileTypeFilter=false;
	private $sortBy='name';				// Sort item
	private $selectType='dir';			// Selection type: "dir" for directories, "file" for single file or "multiple" for multiple files and directories

	function css(){
?>
<style type="text/css">
body{overflow:hidden}
#fileBrowserControls{position:absolute;right:0.4em;bottom:0.4em}
</style>
<?php
	}

	/**
	 * Constructor
	 *
	 * @param string $baseDirectory
	 */
	function __construct($baseDirectory=false){
		if(!$baseDirectory){
			 $this->currentDirectory=$baseDirectory='computerRoot';
			if(cfGGetVar('localFilebrowserDirectory')) $this->currentDirectory=cfGGetVar('localFilebrowserDirectory');
		}
		else{
			$baseDirectory=cfUTF8Decode($baseDirectory,true,true,false);
			if(!is_dir($baseDirectory)) $baseDirectory=(WEnv::host()->isWindows())?'computerRoot':cfGGetVar('specialFolderHome');
			$this->currentDirectory=$baseDirectory;
		}

		// Set sharedMode to "folder" to prevent issues with ef functions
		cfRSetVar('sharedMode','folder');
	}

	/**
	 * @desc Set directory
	 *
	 * @param string $dir
	 * @return boolean result
	 */
	function setDirectory($dir){
		if($dir==='computerRoot' || is_dir($dir)) $this->currentDirectory=$dir; else return false;
		cfGSetVar('localFilebrowserDirectory',$dir);
		return true;
	}

	/**
	 * @desc Set selection type: dir, file, multiple
	 *
	 * @param string $selectType: selection type: "dir" for directories, "file" for single file or "multiple" for multiple files and directories
	 */
	function setSelectType($selectType){
		if(in_array($selectType,array('dir','file','multiple'))) $this->selectType=$selectType; else throw new Exception('Incorrect select type, must be "dir","file" or "multiple"',1);
	}

	/**
	 * @desc Return select type
	 *
	 */
	function getSelectType(){
		return $this->selectType;
	}

	/**
	 * Get files/dirs in selected directory
	 *
	 * @return string: JS representation of files
	 */
	private function getFiles(){
		if(!$this->currentDirectory) $this->currentDirectory=$this->baseDirectory;

		$options=array('noRightsCheck'=>1);

		$_ENV['weezoResEfFiles']=efGetDirContent($this->currentDirectory,true,$this->extensionFilter,$this->fileTypeFilter,$this->sortBy,$options);

		$type=(($this->selectType==='dir')?'dirs':'all');

		return efFilesJSArray($type);
	}

	/**
	 * @Desc Send HTML code to display browser or async update code
	 *
	 * @return void
	 */
	function show(){

		if(cfIsAsync()) return $this->processCommands();

		$files=$this->getFiles();

		$size='400x400';

		cfInsertHEAD(false,array(
			'size'=>$size,
			'sizable'=>1
			));

		echo cfScriptLink('winClient.js');
		echo cfScriptLink('explorer.js');
		echo cfScriptLink('localFileBrowser.js');
		$this->css();
		echo '</head>';
		?>
<body>
<?php
outInsertStandardComForm();

?>
<div class="frame1" id="fileBrowserOuterFrame">
	<input id="fileBrowserLocation" class="textInput" value="<?php echo cfUTF8Encode($this->currentDirectory);?>">
	<div id="fileBrowserFrame" class="frame2" style="overflow:auto"></div>

	<div id="fileBrowserControls">
<?php
echo outBt(cfCaption('genValidate'),outIcon('ok16'),'javascript:LocalFileBrowser.validate()','fileBrowserValidateBt');
echo '&nbsp;&nbsp;&nbsp;';
echo outBt(cfCaption('genCancel'),outIcon('cancel'),'javascript:LocalFileBrowser.cancel()');
?>
	</div>
</div>
<script type="text/javascript">
<?php
efFilesListJSVars();
echo cfJSCaptionVar('explorerComputerRoot');
?>

function winResized(){
	alignBottom($('fileBrowserOuterFrame'),winMe,0);
	alignBottom($('fileBrowserFrame'),winMe,30);

	D.onselectstart=function(event){var s;return (W.event && (s=W.event.srcElement) && (s.tagName=="INPUT"||s.tagName=="TEXTAREA"||$(s).e.parentHasAttribute("uiSelectable")));}
}
setInterval(winResized,10);

LocalFileBrowser.selectType="<?php echo $this->getSelectType(); ?>";
LocalFileBrowser.hostOS="<?php echo WEnv::host()->osFamily()?>";
LocalFileBrowser.update(<?php echo $files.',"'.cfUTF8Encode($this->currentDirectory).'"'; ?>);
</script>
</body>
<?php
	}

	/**
	 * @desc Process POST commands
	 *
	 */
	function processCommands(){
		if(!cfIsAsync()) return ;
		cfAsyncHeader();

		$action=@$_POST['data3'];

		/**
		 * Navigate to a directory or check a directory exists
		 */
		if($action=='go' || $action=='checkDirectory'){
			list($completeFilenameNV,$passedDir,$passedFile)=efGetPassedFileComponents(false);
			list($completeFilename,$passedDir,$passedFile)=efGetPassedFileComponents(true);

			if($passedFile=='..' && WEnv::host()->isWindows()){
				if($passedDir=='//' || substr($passedDir,1,1)==':' && strlen($passedDir)<4) {
					$completeFilename=$passedDir='computerRoot';
					$passedFile='.';
				}
			}

			// Auto-completition
			if(!is_dir($completeFilenameNV) && $completeFilenameNV!=='computerRoot' && $action=='checkDirectory'){
				if(is_dir(dirname($completeFilenameNV))){
					$found=0;
					foreach (cfGlob(dirname($completeFilenameNV).'/*') as $cfn) {
						if(is_dir($cfn) && cfCmpLeft(basename($cfn),basename($completeFilenameNV))){
							$foundCompleteFilename=$cfn;
							$found++;
						}
					}
					// If a single matching file / directory has been found, and this directory is not the current one change
					if($found==1) {
						if($foundCompleteFilename!=$this->currentDirectory) $completeFilename=$foundCompleteFilename;
					}
				}
			}

			// If passed path is ok
			if(is_dir($completeFilename) || $completeFilename=='computerRoot'){
				// Navigate
				$this->setDirectory($completeFilename);
				$dir=$this->currentDirectory;
				if(WEnv::host()->isWindows() && strlen($dir)==2 && substr($dir,1,1)) $dir.='/';
				echo cfAsyncXMLJSaction('LocalFileBrowser.update('.$this->getFiles().',"'.cfUTF8Encode($dir).'")');
			}
			// Just check if entered directory is valid
			elseif($action=='checkDirectory'){
				echo cfAsyncXMLJSaction('LocalFileBrowser.setValidDir(0)');
			}

		}
		die(cfAsyncFooter());
	}
}