<?php
/**
 * Database-based music explorer functions
 * used by /res/explorer/music2 and /res/explorer/music3
 *
 * Display an audio files explorer based on
 * 	- iTunes music database or
 * 	- winamp music database or
 * 	- folders music database
 *
 * Audio files list
 * Playlist functions
 * Audio player (streaming)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.5
 */

// SQlite database name (located into resource data directory)
define('W_AUDIO_DATABASE_NAME','music4.db');
define('W_AUDIO_TMP_DATABASE_NAME','music4.db.tmp');
define('W_AUDIO_PLAYLIST_DATABASE_NAME','music_playlists.db');

// Winamp database columns identifiers
define('FIELD_UNDEFINED',255);
define('FIELD_COLUMN',0);
define('FIELD_INDEX',1);
define('FIELD_REDIRECTOR',2);
define('FIELD_STRING',3);
define('FIELD_INTEGER',4);
define('FIELD_BOOLEAN',5);
define('FIELD_BINARY',6);
define('FIELD_GUID',7);
define('FIELD_FLOAT',9);
define('FIELD_DATETIME',10);
define('FIELD_LENGTH',11);
define('FIELD_STRINGUTF8',12);
$_ENV['supportedPlaylists']=array('m3u','pls');

require_once(INCLUDE_DIR.'databaseFunctions.php');

/**
 * Return an "explorerFunctions" formated array with audio meta data from a database formated track
 * @param array $track: database track item
 * @return array
 * */
function mDBPlaylistItemFromTrack($track){
	return array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track),true,true,false), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
}


/**
 * @desc Winamp music database: convert an "encoded" audio or cover filename into real filename
 * 		audio encoded format: *resourceBasePath* /1234.mp3 where 1234 is track id in database
 * 		cover encoded format: *resSpecific* /(embeded|folder)1234.mp3 where 1234 is track id in database
 * @param string $encodedFilename: encoded audio / cover filename
 * @param bool $resampledAudioFile:
 * @return string: real complete filename or false if not found / not allowed
 */
function mDBWinampExtAccessFunction($encodedFilename,&$resampledAudioFile){
	if(substr($encodedFilename,0,14)!="*resSpecific*/" && substr($encodedFilename,0,19)!="*resourceBasePath*/") return false;
	$cfn=substr($encodedFilename,strpos($encodedFilename,"*/")+2);

	// Cover
	// Embeded in file tags: function should return actual filename
	if(substr($cfn,0,7)=='embeded') $cfn=substr($cfn,7);
	// Folder image: function should find and return best suitable image located in audio file's folder
	elseif(substr($cfn,0,6)=='folder') {$cfn=substr($cfn,6);$folderImage=1;}

	if(!is_numeric($cfn=cfFileWithoutExtension($cfn))) return false;

	// Search for track in database
	if(!($db=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_DATABASE_NAME))) return false;
	$cfn=dbSingleQuery('SELECT Location FROM tracks WHERE id="'.$cfn.'"',$db);
	dbClose($db);

	// If not found, exit
	if(!$cfn) return false;
	$cfn=cfUTF8Decode($cfn,true,false,false);

	// If cover is requested and il must be searched for in file's directory
	if(isset($folderImage))	{
		require_once(INCLUDE_DIR.'explorerFunctions.php');
		$coverCFN=efGetCoverFromFolder($cfn,$score);
		// If no image found, return transparent (lightweight) image
		return ($score>=0)?$coverCFN:cfAppDocRoot().'/gfx/v.gif';
	}
	if(cfFileExtension($cfn)!="mp3") $resampledAudioFile=true;
	return $cfn;
}

/**
 * @desc Folder music database: convert an "encoded" audio or cover filename into real filename
 * 		audio encoded format: *resourceBasePath* /1234.mp3 where 1234 is track id in database
 * 		cover encoded format: actual filename
 * @param string $encodedFilename: encoded audio / cover filename
 * @param bool $resampledAudioFile:
 * @return string: real complete filename or false if not found / not allowed
 */
function mDBFolderExtAccessFunction($encodedFilename,&$resampledAudioFile){
	global $resampledAudioFile;

	// If called by /ext/image.php, verify that format is:
	// (resourceBasePath)/(folder|embeded)XXXX.mp3
	if(basename($_SERVER["SCRIPT_NAME"])=='image.php'){
		$encodedFilename=str_replace('\\','/',$encodedFilename);
		$encodedFilename=cfFileWithoutExtension(substr($encodedFilename,strrpos($encodedFilename,'/')+1));

		// Retrieve image from folder
		if(substr($encodedFilename,0,6)=='folder') {$folderImage=1;$encodedFilename=substr($encodedFilename,6);}
		// Retrieve image from file
		elseif(substr($encodedFilename,0,7)=='embeded') $encodedFilename=substr($encodedFilename,7);
		else return false;
		if(!is_numeric($encodedFilename)) return false;

		// Search for track in database
		if(!($db=dbOpenDb(cfAppResourceDir()."/".W_AUDIO_DATABASE_NAME))) return false;
		$cfn=dbSingleQuery('SELECT Location FROM tracks WHERE id="'.$encodedFilename.'"',$db);
		dbClose($db);

		// If not found, exit
		if(!$cfn) return false;
		$cfn=cfUTF8Decode($cfn,true,false,false);

		// If cover is requested and il must be searched for in file's directory
		if(isset($folderImage))	{
			require_once(INCLUDE_DIR.'explorerFunctions.php');
			$coverCFN=efGetCoverFromFolder($cfn,$score);
			// If no image found, return transparent (lightweight) image
			return ($score>=0)?$coverCFN:cfAppDocRoot().'/gfx/v.gif';
		}
		return $cfn;
	}

	// If called by /ext/audio.php
	if(substr($encodedFilename,0,14)!='*resSpecific*/' || !is_numeric($cfn=cfFileWithoutExtension(substr($encodedFilename,14)))) return false;
	if(!($db=dbOpenDb(cfAppResourceDir()."/".W_AUDIO_DATABASE_NAME))) return false;
	$cfn=dbSingleQuery('SELECT Location FROM tracks WHERE id="'.$cfn.'"',$db);
	dbClose($db);
	if(cfFileExtension($cfn)!="mp3") $resampledAudioFile=true;
	return cfUTF8Decode($cfn,true,false,false);
}

/**
 * @desc Folder music database: convert an "encoded" audio or cover filename into real filename
 * 		audio encoded format: *resourceBasePath* /1234.mp3 where 1234 is track id in database
 * 		cover encoded format: actual filename
 * @param string $encodedFilename: encoded audio / cover filename
 * @param bool $resampledAudioFile:
 * @return string: real complete filename or false if not found / not allowed
 */
function mDBiTunesExtAccessFunction($encodedFilename,&$resampledAudioFile){
	global $resampledAudioFile;
	// If called by /ext/image.php, verify that format is:
	// (resourceBasePath)/(folder|embeded)XXXX.mp3
	if(basename($_SERVER["SCRIPT_NAME"])=='image.php'){
		if(cfFileRights($encodedFilename.'.itc2','download')) return $encodedFilename.'.itc2';
		if(cfFileRights($encodedFilename.'.itc','download')) return $encodedFilename.'.itc';
		return false;
	}

	// If called by /ext/audio.php
	if(substr($encodedFilename,0,14)!='*resSpecific*/' || !is_numeric($cfn=cfFileWithoutExtension(substr($encodedFilename,14)))) return false;
	if(!($db=dbOpenDb(cfAppResourceDir()."/".W_AUDIO_DATABASE_NAME))) return false;
	$cfn=dbSingleQuery('SELECT Location FROM tracks WHERE id="'.$cfn.'"',$db);
	dbClose($db);
	if(cfFileExtension($cfn)!="mp3") $resampledAudioFile=true;
	return cfUTF8Decode(rawurldecode(str_replace("''","'",$cfn)),true,true,false);
}


/**
 * @desc initialize data
 *
 */
function initResourceData(){
	// Define resource-vars that will be saved if user is singleUser
	cfSetSavedRVar('displayType','groupBy','itemsPerPage','filterTd','displayMultipleDownload');

	// If resource already initialized, return
	if(cfRGetVar('resourceParametersInitialized')) return;

	global $itemsPerPageList;
	cfRSetVar('fileTypeFilter',array('audio'));// array of file types of displayed files. Set to false if no filter on file type (see mime_type.php for list of file types)
	cfRSetVar('extensionFilter',array('itc'=>1,'itc2'=>1)); // Also allow itunes covers files
	cfRSetVar('playlistDisplayed',true); // Show main playlist
	cfRSetVar('needAudioInfo',true); // Indicate efGetDirContent to get id3 info
	cfRInitVar('sortField','no'); // Files grouping
	cfRSetVarArray('playlist'); // Reset main playlist


	// extAccessFunction used to check file rights in /ext/audio.php
	if(SOURCE_TYPE=='iTunes') cfRSetVar('extAccessFunction','global $resampledAudioFile; require_once(INCLUDE_DIR."musicDBFunctions.php"); return mDBiTunesExtAccessFunction($completeFilename,$resampledAudioFile);');
	if(SOURCE_TYPE=='folder') cfRSetVar('extAccessFunction','global $resampledAudioFile; require_once(INCLUDE_DIR."musicDBFunctions.php"); return mDBFolderExtAccessFunction($completeFilename,$resampledAudioFile);');
	if(SOURCE_TYPE=='winamp') cfRSetVar('extAccessFunction','global $resampledAudioFile; require_once(INCLUDE_DIR."musicDBFunctions.php"); return mDBWinampExtAccessFunction($completeFilename,$resampledAudioFile);');



	cfRInitVar('plAddAllShuffleSize',50); // Set "random play all libray" playlist size

	if(cfRGetVar('defaultOutputBitRate')) cfRSetVar('outputBitRate',cfRGetVar('defaultOutputBitRate'));
	else cfRSetVar('outputBitRate',cfRGetVar('maxOutputBitRate'));

	if(cfRGetVar('defaultAudioPlayer')) cfRSetVar('audioPlayer',cfRGetVar('defaultAudioPlayer')); else cfRSetVar('audioPlayer','embeded');
	cfRInitVar('efSortBy','name'); cfRInitVar('efSortOrder','asc');// sort files by ascending alphabetical order
	cfRSetVar('useHTTPXmlRequest',true); // Use HTTPXmlRequest for communication between browser and server
	if(cfRGetVar('resourceConfigured')) cfRSetVar('resourceParametersInitialized',true); // indicate that parameters have been initialized
	else $errorMessageCritical=cfCaption('configErrorNotConfigured');

	cfRInitVar('displayType','displayCovers');
	cfRInitVar('groupBy','Artist',array('Artist','Album','Playlist'));

	cfRInitVar('itemsPerPage',$itemsPerPageList[2]);
	if(!in_array(cfRGetVar('itemsPerPage'),$itemsPerPageList)) cfRSetVar('itemsPerPage',$itemsPerPageList[2]);
	cfRSetVar('displayOffset',0); // current offset from start of DB

	cfRSetVar('defaultAudioWindowHeight',DEFAULT_AUDIO_WINDOW_HEIGHT);
	cfRSetVar('defaultPlaylistWindowHeight',DEFAULT_PLAYLIST_WINDOW_HEIGHT);

	// Cover sizes
	cfRInitVar('fullviewCoverWidth',FULLVIEW_COVER_WIDTH);
	cfRInitVar('fullviewCoverExtWidth',FULLVIEW_COVEREXT_WIDTH);

	cfRSetVar('subFoldersIncluded',true); // Allow access to subfolers of artwork base folder
	cfRSetVar('efCurrentDirectory','*resSpecific*'); // Set current directory to *resSpecific* (for inclusion in comForm)
	cfRSetVar('displayMultipleDownload',true); // unfold multiple download / playlist box

	// Browser specific parameters
	if(!cfBGetVar('download'))	{
		cfRSetVar('multipleDownloadAllowed',false);
		cfRSetVar('downloadShortcut',false);
		cfRSetVar('uploadAllowed',false);
	}
	if(!cfBGetVar('drag') || cfIsWII()) cfRSetVar('maxDragItems',0);

	// Adjust audio player to browser capabilities
	if(!cfBGetVar(cfRGetVar('audioPlayer'))) cfRSetVar('audioPlayer',((cfBGetVar('flash'))?'flash':((cfBGetVar('wmp'))?'wmp':((cfBGetVar('qt'))?'qt':'qtReenc'))));

	// iPhone specific
	if(cfBGetVar('name')=='iPhone') {
		cfRSetVar('meta','<meta name="viewport" content="width=550, user-scalable=no,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">');
		cfRSetVar('onload','updateOrientation();scrollTo(0,1)');
	}
	if(cfBGetVar('noScrollDiv')) {
		cfRSetVar('scrollMainFileTable',false);
		cfRSetVar('mainFileTableToggleAllowed',false);
	}

	// Inline player
	if(!cfTGetVar('frames')) cfRSetVar('inlinePlayer',true);// Set inline player for frameless themes
	if(cfBGetVar('standaloneMediaPlayer')) cfRSetVar('inlinePlayer',false);// Unset inline player for devices that doesn't support it

	// Filter panel displayed (default)
	cfRInitVar('filterTd',1);
	cfRSetVar('parametersLoaded',true);

	// Set user-level defined rights
	cfRInitUserRightsVar('downloadShortcut');
	cfRInitUserRightsVar('commentsAllowed');
	cfRInitUserRightsVar('multipleDownloadAllowed');
	cfRInitUserRightsVar('storePlaylistAllowed');
}

/**
 * @desc check if iTunes database has been modified and, if so parse it into :
 * 		- library table (general library parameters)
 * 		- tracks table (tracks data)
 * 		- playlists table (playlists id and name)
 * 		- playlistsTracks table (playlist id <-> track id)
 * @param boolean $forceUpdate : set to true to force database update
 * @return true if success
 */
function updateDBITunes($forceUpdate=false){
	global $db;
	$checkFilesExist=cfRGetVar('checkFilesExist');

	//$myMusicFolder=customQueryRegistry("HKEY_CURRENT_USER", "software\\Apple Computer, Inc.\\iTunes\\", "Win2KMyMusicFolder");

	$libraryFolder=cfRGetVar('musicITunesFolder');
	if(substr($libraryFolder,-3)=='xml') $libraryFolder=dirname($libraryFolder);
	$libraryFile=$libraryFolder.'/iTunes Music Library.xml';

	$prefixLength=(WEnv::host()->isWindows())?17:16; // Size of prefix removed from "Location" tag : file://localhost/path on osx, file://localhost/C:/path on windows // prod

	if(!file_exists($libraryFile)) outDisplayErrorPage(cfCaption('musicITunesFileNotFound','iTunes'));

	// Check library file hash to see if file has changed
	$libraryFileHash=md5_file($libraryFile);
	if(!$forceUpdate){
		if(cfRGetVar('databaseUpdateMethod')=='manual') return true;
		$lastLibraryFileHash=dbSingleQuery('SELECT value FROM library WHERE property="File Hash"',$db);
		if($lastLibraryFileHash==$libraryFileHash) return true;
	}

	set_time_limit(0);
	// Close session so other scripts aren't blocked
	wSession_write_close();

	// Display "refresh" page
	$msg='<script type="text/javascript">function uPB(nb,total){if(!total) {progressSetPerc("pBar",0);progressSetText("pBar","-");} else {progressSetPerc("pBar",100*nb/total);progressSetText("pBar",nb+" / "+total)}}</script><div id="action">'.cfCaption('musicRefreshingDatabase').'</div><br/><div id="progress" style="position:absolute;left:50%"><div style="position:relative;left:-150px">';
	$msg.='<div class="eTC">'.outProgressBar(0,'300px','-','pBar').'</div>';
	$msg.='</div></div>';

	echo outDisplayErrorPage($msg,array('icon'=>'clock','title'=>cfCaption('genWait')),false);
	flush();

	// Create temporary DB
	dbClose($db);
	openMusicDbTmp();

	// Parse iTunes XML file
	$xml=simplexml_load_file($libraryFile);
	$key=false;
	$properties=array();
	$artworks=array();
	dbExec('INSERT INTO library VALUES ("File Hash","'.$libraryFileHash.'")',$db);
	dbBeginTransaction($db);
	foreach ($xml->dict[0]->children() as $node){
		// Playlist main paramters
		if($node->getName()=='key') $key=(string)$node;
		else {
			if($key && $key!='Tracks' && $key!='Playlists') {
				$properties[$key]=(string)$node;
				dbExec('INSERT INTO library VALUES ("'.$key.'","'.$properties[$key].'")',$db);
				if($key=='Library Persistent ID'){
					// Search for covers (artworks)
					echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('musicRefreshingDatabase').'"</script>'."\n"; flush();
					// iTunes 8+
					if(is_dir($libraryFolder.'/Album Artwork/Cache/'.$properties['Library Persistent ID']))
						$artworks = getITunesArtworks($libraryFolder.'/Album Artwork/Cache/'.$properties['Library Persistent ID'],null,'itc2');
					// iTunes 7-
					elseif(is_dir($libraryFolder.'/Album Artwork/Local/'.$properties['Library Persistent ID']))
						$artworks = getITunesArtworks($libraryFolder.'/Album Artwork/Local/'.$properties['Library Persistent ID'],null,'itc');
					// Artwork not found
					else $artworks=array();
				}
				$key=false;
			}
			// Parse Tracks
			elseif($key=='Tracks') {
				$nb=0;

				$nbTracks=count($node->dict);
				echo '<script type="text/javascript">uPB(0,'.$nbTracks.');dgi("action").innerHTML="'.cfCaption('musicRefreshingDatabase').'"</script>'."\n"; flush();
				$lastRefresh=microtime(true);
				// Single track
				foreach ($node->dict as $trackNode){
					$key2=false;
					$track=emptyTrack();
					foreach ($trackNode->children() as $item){
						if($item->getName()=='key') $key2=(string)$item;
						elseif($key2) {
							$track[$key2]=dbEscapeString((string)$item,$db);
							$key2=false;
						}
					}
					// File location (urlEncode(utf8))
					if(isset($track['Location'])) $track['Location']=substr($track['Location'],$prefixLength); else $track['Location']='';

					// If track ID and persistent ID found, process track
					if(($track['Persistent ID']) && ($track['Track ID'])){
						// Track year
						if($track['Year'] && !is_numeric($track['Year'])) $track['Year']=false;
						// Date added
						$td=$track['Date Added'];
						$track['Date Added']=mktime(substr($td,11,2),substr($td,14,2),substr($td,17,2),substr($td,5,2),substr($td,8,2),substr($td,0,4));
						// Artwork
						if(isset($artworks[$track['Persistent ID']])) $track['Artwork']=substr($artworks[$track['Persistent ID']],1).'/'.$properties['Library Persistent ID'].'-'.$track['Persistent ID'];

						$track['Location']=rawurldecode(str_replace("''","'",$track['Location']));

						// Correct artist name for compilations
						if(isset($track['Album Artist'])) $track['Artist']=$track['Album Artist'];
						elseif(isset($track['Compilation'])) $track['Artist']=cfCaption('musicCompilation').' - '.$track['Album'];

						// Insert into DB

						if(!$checkFilesExist || file_exists(cfUTF8Decode($track['Location'],true,true,false))){
							dbExec("INSERT INTO tracks VALUES ('".$track['Track ID'] . "','".$track['Name'] . "','".
							$track['Artist'] . "','".$track['Album'] . "','".$track['Track Number'] . "','".$track['Kind'] . "','".
							floor($track['Total Time']/1000) . "','".$track['Date Added'] . "','".$track['Comments'] . "','".
							$track['Persistent ID'] . "','".$track['Location'] . "','".$track['Year'] . "','".$track['Artwork'] . "')",$db);
						}
						$nb++;
						if($nb%5000==0){
							dbCommitTransaction($db);
							dbBeginTransaction($db);
						}
						if(microtime(true)-$lastRefresh > 0.1){
							echo '<script type="text/javascript">uPB('.$nb.','.$nbTracks.');</script>'."\n"; flush();
							$lastRefresh=microtime(true);
						}
					}
				}
				dbCommitTransaction($db);
				dbBeginTransaction($db);
				echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();
			}
			// Parse playlists
			elseif($key=='Playlists') {
				$lastRefresh=microtime(true);
				$nbPlaylists=0;
				foreach ($node->children() as $playlistNode){
					$key2=false;
					$playlist=array('Name'=>false,'Playlist ID'=>false,'Playlist Type'=>false);
					foreach ($playlistNode->children() as $item){
						if($item->getName()=='key') {
							$key2=(string)$item;
							if($key2=='Music' || $key2=='Visible') $playlist['Playlist Type']=$key2;
						}
						elseif($key2 && $item->getName()!='array' && $item->getName()!='data') {
							$playlist[$key2]=dbEscapeString((string)$item,$db);
							$key2=false;
						}
						// Parse playlist tracks
						elseif($key2=='Playlist Items' && $item->getName()=='array' && !$playlist['Playlist Type'] && $playlist['Playlist ID'] && $playlist['Name']) {
							dbExec("INSERT INTO playlists VALUES ('".$playlist['Playlist ID']. "','".$playlist['Name'].",'s')",$db);
							foreach ($item->dict as $pltrack){
								dbExec("INSERT INTO playlistsTracks VALUES ('".$playlist['Playlist ID']. "','".(string)$pltrack->integer[0]."')",$db);
							}
							$key2=false;
							$nbPlaylists++;
						}
					}
				}
				echo '<script type="text/javascript">uPB('.$nbPlaylists.','.$nbPlaylists.');dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();
				dbCommitTransaction($db);
				dbBeginTransaction($db);
			}
		}
	}
//dbDisplayTable('tracks');exit;
	dbCommitTransaction($db);

	// Restore user stored playlists
	updateDBInsertStoredPlaylists();

	// Close DB
	dbClose($db);

	// Commit changes to actual DB
	commitTmpMusicDB();

	// Reload page
	echo '<form name="reloadForm"></form><script type="text/javascript">document.reloadForm.submit();</script>'."\n"; flush();
	exit();
}

/**
 * @desc Update music database with audio files found in winamp database
 * 		- library table (general library parameters)
 * 		- tracks table (tracks data)
 * 		- playlists table (playlists id and name)
 * 		- playlistsTracks table (playlist id <-> track id)
 * @param boolean $forceUpdate : set to true to force database update
 * @return true if success
 */
function updateDBWinamp($forceUpdate=false){
	if(!$forceUpdate) return true;
	/**
	 * Read One Winamp Database field
	 */
	function readWinampDatabaseField(&$fp){
		static $nnb;
		$fieldHeader=array();
		$fieldHeader['nextFieldOffset']=1;
		$ret=array();
		while($fieldHeader['nextFieldOffset']){
			if(!($rawHeader=fread($fp,14))) return $ret;

			$fieldHeader=unpack('Cid/CfieldType/VfieldDataSize/VnextFieldOffset/VprevFieldOffset',$rawHeader);

			$data=fread($fp,$fieldHeader['fieldDataSize']);

			switch ($fieldHeader['fieldType']){
				case FIELD_COLUMN:
					$data=substr($data,3);

					break;
				case FIELD_STRING:
				case FIELD_STRINGUTF8:
					$data=str_replace("\0",'',substr($data,4));
					break;
				case FIELD_INTEGER:
				case FIELD_DATETIME:
				case FIELD_LENGTH:
					$data=unpack('Vint',$data);
					$data=$data['int'];
					break;
				case FIELD_BOOLEAN:
					if(!$data) $data=false; else $data=true;
					break;
				case FIELD_BINARY:
					$data=substr($data,2);
					$data=$data['int'];
					break;
				case FIELD_GUID:
					break;
				case FIELD_FLOAT:
					$data=unpack('ffloat',$data);
					$data=$data['float'];
					break;
				default:
					$data='?';
			}
			$ret[$fieldHeader['id']]=$data;
		}
		return $ret;
	}

	/**
	 * Count winamp database number of records
	 */
	function countWinampDatabaseField(&$fp){
		$nbTracks=0;

		while(!feof($fp)){
			$fieldHeader=array();
			$fieldHeader['nextFieldOffset']=1;
			while($fieldHeader['nextFieldOffset']){
				if(!($rawHeader=fread($fp,14))) {$nbTracks--; break;} // EOF
				$fieldHeader=unpack('Cid/CfieldType/VfieldDataSize/VnextFieldOffset/VprevFieldOffset',$rawHeader);
				fseek($fp,$fieldHeader['fieldDataSize'],SEEK_CUR);
			}
			$nbTracks++;
		}
		return $nbTracks;
	}

	global $db;

	$lookForEmbededCover=cfRGetVar('lookForEmbededCover');

	require(INCLUDE_DIR.'miscFunctions.php');

	// Winamp Library file
	$libraryFile=cfRGetVar('musicWinampDatabaseFile');

	if(!file_exists($libraryFile)) outDisplayErrorPage(cfCaption('musicITunesFileNotFound','Winamp').'<br>('.$libraryFile.')');

	// Check library file hash to see if file has changed
	$libraryFileHash=md5_file($libraryFile);
	if(!$forceUpdate){
		if(cfRGetVar('databaseUpdateMethod')=='manual') return true;
		$lastLibraryFileHash=dbSingleQuery('SELECT value FROM library WHERE property="File Hash"',$db);
		if($lastLibraryFileHash==$libraryFileHash) return true;
	}

	set_time_limit(0);
	// Close session so other scripts aren't blocked
	wSession_write_close();

	// Display "refresh" page
	$msg='<script type="text/javascript">function uPB(nb,total){if(!total) {progressSetPerc("pBar",0);progressSetText("pBar","-");} else {progressSetPerc("pBar",100*nb/total);progressSetText("pBar",nb+" / "+total)}}</script><div id="action">'.cfCaption('musicRefreshingDatabase').'</div><br/><div id="progress" style="position:absolute;left:50%"><div style="position:relative;left:-150px">';
	$msg.='<div class="eTC">'.outProgressBar(0,'300px','-','pBar').'</div>';
	$msg.='</div></div>';

	echo outDisplayErrorPage($msg,array('icon'=>'unknown','title'=>cfCaption('genWait')),false);
	flush();

	// Create temporary database
	dbClose($db);
	openMusicDbTmp();

	// Init SQLITE query
	dbBeginTransaction($db);

	dbExec('INSERT INTO library VALUES ("File Hash","'.$libraryFileHash.'")',$db);

	// Set progress bar label
	echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('musicRefreshingDatabase').'"</script>'."\n"; flush();

	$nb=0;
	$playlistNb=0;
	$artworkNames=array('front','cover','folder');


	/**
	 * Parse Winamp database
	 */
	$fp=fopen($libraryFile,'rb');
	fseek($fp,8,SEEK_SET);

	// Read columns (array columnId => column name)
	$columns=readWinampDatabaseField($fp);

	// Translate from winamp names to local names
	foreach ($columns as $id=>$name){
		if($name=='length') $columns[$id]='Total Time';
		if($name=='filename') $columns[$id]='Location';
		if($name=='genre') $columns[$id]='Kind';
		if($name=='trackno') $columns[$id]='Track Number';
	}

	// Skip indexes
	readWinampDatabaseField($fp);

	// Read number of entries
	$pos=ftell($fp);
	$nbTracks=countWinampDatabaseField($fp);
	fseek($fp,$pos,SEEK_SET);

	// Read tracks
	$nb=0;
	$tracks=array();

	$unknownArtist=cfCaption('explorerAudioUnknownArtist');
	$unknownAlbum=cfCaption('explorerAudioUnknownAlbum');
	$unknownTitle=cfCaption('explorerAudioUnknownTitle');

	$lastUpdate=microtime(true);
	while (!feof($fp)) {
		// Read track from source DB
		$data=readWinampDatabaseField($fp);

		// Set default values
		$track=array();
		$track['Persistent ID']=$nb;
		$track['Track ID']=$nb;
		$track['Total Time']=0;

		$track['artist']=$unknownArtist;
		$track['album']=$unknownAlbum;
		$track['title']=$unknownTitle;
		$track['comment']='';
		$track['year']='1972';
		$track['Kind']=0;
		$track['Track Number']=0;

		// Convert columns ID to columns names
		foreach ($data as $id=>$value) $track[$columns[$id]]=$value;
		// Skip if location not found
		if(!isset($track['Location']) /*|| !file_exists($track['Location'])*/) continue;

		if(!isset($track['Date Added'])) $track['Date Added']=@filemtime($file);

		// Group by akbum artist
		if(isset($track['albumartist'])) $track['artist']=$track['albumartist'];
		//if(isset($track['Total Time'])) $track['Total Time']*=1000;

		// Check for cover
		if($lookForEmbededCover){
			$cover=new id3V2Ext($track['Location'],'APIC');
			if($cover->hasImage()) $track['Artwork']='embeded'.$nb.'.mp3';
			else $track['Artwork']='folder'.$nb.'.mp3';
		}
		else $track['Artwork']='folder'.$nb.'.mp3';


		// Insert in DB
		foreach ($track as $key=>$value) $track[$key]=dbEscapeString(cfUTF8Encode($value,false,false),$db);
			dbExec("INSERT INTO tracks VALUES ('".$track['Track ID'] . "','".$track['title']. "','".
				$track['artist'] . "','".$track['album'] . "','".$track['Track Number'] . "','".$track['Kind'] . "','".
				$track['Total Time'] . "','".$track['Date Added'] . "','".$track['comment'] . "','".
				$track['Persistent ID'] . "','".$track['Location'] . "','".$track['year'] . "','".$track['Artwork'] . "')",$db);

		// Also save in an array (name=>id) so mapping can be done for playlists
		$tracks[$track['Location']]=$track['Track ID'];

		$nb++;

		// Update progress bar
		if((microtime(true)-$lastUpdate>0.500)){
			echo '<script type="text/javascript">uPB('.$nb.','.$nbTracks.');</script>'."\n"; flush();
			dbCommitTransaction($db);
			dbBeginTransaction($db);
			$lastUpdate=microtime(true);
		}
	}

	fclose($fp);

	echo '<script type="text/javascript">uPB('.$nbTracks.','.$nbTracks.');</script>'."\n"; flush();

	// Save to database
	dbCommitTransaction($db);


	/**
	 * Parse playlists
	 */
	$playlists=array();
	dbBeginTransaction($db);
	echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();
	if(file_exists(dirname($libraryFile).'/playlists.xml')){
		// Parse main XML playlist file
		$domDocument = new DOMDocument;
		if((@$result=$domDocument->load(dirname($libraryFile).'/playlists.xml'))){
			$root=$domDocument->documentElement;
			$encoding=$domDocument->actualEncoding;
			if(($pl=$root->getElementsByTagName('playlist'))){
				for($i=0;$i<$pl->length;$i++){
					$playlist=$pl->item($i);
					if($playlist->hasAttribute('filename') /* && $playlist->hasAttribute('id')*/ && $playlist->hasAttribute('title') && @file_exists(dirname($libraryFile).'/'.$playlist->getAttribute('filename'))){
						$playlists[$i]=array(
							'filename'=>$playlist->getAttribute('filename'),
							'title'=>dbEscapeString($playlist->getAttribute('title')),$db);
					}
				}
			}
			$nbPlaylists=count($playlists);
			echo '<script type="text/javascript">uPB(0,'.$nbPlaylists.');dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();

			// Read individual playlists
			$lastUpdate=microtime(true);
			$nb=0;
			foreach ($playlists as $id=>$playlist){
				$lines=explode("\n",@file_get_contents(dirname($libraryFile).'/'.$playlist['filename']));
				$playlistsTracks=array();
				foreach ($lines as $line) {
					$line=trim($line);
					if(substr($line,0,1)!='#' && isset($tracks[$line])) $playlistsTracks[]=$tracks[$line];
				}
				// Non-empty playlist: add to database
				if(count($playlistsTracks)) {
					dbExec("INSERT INTO playlists VALUES ('".$id. "','".$playlist['title']."','s')",$db);
					foreach ($playlistsTracks as $trackId){
						dbExec("INSERT INTO playlistsTracks VALUES ('".$id. "','".$trackId."')",$db);
					}
				}
				else $nbPlaylists--;
				$nb++;
				// Update progress bar & commit SQL requests
				if((microtime(true)-$lastUpdate>0.500)){
					echo '<script type="text/javascript">uPB('.$nb.','.$nbPlaylists.');dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();
					$lastUpdate=microtime(true);
					dbCommitTransaction($db);
					dbBeginTransaction($db);
				}
			}
			unset($lines);
		}
		unset($domDocument);
	}
	// Update progress bar & commit SQL requests
	echo '<script type="text/javascript">uPB('.$nbPlaylists.','.$nbPlaylists.');dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n";
	flush();
	dbCommitTransaction($db);

	// Restore user stored playlists
	updateDBInsertStoredPlaylists();

	//dbDisplayTable('tracks');exit;

	// Close DB
	dbClose($db);

	// Commit changes to actual DB
	commitTmpMusicDB();

	// Reset time limit
	set_time_limit(ini_get('max_execution_time'));

	// Reload page
	echo '<form name="reloadForm"></form><script type="text/javascript">document.reloadForm.submit();</script>'."\n"; flush();
	exit();
}

/**
 * @desc Update music database with audio files found in cfRGetVar('path') subfolders :
 * 		- library table (general library parameters)
 * 		- tracks table (tracks data)
 * 		- playlists table (playlists id and name)
 * 		- playlistsTracks table (playlist id <-> track id)
 * @param boolean $forceUpdate : set to true to force database update
 * @return true if success
 */
function updateDBFolders($forceUpdate=false){
	global $db;

	// No automatic updates, only update if requested by user
	if(!$forceUpdate) return true;

	require_once(INCLUDE_DIR.'fileInfoFunctions.php');

	$filesListTmpFile=cfAppTempDir().'/tmp_audioDB_'.cfRGetVar('id').'.txt';
	$indexVarName=basename($filesListTmpFile);

	// Close session so other scripts aren't blocked, and remove time limit as database update may take a little time
	set_time_limit(0);
	wSession_write_close();

	/**
	 * Asynchronous audio files parsing
	 */
	if(@$_GET['updateDatabase']==='parseFiles') die(updateDBFoldersAsyncParseFiles($filesListTmpFile));


	// Display "refresh" page
	$msg='<script type="text/javascript">function uPB(nb,total){if(!total) {progressSetPerc("pBar",0);progressSetText("pBar","-");} else {progressSetPerc("pBar",100*nb/total);progressSetText("pBar",nb+" / "+total)}} function uAct(c){dgi("action").innerHTML=c}</script><div id="action">'.cfCaption('musicRefreshingDatabase').'</div><br/><div id="progress" style="position:absolute;left:50%"><div style="position:relative;left:-150px">';
	$msg.='<div class="eTC">'.outProgressBar(0,'300px','-','pBar').'</div>';
	$msg.='</div></div>';
	echo outDisplayErrorPage($msg,array('icon'=>'unknown','title'=>cfCaption('genWait')),false);
	flush();

	// Create temporary DB
	dbClose($db);

	openMusicDbTmp();

	// Set progress bar label
	echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('musicRefreshingDatabase').'"</script>'."\n"; flush();

	$nb=0;
	$playlistNb=0;
	$artworkNames=array('front','cover','folder');
	$dirs=array(cfRGetVar('path'));
	$filesList=array(); // Array of all audio files
	$playlistsList=array(); // Array of all playlist files

	// Debug
	if($debug=cfRGetVar('debug')) file_put_contents(cfAppDataDir().'/musicLog.txt','Audio library debug file');

	/**
	 * Browse base folder and subfolders and generate 2 lists for audio files and playlists
	 */
	$nbTracks=0; $lastUpdate=microtime(true);
	while ($dir=array_pop($dirs)) if(cfFileRights($dir,'state',true)=='authorized' && $dp=opendir($dir)){
		$tmpQuery=array();
		while ($file=readdir($dp)) if($file!='.' && $file!='..'){
			$file=$dir.'/'.$file;
			// If dir, add to directories array
			if(is_dir($file)) $dirs[]=$file;

			// If audio file, process
			if(efFileType($file)=='audio' && cfFileRights($file,'download',false)){
				// Playlist
				if(in_array(cfFileExtension($file),$_ENV['supportedPlaylists'])){
					$playlistsList[]=$file;
				}
				// Audio file
				else{
					$nbTracks++;
					$filesList[]=$file;
				}
			}
		}
		// Sync progress update
		if((microtime(true)-$lastUpdate>0.500)){
			echo '<script type="text/javascript">uPB(0,'.$nbTracks.');</script>'."\n"; flush();
			$lastUpdate=microtime(true);
		}
		closedir($dp);
	}


	// Save files to disk for asynchronous processing (see below)
	file_put_contents($filesListTmpFile,serialize($filesList));
	unset($filesList);


	/*
	 * Playlists
	 */

	// Init SQLITE query
	dbBeginTransaction($db);

	foreach ($playlistsList as $file){
		// Parse playlist
		$playlistTracks=fiParsePlaylistFile($file,true,false);

		// If not empty
		if(count($playlistTracks)) {
			// Insert into DB
			dbExec("INSERT INTO playlists VALUES ('".$playlistNb. "','".dbEscapeString(cfUTF8Encode(cfFileWithoutExtension(basename(str_replace(chr(0),'',$file)))),$db)."','s')",$db);

			// Insert tracks filenames into temporary DB for further matching with track ids.
			foreach ($playlistTracks as $plT){
				dbExec("INSERT INTO playlistsTemp VALUES ('".$playlistNb. "','".dbEscapeString(cfUTF8Encode(str_replace(chr(0),'',$plT['cfn'])),$db)."')",$db);
			}
			$playlistNb++;
		}
	}

	// Commit to DB
	dbCommitTransaction($db);

	/**
	 * Files
	 * Audio files are processed asynchronously so if scripts fails at some point, processing can restart at next file
	 */
	$scriptURI=$_SERVER['SCRIPT_NAME'].'?resId='.((isset($_POST['resId']))?$_POST['resId']:@$_GET['resId']).'&updateDatabase=parseFiles';

	// Store index of file to be processed
	cfMSetVar($indexVarName,0);

	// Close DB so asynchronous update may safely update database
	dbClose($db);


	/**
	 * Start async files parsing. Restart request as long as we're not done
	 */
	while(($pos=cfMGetVar($indexVarName))<$nbTracks){
		if($debug && $pos!=0) cfAppendTextToFile('Restart at position '.$pos,cfAppDataDir().'/musicLog.txt');
		cfPHPAsyncScript($scriptURI,0,'updateDBFoldersAsyncFlush');
	}
	@unlink($filesListTmpFile);




	// Reopen database for updating playlistsTracks table based on actual audio files into database
	$db=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME);

	// Generate playlistsTracks table (link tracks filenames with tracks ids)
	echo '<script type="text/javascript">uPB(0,0);dgi("action").innerHTML="'.cfCaption('explorerAudioPlaylists').'"</script>'."\n"; flush();
	dbExec('INSERT INTO playlistsTracks SELECT tmp.PlaylistID as PlaylistID, tracks.id as TrackId FROM playlistsTemp AS tmp INNER JOIN tracks ON tmp.TrackFilename=tracks.Location',$db);

	//dbDisplayTable('tracks');exit;


	// Restore user stored playlists
	updateDBInsertStoredPlaylists();


	// Close DB
	dbClose($db);

	if($debug) cfAppendTextToFile('Commit to DB...',cfAppDataDir().'/musicLog.txt');

	// Commit changes to actual DB (move temporary file over regular DB file)
	commitTmpMusicDB();

	if($debug) cfAppendTextToFile('All OK, reload page',cfAppDataDir().'/musicLog.txt');

	// Reload page
	echo '<form name="reloadForm"></form><script type="text/javascript">document.reloadForm.submit();</script>'."\n"; flush();
	exit();
}

/**
 * @desc Flush data returned by asynchronous audio file parsing (updateDBFoldersAsyncParseFiles function)
 *
 * @param unknown_type $data
 */
function updateDBFoldersAsyncFlush($data){
	echo $data;
	flush();
}

/**
 * @desc
 *
 * @param string $filesListTmpFile: files containing serialized list of all audio files to process
 */
function updateDBFoldersAsyncParseFiles($filesListTmpFile){
	$lastUpdate=microtime(true);
	$debug=cfRGetVar('debug');

	// Remove errors so they are not displayed into sync javascript progress update
	ini_set('display_errors',false);

	// Retreive audio files list
	$files=@unserialize(@file_get_contents($filesListTmpFile));
	if(!is_array($files)) die();
	$nbTracks=count($files);

	// Retreive index of 1st file to be processed (0 except if a failure occured)
	$indexVarName=basename($filesListTmpFile);
	$index=cfMGetVar($indexVarName);

	// Retreive pending queries (unset except if a failure occured)
	$queryVarName=$indexVarName.'_query';
	$tmpQuery=cfMGetVar($queryVarName);
	$queryItems=0;

	// Open database
	$db=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME);

	if($debug) cfAppendTextToFile('(re) start parsing at position '.$index,cfAppDataDir().'/musicLog.txt');
	if($debug && $index) cfAppendTextToFile('Resume from failure on file '.$files[$index-1],cfAppDataDir().'/musicLog.txt');

	$basePathLength=strlen(cfRGetVar('path')); // Persistent ids are computed on base path relative filenames

	while ($index<$nbTracks) {
		// Retreive file to be parsed
		$file=$files[$index];

		// Increase and save index so if file parsing fails, parsing will be resumed at next file
		cfMSetVar($indexVarName,++$index);

		// Debug
		if($debug) cfAppendTextToFile('(Parse '.$file,cfAppDataDir().'/musicLog.txt');

		$track=@efGetAudioInfo($file,false,array('noCache'));
		if($debug) cfAppendTextToFile('(Parsed',cfAppDataDir().'/musicLog.txt');

		if(!isset($track['artist'])) $track=array('artist'=>false,'album'=>false,'track'=>false,'year'=>false,'genre'=>false,'comment'=>false);

		$track['Persistent ID']=md5(substr($file,$basePathLength));
		$track['Track ID']=$index; // Mettre un hash de $file  cause des playlists
		$track['Total Time']=((isset($track['length']) && is_numeric($track['length']))?floor($track['length']/1000):0);
		$track['Date Added']=@filemtime($file);
		$track['Location']=$file;

		// Look for artwork in embeded ID3 tags
		if(isset($track['APIC']))
			$track['Artwork']='embeded'.$index.'.mp3';
		else
			$track['Artwork']='folder'.$index.'.mp3';

		$track['Kind']=$track['genre'];
		$track['Track Number']=$track['track'];
		$track['artist']=trim(str_replace(chr(0),'',$track['artist']));
		$track['album']=trim(str_replace(chr(0),'',$track['album']));

		// SQLite protect and utf-8 encode data
		foreach ($track as $key=>$value) $track[$key]=dbEscapeString(cfUTF8Encode($value,false,false),$db);

		// Build query
		$tmpQuery[]="INSERT INTO tracks VALUES ('".$track['Track ID'] . "','".str_replace(chr(0),'',$track['title']). "','".
		$track['artist'] . "','".$track['album'] . "','".$track['Track Number'] . "','".$track['Kind'] . "','".
		$track['Total Time'] . "','".$track['Date Added'] . "','".$track['comment'] . "','".
		$track['Persistent ID'] . "','".$track['Location'] . "','".$track['year'] . "','".$track['Artwork'] . "')";

		$queryItems++;

		// Sync progress update
		if((microtime(true)-$lastUpdate>0.500)){
			echo '<script type="text/javascript">uPB('.$index.','.$nbTracks.');</script>'."\n"; flush();
			$lastUpdate=microtime(true);
		}

		// Commit pending queries
		if($queryItems>=100 || $index>=$nbTracks){
			dbBeginTransaction($db);
			foreach ($tmpQuery as $sq) dbExec($sq,$db);
			dbCommitTransaction($db);

			// Reset pending queries (commited by cfMSetVar below)
			$tmpQuery=array();
			$queryItems=0;
		}

		// Or save to memory for if a problem occurs
		cfMSetVar($queryVarName,$tmpQuery);
	}

	dbClose($db);
	exit;
}

/**
 * @desc Insert stored playlists into main database playlists & playlistTracks tables.
 * This function is called after a database update/refresh
 *
 */
function updateDBInsertStoredPlaylists(){
	global $db;

	if(!($dbPl=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME))) return;

	// Get playlists to be imported
	$playlists=dbArrayQuery('SELECT * FROM usersPlaylists',$dbPl);
	// Get playlists tracks to be imported
	$playlistsTrackPIds=dbArrayQuery('SELECT PlaylistID, PersistentID FROM usersPlaylistsTracks',$dbPl);
	dbClose($dbPl);

	//dbExec("attach database '".cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME."' as plBase",$db);

	// Get destination playlist max id
	$maxId=dbSingleQuery('SELECT max(PlaylistID) FROM playlists',$db)+1;

	dbBeginTransaction($db);
	foreach ($playlists as $playlist){

		$query='';

		// Generate tracks insertion queries, and check playlist is not empty
		foreach ($playlistsTrackPIds as $track){
			if($track['PlaylistID']==$playlist['PlaylistID']) {
				$query.=' or PersistentID="'.$track['PersistentID'].'"';
				$tracks[]=$track;
			}
		}
		$query='SELECT id,PersistentID FROM tracks WHERE '.substr($query,4);
		$matches=dbArrayQuery($query,$db);

		// Create playlist in dest table if playlist is not empty
		if(count($matches)){
			// Create Playlist
			dbExec("INSERT INTO playlists VALUES ('".$maxId. "','".$playlist['Name']."','".$playlist['PlaylistID']."')",$db);

			// Insert Tracks
			foreach ($tracks as $nb=>$track){
				foreach ($matches as $match) if($match['PersistentID']==$track['PersistentID']){
					$query="INSERT INTO playlistsTracks VALUES (".$maxId. ",'".$match['id']."')";
					dbExec($query,$db);
					break;
				}
			}
			$maxId++;
		}
	}
	dbCommitTransaction($db);
}

/**
 * @desc Debug dump music database content
 *
 */
function mDBDumpBase(){
	global $db;
	dbDisplayTable('playlists');
	dbDisplayTable('playlistsTracks');
	dbDisplayTable('tracks');
	dbClose($db);
	$db=false;

	$dbPl=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME);
	dbDisplayTable('usersPlaylists',array(),$dbPl);
	dbDisplayTable('usersPlaylistsTracks',array(),$dbPl);
	dbClose($dbPl);
}

/**
 * @desc protect " with a \ for inclusion in javascript arrays
 *
 * @param string $string : string to protect
 * @return string : protected string
 */
function esc($string){return addslashes($string);}

/**
 * @desc protect " with a \ and limit size for inclusion in select
 *
 * @param string $string : string to protect
 * @return string : protected string
 */
function esc2($string){return addslashes(cfStrTruncate($string,35));}

/**
 * @desc open sqlite database and create tables if needed
 *
 * @return boolean : true if DB has been created, false if DB already created
 *
 */
function openMusicDb($dbFilename=false){
	global $db;
	// Open main DB
	if($dbFilename) $db=dbOpenDb($dbFilename);
	else $db=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_DATABASE_NAME);

	if($db==false) die("DB creation error");

	// Create playlists table if not existing
	if(!dbTableExists('library',$db)) {
		$createDB=true;

		@dbExec('CREATE TABLE library (property TEXT, value TEXT)',$db);

		// Create tracks table if not existing
		@dbExec('CREATE TABLE tracks ('.
		'id PRIMARY KEY, '.
		'Name TEXT, '.
		'Artist TEXT, '.
		'Album TEXT, '.
		'TrackNumber INTEGER, '.
		'Kind TEXT, '.
		'TotalTime INTEGER, '.
		'DateAdded INTEGER, '.
		'Comments TEXT, '.
		'PersistentID TEXT, '.
		'Location TEXT, '.
		'Year INTEGER, '.
		'Artwork TEXT'.
		')',$db);

		// Create playlists table if not existing
		@dbExec('CREATE TABLE playlists (PlaylistID PRIMARY KEY, Name TEXT, type TEXT)',$db); // playlist type is 's' for 'source' and PlaylistID of playlist in music_playlists.db database for user-generated playlists

		// Create playlists <-> tracks table if not existing
		@dbExec('CREATE TABLE playlistsTracks (PlaylistID INTEGER, TrackID INTEGER)',$db);

		// Create playlists temporary table if not existing, used for creating playlists from m3u or pls playlist files
		@dbExec('CREATE TABLE playlistsTemp (PlaylistID INTEGER, TrackFilename TEXT)',$db);
	}
	else $createDB=false;

	/**
	 * Stored playlists database
	 */
	if(!file_exists(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME)){
		if($dbPl=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME)){
			// Create playlists table if not existing
			if(!dbTableExists('usersPlaylists',$dbPl)) @dbExec('CREATE TABLE usersPlaylists (PlaylistID PRIMARY KEY, Name TEXT)',$dbPl);

			// Create playlists <-> tracks table if not existing
			if(!dbTableExists('usersPlaylistsTracks',$dbPl)) @dbExec('CREATE TABLE usersPlaylistsTracks (PlaylistID INTEGER, PersistentID INTEGER)',$dbPl);

			dbClose($dbPl);
		}
	}


	return $createDB;
}

/**
 * @desc Open temporary database used for generation
 *
 * @return boolean result
 */
function openMusicDbTmp(){
	global $db;
	// Unlink previous temporary database
	if(file_exists(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME) && !@unlink(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME)){
		// Or empty table if unlink impossible
		$db=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_DATABASE_NAME);
		dbBeginTransaction($db);
		dbExec('DELETE FROM library',$db);
		dbExec('DELETE FROM tracks',$db);
		dbExec('DELETE FROM playlists',$db);
		dbExec('DELETE FROM playlistsTracks',$db);
		dbExec('DELETE FROM playlistsTemp',$db);
		dbCommitTransaction($db);
		return true;
	}
	// Create new temporary database
	return openMusicDb(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME);
}

/**
 * @desc Commit temporary db
 *
 */
function commitTmpMusicDB(){
	@unlink(cfAppResourceDir().'/'.W_AUDIO_DATABASE_NAME);
	@rename(cfAppResourceDir().'/'.W_AUDIO_TMP_DATABASE_NAME,cfAppResourceDir().'/'.W_AUDIO_DATABASE_NAME);
}

/**
 * @desc recurse-read iTunes subfolders for artworks (.itc/.itc2)
 *
 * @param string $baseFolder: top folder
 * @param string $folder: current folder
 * @param string $extension: cover files extension (.itc/.itc2)
 * @return array (TrackId=>$baseFolder's subfolder, ...)
 */
function getITunesArtworks($baseFolder,$folder=null,$extension='itc'){
	if($folder==null) $folder=$baseFolder;
	$artworks=array();

	// Recurse browse subdirs
	foreach (cfGlob($folder.'/*') as $single){if(is_dir($single)) $artworks=array_merge($artworks,getITunesArtworks($baseFolder,$folder.'/'.basename($single),$extension));}

	// Look for itc or itc2 files
	foreach (cfGlob($folder.'/*') as $single){
		$ext=cfFileExtension($single); if($ext!='itc' && $ext!='itc2') continue;
		$PID=cfFileWithoutExtension(basename($single));
		$PID=substr($PID,strpos($PID,'-')+1);
		$artworks[$PID]=substr($folder,strlen($baseFolder));
	}

	return $artworks;
}

/**
 * @desc return empty track array
 *
 * @return array
 */
function emptyTrack(){
	return array('Track ID'=>false,
	'Name'=>false,
	'Artist'=>false,
	'Album'=>false,
	'Track Number'=>false,
	'Kind'=>false,
	'Total Time'=>false,
	'Date Added'=>false,
	'Comments'=>false,
	'Persistent ID'=>false,
	'Location'=>false,
	'Year'=>false,
	'Artwork'=>false);
}

/**
 * @desc process POST commands
 *
 */
function mDBprocessPost(){
	global $db;
	global $itemsPerPageList;

	cfAsyncXMLChangePhpVarValue('filterTd'); // process hide / show filter panel

	// If no POST command sent, return
	if(!isset($_POST['data3'])) return;
	$action=$_POST['data3'];

	// Strip backslashes in artist / album name
	if(isset($_POST['data2'])) $_POST['data2']=stripcslashes($_POST['data2']);
	if(isset($_POST['data4'])) $_POST['data4']=stripcslashes($_POST['data4']);

	/**
	 * Filter parameters
	 */
	if($action=='changeGroupBy' || $action=='changeDisplayType' || $action=='search' || $action=='changeItemsPerPage' || $action=='refreshDisplay' || $action=='changeOffset'){
		// Send async response header
		cfAsyncHeader();

		// Refresh selection area content
		//if($action=='refreshDisplay') asyncRefreshDisplay();
		// Change display type
		if($action=='changeDisplayType' && isset($_POST['data2'])) {
			if(in_array($_POST['data2'],array('displayCompact','displayFull','displayCovers','displayCoversExt'))) cfRSetVar('displayType',$_POST['data2']);
		}
		// Search filter
		if($action=='search' && isset($_POST['data2'])) {
			if(!isset($_POST['data4']) || !$_POST['data4']) echo cfAsyncXMLJSaction('dgi("searchInput").value="";');
			cfRSetVar('groupBy','Search');
			if(substr($_POST['data2'],0,1)=='=') $_POST['data2']=substr($_POST['data2'],1); // if search arg starts by '=', don't add wildcards
			elseif(strpos($_POST['data2'],'*')===false && strpos($_POST['data2'],'?')===false) $_POST['data2']='*'.$_POST['data2'].'*'; // else if no wildcard or ?; add * at start and end of search
			cfRSetVar('searchValue',str_replace('*','%',str_replace('?','_',($_POST['data2']))));
		}
		// Change grouping option
		if($action=='changeGroupBy' && isset($_POST['data2'])) {
			if($_POST['data2']!=cfRGetVar('groupBy')) cfRSetVar('displayOffset',0);
			if($_POST['data2']=='Artist' || $_POST['data2']=='Album' || $_POST['data2']=='Year' || $_POST['data2']=='Playlist')
				cfRSetVar('groupBy',$_POST['data2']);
		}
		// Change number of items displayed per page
		if($action=='changeItemsPerPage' && isset($_POST['data2']))
			if(in_array($_POST['data2'],$itemsPerPageList))	cfRSetVar('itemsPerPage',$_POST['data2']);

		// Change current offset
		if($action=='changeOffset' && isset($_POST['data2']))
			cfRSetVar('displayOffset', $_POST['data2']*cfRGetVar('itemsPerPage'));

		/**
		 * Refresh display
		 */
		// If mobile, we're on options page, don't update anything
		if(cfIsMobile() && ($action=='changeDisplayType' || $action=='changeItemsPerPage')) {}
		// Regular browser: update view
		else asyncRefreshDisplay();

		dbClose($db);
		cfAsyncFooter();
	}
	// Show artist / album / playlist detail
	// $_POST['data1'] : Album name ('Album' detail only);
	// $_POST['data2'] : Artist name or playlist ID
	// $_POST['data4'] : destination DOM id
	// $_POST['data5'] : detail type ('Artist', 'Album' or 'Playlist')
	elseif($action=='showDetail' && isset($_POST['data2']) && isset($_POST['data4']) && isset($_POST['data5'])){
		// Send async response header
		cfAsyncHeader();
		switch ($_POST['data5']){
			case 'Artist':
				showDetailArtist(stripslashes($_POST['data2']),$_POST['data4']);
				break;
			case 'Album':
				showDetailAlbum(stripslashes($_POST['data1']),stripslashes($_POST['data2']),$_POST['data4']);
				break;
			case 'Playlist':
				showDetailPlaylist(stripslashes($_POST['data2']),$_POST['data4']);
				break;
		}
		dbClose($db);
		echo cfAsyncFooter();
		exit();
	}

	/**
	 * Store playlist
	 */
	elseif ($action=='plStore' && $_POST['data2']){
		cfAsyncHeader();

		// Playlist name
		$playlistName=dbEscapeString(cfStrTruncate($_POST['data2'],128),$db);

		// Playlist tracks: convert from track id to track persistent id
		if(@count(cfRGetVar('playlist')) && cfRGetVar('storePlaylistAllowed')){
			$query='';
			foreach (cfRGetVar('playlist') as $track) $query.=' or id="'.dbEscapeString($track['externalRef'],$db).'"';

			// Convert from track id to track persistent id
			$query='SELECT id,PersistentID FROM tracks WHERE '.substr($query,4);
			$tmp=dbArrayQuery($query,$db);

			// Reorder results
			$persistentIds=array();
			foreach (cfRGetVar('playlist') as $track){
				foreach ($tmp as $idPid){
					if($idPid['id']==$track['externalRef']) {
						$persistentIds[$idPid['id']]=$idPid['PersistentID'];
						break;
					}
				}
			}

			// Save to stored playlists database
			if($dbPl=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME)){
				$maxId=dbSingleQuery('SELECT max(PlaylistID) FROM usersPlaylists',$dbPl)+1;
				dbExec("INSERT INTO usersPlaylists VALUES ('".$maxId. "','".$playlistName."')",$dbPl);
				$maxId=dbSingleQuery('SELECT max(PlaylistID) FROM usersPlaylists',$dbPl);

				dbBeginTransaction($dbPl);
				foreach (cfRGetVar('playlist') as $track) dbExec("INSERT INTO usersPlaylistsTracks VALUES (".$maxId. ",'".$persistentIds[$track['externalRef']]."')",$dbPl);
				dbCommitTransaction($dbPl);
				dbClose($dbPl);
				$usersPlaylistsID=$maxId;
			}
			else $error='cannot open playlists database for writing new playlist';

			// Add to main database playlists table
			if(isset($usersPlaylistsID)){
				$maxId=dbSingleQuery('SELECT max(PlaylistID) FROM playlists',$db)+1;

				dbExec("INSERT INTO playlists VALUES ('".$maxId. "','".$playlistName."','".$usersPlaylistsID."')",$db);

				dbBeginTransaction($db);
				foreach (cfRGetVar('playlist') as $track){
					dbExec("INSERT INTO playlistsTracks VALUES (".$maxId. ",'".$track['externalRef']."')",$db);
				}
				dbCommitTransaction($db);
			}
		}

		// Update display if playlists view is selected
		if(cfRGetVar('groupBy')=='Playlist') echo cfAsyncXMLJSaction('displayList('.groupByPlaylistData().',true)');

		dbClose($db);
		echo cfAsyncFooter();

		exit();

	}

	/**
	 * Delete stored playlist
	 */
	elseif ($action=='plDelete' && $_POST['data2']){
		cfAsyncHeader();

		// Playlist name
		$playlistId=$_POST['data2'];

		// Playlist tracks: convert from track id to track persistent id
		if(cfRGetVar('storePlaylistAllowed') && is_numeric($playlistId)){
			/**
			 * Remove from playlists table
			 */

			// 1st, ensure playlist exists and can be deleted
			$query='SELECT * FROM playlists WHERE PlaylistID="'.$playlistId.'"';
			$res=dbArrayQuery($query,$db);

			// If playlist not found or playlist is not user-generated, abort
			if(count($res)==0 || $res[0]['type']=='s') die(cfAsyncFooter());

			// Remove from playlists
			dbExec('DELETE FROM playlists WHERE PlaylistID="'.$playlistId.'"',$db);

			// Remove from tracks
			dbExec('DELETE FROM playlistsTracks WHERE PlaylistID="'.$playlistId.'"',$db);

			/**
			 * Delete from stored playlists database
			 */
			if($dbPl=dbOpenDb(cfAppResourceDir().'/'.W_AUDIO_PLAYLIST_DATABASE_NAME)){
				// Remove from playlists
				dbExec('DELETE FROM usersPlaylists WHERE PlaylistID="'.$res[0]['type'].'"',$dbPl);
				// Remove from tracks
				dbExec('DELETE FROM usersPlaylistsTracks WHERE PlaylistID="'.$res[0]['type'].'"',$dbPl);
				dbClose($dbPl);
			}

			// Update display
			echo cfAsyncXMLJSaction('displayList('.groupByPlaylistData().',true)');
		}

		//$query='INSERT INTO usersPlaylistsTracks';

		dbClose($db);
		cfAsyncFooter();
	}




	/**
	 * Send playlist to browser as M3U file
	 */
	if($action=='view' && @$_POST['data4']=='forceM3U'){
		efSendM3UFile('*playlist*.m3u');
		exit;
	}


	/**
	 * Inline play
	 * $_POST['data2'] format :
	 * 	- track number,
	 *  - or *resSpecific* /tracknumber.mp3,
	 *  - or *playlist*.m3u,
	 *  - or artist/"artist name"
	 *  - or album/"artist name"/"album name"
	 *
	 * Request may be async for regular browsers, and sync for mobiles
	 */
	elseif($action=='view' && isset($_POST['data2']) && (cfRGetVar('inlinePlayer')||(1&&cfIsMobile()))){
		// Send async response header
		if(cfIsAsync()) cfAsyncHeader();

		/**
		 * Set Track(s) to play
		 */
		require(INCLUDE_DIR.'viewFunctions.php');
		$passedFile=rawurldecode($_POST['data2']);
		if(substr($passedFile,0,14)=='*resSpecific*/') $passedFile=cfFileWithoutExtension(substr($passedFile,14));

		// Play inline playlist
		if($passedFile=='*playlist*.m3u') {
			// Generate shuffled playlist if needed
			if(cfRGetVar('shufflePlaylist')){
				$shuffle=cfRGetVar('playlist');
				shuffle($shuffle);
				cfRSetVar('playlistShuffled',$shuffle);
				unset($shuffle);
			}
			$fileExtSrc=$passedFile;
		}
		// Play whole artist
		elseif (cfCmpLeft($passedFile,'artist/')){
			@list($foo,$artist)=explode('/',$passedFile);
			$fileExtSrc='*tmp*playlist*.m3u';

			$tracks=dbArrayQuery("SELECT * FROM tracks WHERE Artist='".(dbEscapeString($artist,$db))."' LIMIT 200",$db);

			foreach ($tracks as $k=>$v) $tracks[$k]=mDBPlaylistItemFromTrack($v);
			cfRSetVar('playlistTmp',$tracks);
		}
		// Play whole album
		elseif (cfCmpLeft($passedFile,'album/')){
			@list($foo,$artist,$album)=explode('/',$passedFile);
			$fileExtSrc='*tmp*playlist*.m3u';
			$tracks=dbArrayQuery("SELECT * FROM tracks WHERE Artist='".dbEscapeString($artist,$db)."' AND Album='".(dbEscapeString($album,$db))."' LIMIT 200",$db);

			foreach ($tracks as $k=>$v) $tracks[$k]=mDBPlaylistItemFromTrack($v);
			cfRSetVar('playlistTmp',$tracks);
		}
		// Play whole playlist
		elseif (cfCmpLeft($passedFile,'playlist/')){
			@list($foo,$playlistId)=explode('/',$passedFile);
			$fileExtSrc='*tmp*playlist*.m3u';
			//$tracks=dbArrayQuery("SELECT * FROM playlistsTracks WHERE PlaylistID='".dbEscapeString($playlistId,$db)."' AND Lower(Album)='".strtolower(dbEscapeString($album,$db))."' LIMIT 200",$db);
			$query="SELECT * FROM tracks INNER JOIN playlistsTracks ON playlistsTracks.TrackID=tracks.id WHERE playlistsTracks.PlaylistID='".dbEscapeString($playlistId,$db)."' LIMIT 200";
			$tracks=dbArrayQuery($query,$db);

			// Remove "track." from every field name (!!! not needed since PDO SQLITE 3)
			/*
			$tracks=array();
			foreach ($tmpTracks as $k=>$v){
				$tmpTrack=array();
				foreach ($v as $k2=>$v2) $tmpTrack[substr($k2,7)]=$v2;
				$tracks[$k]=$tmpTrack;
			}
			*/
			foreach ($tracks as $k=>$v) $tracks[$k]=mDBPlaylistItemFromTrack($v);
			cfRSetVar('playlistTmp',$tracks);
		}
		// If not playlist, artist or album, it's a track, identified by it's numerical id, so if it's not numerical, abort
		elseif(!is_numeric($passedFile)) {echo cfAsyncFooter();exit;}
		// Single mp3
		else {
			$fileExtSrc='*resSpecific*/'.$passedFile.'.mp3';
			$tags=@efGetAudioInfo(dbSingleQuery("SELECT Location FROM tracks WHERE id='".dbEscapeString($passedFile,$db)."'",$db),false);

			cfRSetVar('singleMP3Label', rawurldecode($tags['label']));
		}

		/**
		 * Insert player
		 */
		// Async insertion (regular browser)
		if(cfIsAsync()){
			// Play file by inserting flash player pointing on file
			$script='';
			$audioPlayer=vfInsertAudioPlayer($fileExtSrc,'normal',true,'95%',40,false,$script);
			echo cfAsyncXMLInnerHTMLbyId($audioPlayer,'flashPlayerAsyncOuterDiv'); // Insert new flash audio player
			if($script) echo cfAsyncJSaction($script); // If a script is returned (HTML5 audio), it overrides flash player

			// Update song info displayed on right-side of player
			echo cfAsyncXMLJSaction(inlineUpdatePlayerInfoScript($passedFile));

			dbClose($db);
			die(cfAsyncFooter());
		}
		// Sync display (mobile)
		else{
			vfViewFile($fileExtSrc,(isset($_POST['data3']))?(cfUTF8Decode(($_POST['data3']))):(''),
				(isset($_POST['data4']))?(cfUTF8Decode(($_POST['data4']))) : (''),
				(isset($_POST['data5'])?(cfUTF8Decode(($_POST['data5']))):('')));
		}
	}

	// Update cover & played track info (from information sent by flash player)
	elseif ($action=='inlineUpdatePlayerInfo' && (cfRGetVar('inlinePlayer')||cfIsMobile()) && isset($_POST['data2'])){
		// Send async response header
		cfAsyncHeader();
		if(is_numeric($_POST['data2'])) echo cfAsyncXMLJSaction(inlineUpdatePlayerInfoScript($_POST['data2']));
		dbClose($db);
		echo cfAsyncFooter();
		exit();
	}

	/**
	 * Playlist related actions
	 */

	// Add artist / playlist / track / random to playlist
	elseif (($action=='plAddArtist' || $action=='plAddPlaylist' || $action=='plAddTrack' || $action=='plAddAllShuffle') && isset($_POST['data2'])){
		// Send async response header
		cfAsyncHeader();
		playlistAddItem($action,$_POST['data2']);
		echo cfAsyncXMLJSaction('plRefresh('.efPlayListContentArray().')');
		echo efMultipleDownloadUpdateBt();
		echo cfAsyncXMLJSaction('mProcessing(false)');
		dbClose($db);
		cfAsyncFooter();
	}
	// Add album to playlist (data2=artist name, data4=album name)
	elseif ($action=='plAddAlbum' && isset($_POST['data2']) && isset($_POST['data4'])){
		// Send async response header
		cfAsyncHeader();
		playlistAddItem($action,$_POST['data2'],$_POST['data4']);
		echo cfAsyncXMLJSaction('plRefresh('.efPlayListContentArray().')');
		echo efMultipleDownloadUpdateBt();
		echo cfAsyncXMLJSaction('mProcessing(false)');
		dbClose($db);
		cfAsyncFooter();
	}

	// If no resource specific command is passed, process explorer common commands
	else efProcessPOST();

	// Common playlist actions
	if($action=='plAdd' || $action=='plSupp' || $action=='mSuppAll' || $action=='plUp' || $action=='plDown'){
		// Send async response header
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('plRefresh('.efPlayListContentArray().')');
		// Update buttons
		echo efMultipleDownloadUpdateBt();
		// Update reloading icon
		echo cfAsyncXMLJSaction('mProcessing(false)');
		die(cfAsyncFooter());
	}

	// Async garbage
	if(cfIsAsync() && !cfIsMobile()){
		// Send async response header
		cfAsyncHeader();
		die(cfAsyncFooter());
	}
}

/**
 * @desc return JS code for (async) updating inline player cover and track info
 *
 * @param string $passedFile : track id or *playlist*.m3u
 * @return string : JS code
 */
function inlineUpdatePlayerInfoScript($passedFile){
		global $db;
		function cleanItem($utf8EncodedItem){
			$ci=cfXMLEncode(cfUTF8Encode(str_replace('"','\\"',cfStrTruncate(cfUTF8Decode($utf8EncodedItem,true,false,false),20)),false,false));
			if($ci) return $ci;
			return '-';
		}

		$jsAction='inlineSetSongPlayed(';
		// Playlist
		if(in_array(cfFileExtension($passedFile),$_ENV['supportedPlaylists']))
			$jsAction.='"'.dirname($_SERVER['PHP_SELF']).'/playlist.jpg","'.cfCaption('explorerAudioPlaylist').'","-","-")';
		// Single track
		else{
			$trackData=dbArrayQuery('SELECT Artist,Album,Name,Artwork FROM tracks WHERE id="'.dbEscapeString($passedFile,$db).'"',$db);
			// Not found!
			if(!count($trackData)) return '';
			$artistName=$trackData[0]['Artist']; $albumName=$trackData[0]['Album']; $trackName=$trackData[0]['Name'];
			// Get audio track cover
			// If track has artwork info, use it
			if($trackData[0]['Artwork']) $art=$trackData[0]['Artwork'];
			// else, select artwork from other tracks from same artist/album
			else $art=dbSingleQuery('SELECT max(Artwork) FROM tracks WHERE Artist="'.dbEscapeString($artistName,$db).'" AND Album="'.dbEscapeString($albumName,$db).'"',$db);
			// If iTunes, add itc extension (not stored into db)
			//if(SOURCE_TYPE=='iTunes') $art.='.itc';
			if((SOURCE_TYPE!='iTunes' && $art) || strlen($art)>18) {
				if(cfIsMobile())
					$w=max(DEFAULT_INLINE_COVER_WIDTH,min(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight'))/2);
				else
					$w=DEFAULT_INLINE_COVER_WIDTH;

				$jsAction.='"'.stripslashes(cfExtImage(cfRGetVar('path').'/'.$art,$w,$w)).'"';
			}
			else  $jsAction.='"'.dirname($_SERVER['PHP_SELF']).'/cover.png"';
			// Display cover, artist, album and track name
			$jsAction.=',"'.cleanItem($artistName).'"'.',"'.cleanItem($albumName).'"'.',"'.cleanItem($trackName).'")';
		}

		return $jsAction;
}

/**
 * @desc return "unknown artist" if provided name is empty, else return name
 *
 * @param string $name
 * @return string
 */
function artistName($name){
	if($name=='') return cfCaption('explorerAudioUnknownArtist'); return $name;
}

/**
 * @desc return "unknown album" if provided name is empty, else return name
 *
 * @param string $name
 * @return string
 */
function albumName($name){
	if($name=='') return cfCaption('explorerAudioUnknownAlbum'); return $name;
}

/**
 * @desc Return current mixed groupBy/display type filter (artistFull, albumCompact...)
 *
 * @return string
 */
function mBDMixedFilter(){
	return str_replace('display',strtolower(cfRGetVar('groupBy')),cfRGetVar('displayType'));
}


/**
 * @desc Return displayType, artist data array, select list array, current offset
 * @return string : "display type, Array(Array('Artist1Name',nbAlbums1,nbTracks1,artSrc),...), Array('Artist1 - ArtistX','ArtistX+1 - Artist2X',...)
 *
 */
function groupByArtistData($filter=false){
	global $db;

	@dbExec('DROP VIEW tmpArtist',$db);
	if(SOURCE_TYPE=='iTunes') $op='max'; else $op='min'; // iTunes needs to get highest value as it contains empty elements, other need lowest as values are folderXXX or embedXXX, and embed is prefered over folder
	dbExec("CREATE VIEW tmpArtist AS SELECT max(Artist) as Artist, max(Album) AS Album, count(id) AS nbTracks, ".$op."(Artwork) AS art FROM tracks ".(($filter===false)?"":$filter." ")."GROUP BY lower(Album),lower(Artist);",$db);
	$list=dbArrayQuery("SELECT min(Artist) AS Artist, count(Album) AS nbAlbums, sum(nbTracks) AS nbTracks, ".$op."(art) AS art FROM tmpArtist GROUP BY lower(Artist) ORDER BY lower(Artist) ASC",$db);

	// Count results to detect result-less search request
	cfRSetVar('searchNbResults',cfRGetVar('searchNbResults')+count($list));

	/**
	 * Generate content array
	 */
	$nbItems=0;$pos=0;
	$array='"artist","'.strtolower(str_replace('display','',cfRGetVar('displayType'))).'","'.mBDMixedFilter().'", [';
	foreach ($list as $item){
		if(cfRGetVar('itemsPerPage')==0 || $filter!==false || ($pos>=cfRGetVar('displayOffset') && $pos<(cfRGetVar('displayOffset')+cfRGetVar('itemsPerPage')))){
			if($nbItems!=0) $array.=',';
			if(SOURCE_TYPE=='iTunes')
				if(cfRGetVar('displayType')!=='displayCompact' && strlen($item['art'])>18)
					$art=str_replace(cfRGetVar('Library Persistent ID'),'#',$item['art']);
				else
					$art='';
			else
				$art=cfProtectFilenameURL($item['art']);
			$array.='["'.(esc(artistName($item['Artist']))).'",'.$item['nbAlbums'].','.$item['nbTracks'].',"'.$art.'"]';
			$nbItems++;
		}
		$pos++;
	}
	$array.='], ';

	reset($list);
	/**
	 * Generate select array
	 */
	if($filter!==false) $array.='[],';
	elseif(cfRGetVar('itemsPerPage')==0) $array.='[",'.esc2(artistName($list[0]['Artist'])).' <-----> '.esc2(artistName($list[count($list)-1]['Artist'])).'"],';
	else {
		$array.='[';
		for($pos=0;$pos<=floor((count($list)-1)/cfRGetVar('itemsPerPage'));$pos++){
			if($pos>0) $array.=',';
			if($pos*cfRGetVar('itemsPerPage')<=cfRGetVar('displayOffset') && ($pos+1)*cfRGetVar('itemsPerPage')>cfRGetVar('displayOffset')) $array.='"S'; else $array.='"N';
			$array.=esc2(artistName($list[$pos*cfRGetVar('itemsPerPage')]['Artist'])).' <-----> '.esc2(artistName($list[min(count($list)-1,(($pos+1)*cfRGetVar('itemsPerPage')-1))]['Artist'])).'"';
		}
		$array.='], ';
	}

	/**
	 * Current offset
	 */
	return $array.(int)cfRGetVar('displayOffset');
}

/**
 * @desc Return displayType, album data array, select list array, current offset
 * @return string : "display type, Array(Array('Album1Name','Artist1Name',nbTracks1,totalTime1),...),...),...)
 */
function groupByAlbumData($filter=false){

	global $db;
	// iTunes needs to get highest value as it contains empty elements, other need lowest as values are folderXXX or embedXXX, and embed is prefered over folder
	if(SOURCE_TYPE=='iTunes') $op='max'; else $op='min';
	$list=dbArrayQuery("SELECT min(Album) AS Album, min(Artist) AS Artist, ".$op."(Artwork) AS art, count(*) AS nbTracks, sum(TotalTime) AS totalTime FROM tracks ".(($filter===false)?'':$filter.' ')."GROUP BY lower(Album), lower(Artist) ORDER BY lower(Artist) ASC",$db);

	// Count results to detect result-less search request
	cfRSetVar('searchNbResults',cfRGetVar('searchNbResults')+count($list));

	/**
	 * Generate content array
	 */
	$nbItems=0;$pos=0;
	$array='"album","'.strtolower(str_replace('display','',cfRGetVar('displayType'))).'","'.mBDMixedFilter().'", [';
	foreach ($list as $item){
		if(cfRGetVar('itemsPerPage')==0 || $filter!==false || ($pos>=cfRGetVar('displayOffset') && $pos<(cfRGetVar('displayOffset')+cfRGetVar('itemsPerPage')))){
			if($nbItems!=0) $array.=',';
			if(SOURCE_TYPE=='iTunes')
				if(cfRGetVar('displayType')!=='displayCompact' && strlen($item['art'])>18) $art=str_replace(cfRGetVar('Library Persistent ID'),'#',$item['art'])/*.'.itc'*/; else $art='';
			else $art=cfProtectFilenameURL($item['art']);

			$array.='["'.(esc(albumName($item['Album']))).'","'.(esc(artistName($item['Artist']))).'",'.$item['nbTracks'].','.$item['totalTime'].',"'.$art.'"]';
			$nbItems++;
		}
		$pos++;
	}
	$array.='], ';
	reset($list);

	/**
	 * Generate select array
	 */
	if($filter!==false) $array.='[],';
	elseif(cfRGetVar('itemsPerPage')==0) $array.='[",'.esc2(albumName($list[0]['Artist'])).' <-----> '.esc2(albumName($list[count($list)-1]['Artist'])).'"],';
	else {
		$array.='[';
		for($pos=0;$pos<=floor((count($list)-1)/cfRGetVar('itemsPerPage'));$pos++){
			if($pos>0) $array.=',';
			if($pos*cfRGetVar('itemsPerPage')<=cfRGetVar('displayOffset') && ($pos+1)*cfRGetVar('itemsPerPage')>cfRGetVar('displayOffset')) $array.='"S'; else $array.='"N';
			$array.=esc2(albumName($list[$pos*cfRGetVar('itemsPerPage')]['Artist'])).' <-----> '.esc2(albumName($list[min(count($list)-1,(($pos+1)*cfRGetVar('itemsPerPage')-1))]['Artist'])).'"';
		}
		$array.='], ';
	}

	/**
	 * Current offset
	 */
	return $array.(int)cfRGetVar('displayOffset');
}

/**
 * @desc Return displayType('compact'), playlist data array, select list array, current offset
 * @return string : "display type, Array(Array('PlaylistID','PlaylistName',nbTracks1,canDelete),...), Array(...), current offset
 */
function groupByPlaylistData($filter=false){
	global $db;
	$list=dbArrayQuery("SELECT playlists.playlistId AS PlaylistID, playlists.Name AS PlaylistName, count(*) AS nbTracks, type FROM playlists INNER JOIN playlistsTracks ON playlists.PlaylistID=playlistsTracks.PlaylistID ".(($filter===false)?'':$filter.' ')."GROUP BY playlists.playlistId",$db);

	// Count results to detect result-less search request
	cfRSetVar('searchNbResults',cfRGetVar('searchNbResults')+count($list));

	// Check if user may delete stored playlists
	$mayDelete=cfRGetVar('storePlaylistAllowed');

	/**
	 * Generate content array
	 */
	$nbItems=0;$pos=0;
	$array='"playlist","'.strtolower(str_replace('display','',cfRGetVar('displayType'))).'","'.mBDMixedFilter().'", [';
	foreach ($list as $item){
		if(cfRGetVar('itemsPerPage')==0 || $filter!==false || ($pos>=cfRGetVar('displayOffset') && $pos<(cfRGetVar('displayOffset')+cfRGetVar('itemsPerPage')))){
			if($nbItems!=0) $array.=',';
			$array.='["'.$item['PlaylistID'].'","'.esc($item['PlaylistName']).'",'.$item['nbTracks'].','.(($mayDelete && $item['type']=='s')?0:1).']';
			$nbItems++;
		}
		$pos++;
	}
	$array.='], ';
	reset($list);
	/**
	 * Generate select array
	 */
	if($filter!==false) $array.='Array(),';
	elseif(cfRGetVar('itemsPerPage')==0) $array.='Array(",'.esc2($list[0]['PlaylistName']).' <-----> '.esc2($list[count($list)-1]['PlaylistName']).'"),';
	else {
		$array.='[';
		for($pos=0;$pos<=floor((count($list)-1)/cfRGetVar('itemsPerPage'));$pos++){
			if($pos>0) $array.=',';
			if($pos*cfRGetVar('itemsPerPage')<=cfRGetVar('displayOffset') && ($pos+1)*cfRGetVar('itemsPerPage')>cfRGetVar('displayOffset')) $array.='"S'; else $array.='"N';
			$array.=esc2($list[$pos*cfRGetVar('itemsPerPage')]['PlaylistName']).' <-----> '.esc2($list[min(count($list)-1,(($pos+1)*cfRGetVar('itemsPerPage')-1))]['PlaylistName']).'"';
			//$array.=cfUTF8Encode($list[$pos*cfRGetVar('itemsPerPage')]['PlaylistName']).' <-----> '.cfUTF8Encode($list[min(count($list)-1,(($pos+1)*cfRGetVar('itemsPerPage')-1))]['PlaylistName']).'"';
		}
		$array.='], ';
	}
	/**
	 * Current offset
	 */

	return $array.(int)cfRGetVar('displayOffset');
}

/**
 * @desc Return tracks array() matching filter
 * @return string : "Array(Array('trackId','trackName','albumName','artistName','totalTime1),Array(),...)
 */
function groupByTrackData($filter=false){
	global $db;

	// Get matching tracks
	$list=dbArrayQuery("SELECT id, name, totalTime, Album, Artist FROM tracks ".(($filter===false)?'':$filter.' ')."GROUP BY lower(Album), lower(Artist), name ORDER BY lower(Album) ASC".((cfRGetVar('itemsPerPage'))?" LIMIT ".cfRGetVar('itemsPerPage'):''),$db);
	$filter='';

	// Count results to detect result-less search request
	cfRSetVar('searchNbResults',cfRGetVar('searchNbResults')+count($list));

	/**
	 * Generate content array
	 */
	$tracks='';
	foreach ($list as $item){
		if(!isset($item['Name']) && isset($item['name'])) $item['Name']=$item['name']; // Ugly bug fix
		$tracks.=',["'.$item['id'].'","'.(esc(artistName($item['Name']))).'","'.(esc(albumName($item['Album']))).'","'.(esc(artistName($item['Artist']))).'",'.@$item['TotalTime'].']';
	}
	return '"tracks","detail","tracksDetail", ['.substr($tracks,1).'],[],0';
}


function mDBCaseSort($a,$b){
	$l=strlen($a);
	for($i=0;$i<$l;$i++){
		if($a[$i]!=$b[$i]){
			if(strtolower($a[$i])==$a[$i]) return -1; else return 1;
		}
	}
	return 0;
}

/**
 * @desc async return Javascript code for displaying artist detail (list of tracks grouped by album)
 *
 * @param string $artistName : utf-8 encoded artist name
 * @param string $destId : id of node into which HTML code should be inserted
 *
 * @return output : "Artist", displayType, destId, Array(
 * 						Array(Album1 name, Album1 artwork, Album1 year, Album1 total time, Album1 nb tracks,
 * 							Array(
 *	 							Array(Track1 ID, Track1 name, Track1 total time),
 * 								Array(Track2 ID, Track2 name, Track2 total time),
 * 								...
 * 						...),
 * 					)
 */
function showDetailArtist($artistName, $destId){
	global $db;
	// Empty artist caption : replace by empty string
	if($artistName==cfCaption('explorerAudioUnknownArtist')) $artistName='';

	$list=dbArrayQuery("SELECT Album, id, PersistentId, Name, Artwork, Year, TotalTime FROM tracks WHERE Artist='".dbEscapeString($artistName,$db)."' ORDER BY lower(Album) ASC, TrackNumber ASC",$db);
	$nbTracks=0;
	$array=''; $tracks='';
	$album=''; $artwork=''; $year=0; $totalTime=0; $nbTracks=0;

	// Keep "upercasemost" version of albums names
	$ucAlbums=array();
	foreach ($list as $track) if(!isset($ucAlbums[strtolower($track['Album'])]) || mDBCaseSort($ucAlbums[strtolower($track['Album'])],$track['Album'])<0)
		$ucAlbums[strtolower($track['Album'])]=$track['Album'];
	foreach ($list as $id=>$track) $list[$id]['Album']=$ucAlbums[strtolower($track['Album'])];

	foreach ($list as $track){
		// Write previous album data
		if(albumName($track['Album'])!=$album && $album!=''){
			$array.=',Array("'.esc($album).'","'.$artwork.'","'.$year.'",'.$totalTime.','.$nbTracks.',Array('.substr($tracks,1).'))'."\n";
			$artwork=''; $year=0; $totalTime=0; $nbTracks=0;$tracks='';
		}
		$album=$track['Album'];
		$tracks.=',["'.$track['id'].'","'.esc($track['Name']).'",'.$track['TotalTime'].']';
		$nbTracks++;
		$totalTime+=$track['TotalTime'];
		if(isset($track['year']) && $track['year']) $year=$track['year'];

		if(SOURCE_TYPE=='iTunes')
			{if(!$artwork && strlen($track['Artwork'])>18) $artwork=str_replace(cfRGetVar('Library Persistent ID'),'#',$track['Artwork'])/*.'.itc'*/;}
		else
			{
				if(!$artwork || (substr($artwork,0,6)=='folder' && substr($track['Artwork'],0,5)=='embed')) $artwork=cfProtectFilenameURL($track['Artwork']);
			}

	}
	$array.=',["'.esc($album).'","'.$artwork.'","'.$year.'",'.$totalTime.','.$nbTracks.',['.substr($tracks,1).']]';
	$array=str_replace(chr(0),'',$array);

	echo cfAsyncXMLJSaction('insertDetail("Artist", "'.cfRGetVar('displayType').'", "'.$destId.'",['.substr($array,1).'])');
}

/**
 * @desc async return Javascript code for displaying single album detail
 *
 * @param string $artistName : utf-8 encoded artist name
 * @param string $destId : id of node into which HTML code should be inserted
 *
 * @return output : "Album", displayType, destId,
 * 							Array(
 *	 							Array(Track1 ID, Track1 name, Track1 total time),
 * 								Array(Track2 ID, Track2 name, Track2 total time),
 * 								...
 * 						...),
 * 					)
 */
function showDetailAlbum($albumName, $artistName, $destId){
	global $db;

	// Empty album caption : replace by empty string
	if($albumName==cfCaption('explorerAudioUnknownAlbum')) $albumName='';
	$list=dbArrayQuery("SELECT Album, id, Name, TrackNumber, TotalTime FROM tracks WHERE Artist='".(dbEscapeString($artistName,$db))."' AND Album='".(dbEscapeString($albumName,$db))."' ORDER BY TrackNumber ASC",$db);

	$tracks='';
	foreach ($list as $track){
		$tracks.=',["'.$track['id'].'","'.esc($track['Name']).'",'.$track['TotalTime'].']';
	}
	$tracks=str_replace(chr(0),'',$tracks);
	echo cfAsyncXMLJSaction('insertDetail("Album", "'.cfRGetVar('displayType').'", "'.$destId.'",['.substr($tracks,1).'])');
}

/**
 * @desc async return Javascript code for displaying playlist detail (list of tracks)
 *
 * @param string $playlistId : playlist id
 * @param string $destId : id of node into which HTML code should be inserted
 *
 * @return output : "Playlist", displayType, destId, Array(
 * 						Array(
 *	 						Array(Track1 ID, Track1 name, Track1 total time, Track1 artist, Track1 album),
 * 							Array(Track2 ID, Track2 name, Track2 total time, Track2 artist, Track2 album),
 * 							...
 * 						),
 * 					)
 */
function showDetailPlaylist($playlistId, $destId){
	global $db;
	$list=dbArrayQuery("SELECT t.id AS id, t.Artist AS Artist, t.Album AS Album, t.Name AS Name, t.TotalTime AS TotalTime FROM playlistsTracks pt INNER JOIN tracks t ON t.id=pt.trackId WHERE pt.PlaylistID='".$playlistId."'",$db);

	$tracks='';
	foreach ($list as $track){
		$tracks.=',["'.$track['id'].'","'.esc($track['Name']).'",'.$track['TotalTime'].',"'.esc(artistName($track['Artist'])).'","'.esc(artistName($track['Album'])).'"]';
	}
	$tracks=str_replace(chr(0),'',$tracks);
	echo cfAsyncXMLJSaction('insertDetail("Playlist", "'.cfRGetVar('displayType').'", "'.$destId.'",['.substr($tracks,1).'])');
}

/**
 * @desc return playlist track label from data present in tracks table
 * @param $trackArray : array(id, Artist, Album, Name)
 * @return string : label
 *
 */
function playlistLabelFromTrackInfo($trackArray){
	$label=artistName($trackArray['Artist']).' - '.albumName($trackArray['Album']);
	if(isset($trackArray['TrackNumber']) && $trackArray['TrackNumber']) $label.=' - '.$trackArray['TrackNumber'];

	return $label.' - '.$trackArray['Name'];
}

/**
 * @desc add a track, album, artist or playlist to playlist
 *
 * @param string $itemType : 'plAddTrack' or 'plAddAlbum' or 'plAddArtist' or 'plAddPlaylist'.
 * @param mixed $itemId : track id if 'plAddTrack', playlist id if 'plAddPlaylist', else artist name
 * @param mixed $albumName : album name (only for 'plAddAlbum')
 */
function playlistAddItem($itemType, $itemId, $albumName=false){
	global $db;
	// Get actual playlist content
	$arr=cfRGetVar('playlist');

	// Clear unknown album and unknown artist values
	if($itemId==cfCaption('explorerAudioUnknownArtist')) $itemId=''; //else $itemId=strtolower((cfUTF8Encode(strtolower(cfUTF8Decode($itemId)))));
	if($albumName==cfCaption('explorerAudioUnknownAlbum')) $albumName=''; //else $albumName=strtolower((cfUTF8Encode(strtolower(cfUTF8Decode($albumName)))));
	$itemId=strtolower($itemId);
	$albumName=strtolower($albumName);

	// Set file properties
	switch ($itemType) {
		case 'plAddArtist':
			$list=dbArrayQuery("SELECT id, Artist, Album, TrackNumber, Name, TotalTime FROM tracks WHERE lower(Artist)='".dbEscapeString($itemId,$db)."' ORDER BY lower(Artist) ASC, lower(Album) ASC, TrackNumber ASC",$db);
			foreach ($list as $track)
				$arr[]=array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track)), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
			break;

		case 'plAddAlbum':
			$list=dbArrayQuery("SELECT id, Artist, Album, TrackNumber, Name, TotalTime FROM tracks WHERE lower(Artist)='".dbEscapeString($itemId,$db)."' AND lower(Album)='".dbEscapeString($albumName,$db)."' ORDER BY lower(Artist) ASC, lower(Album) ASC, TrackNumber ASC",$db);
			foreach ($list as $track)
				$arr[]=array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track)), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
			break;

		case 'plAddPlaylist':
			$list=dbArrayQuery("SELECT t.id AS id, t.Artist AS Artist, t.Album AS Album, t.TrackNumber AS TrackNumber, t.Name AS Name, t.TotalTime AS TotalTime FROM tracks t INNER JOIN playlistsTracks pt ON t.id=pt.TrackID WHERE pt.PlaylistID='".dbEscapeString($itemId,$db)."'",$db);
			foreach ($list as $track)
				$arr[]=array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track)), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
			break;

		case 'plAddTrack':
			$list=dbArrayQuery("SELECT id, Artist, Album, TrackNumber, Name, TotalTime FROM tracks WHERE id='".dbEscapeString($itemId,$db)."'",$db);
			if(count($list)==1){
				$track=$list[0];
				$arr[]=array('completeFileName'=>'*resSpecific*/'.$itemId.'.mp3', 'name'=>'*resSpecific*/'.$itemId.'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($list[0])), 'externalRef'=>$list[0]['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
			}
			break;
		case 'plAddAllShuffle':
			$arr=array();// Clear previous playlist
			$list=dbArrayQuery("SELECT id, Artist, Album, TrackNumber, Name, TotalTime FROM tracks ORDER BY Random() LIMIT ".max(1,cfRGetVar('plAddAllShuffleSize')),$db);
			foreach ($list as $track)
				$arr[]=array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track)), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);
			break;

			break;
		default:
			break;
	}
	// &amp; issue (generated by cleanHTML option of cfUTF8Decode)
	foreach ($arr as $k=>$v) $arr[$k]['label']=str_replace('&amp;','&',$arr[$k]['label']);

	// Commit changes to playlist
	cfRUnsetVar('playlist');
	cfRSetVar('playlist',$arr);
	cfRSetVar('noFileRefresh',true);

	// Create multiple download list from playlist
	efConvertPlayListToMDList();
}

/**
 * @desc display filter area HTML code
 * @return void
 *
 */
function displayFilter(){
	global $itemsPerPageList;

	/**
	 * Database update button
	 */
	$output=outDivFrame('frame1','id="filterDiv"','width:16em');
	if(cfRGetVar('databaseUpdateMethod')=='manual') {
		$r=outButton(cfCaption('genDataUpdate'),'javascript:updateDatabase()',outIcon('reload2'));
		//$r=outBt(cfCaption('genDataUpdate'),outIcon('reload2'),'javascript:updateDatabase()');
		if(cfIsOnLAN()) $r.='<div id="debugBt" style="display:none" onclick="dgi(\'debugInstructions\').style.display=\'\';updateDatabase(1)">Debug database update</div>';
		if(cfIsOnLAN()) $r.='<span id="debugInstructions" style="display:none">Database update launched. Look for debug output in:<br>'.(cfAppDataDir().'/musicLog.txt').'</span>';
	}
	else $r='';
	$output.='<div class="frame1Header">'.cfCaption('genFilter')."</div>\n";
	$output.=$r;

	/**
	 * Search
	 */
	$output.=outDivFrame('frame2');
	$output.='<div class="frame2Header"><input id="groupBySearch" type="radio" name="groupBy" value="Search"'.((cfRGetVar('groupBy')=='Search')?' checked="checked"':'').'>&nbsp;&nbsp;'.cfCaption('genSearch').'</div>';
	$output.='<input class="textInput" style="width:60%;margin-right:1em" id="searchInput" onkeyup="validateSearch()" autocorrect="off" autocapitalize="off" value="'.((cfRGetVar('groupBy')=='Search')?str_replace('%','',cfRGetVar('searchValue')):'').'"/>';
	$output.=outButton('','javascript:search(dgi(\'searchInput\').value)',outIcon('search'));
	$output.="</div>\n\n";

	/**
	 * Group by
	 */

	$output.=outDivFrame('frame2');
	$output.='<div class="frame2Header">'.cfCaption('genGroupBy').'</div>';

	// Group by artist
	$output.='<div id="groupByArtistDiv" eH=1 style="width:95%;cursor:pointer;" onclick="groupBy(\'Artist\');">';
	$output.='<input id="groupByArtist" type="radio" name="groupBy" value="Artist"'.((cfRGetVar('groupBy')=='Artist')?' checked="checked"':'').'>&nbsp;&nbsp;'.cfCaption('explorerAudioArtist').'</div>';

	// Group by album
	$output.='<div id="groupByAlbumIdDiv" eH=1 style="width:95%;cursor:pointer;" onclick="groupBy(\'Album\');">';
	$output.='<input id="groupByAlbum" type="radio" name="groupBy" value="Album"'.((cfRGetVar('groupBy')=='Album')?' checked="checked"':'').'>&nbsp;&nbsp;'.cfCaption('explorerAudioAlbum').'</div>';

	//$output.='<div id="groupByYearIdDiv" class="eTC" eH=1 style="width:95%;cursor:pointer;" onclick="groupBy(\'Year\');">';
	//$output.='<input id="groupByYear" type="radio" name="groupBy" value="Year"'.((cfRGetVar('groupBy')=='Year')?' checked="checked"':'').'>&nbsp;&nbsp;'.cfCaption('explorerAudioYear').'</div>';

	// Group by playlist
	$output.='<div id="groupByPlaylistDiv" eH=1 style="width:95%;cursor:pointer;" onclick="groupBy(\'Playlist\');">';
	$output.='<input id="groupByPlaylist" type="radio" name="groupBy" value="Playlist"'.((cfRGetVar('groupBy')=='Playlist')?' checked="checked"':'').'>&nbsp;&nbsp;'.cfCaption('explorerAudioPlaylist').'</div>';
	$output.="</div>\n\n";

	/**
	 * Display options
	 */
	$output.=outDivFrame('frame2');
	$output.='<div class="frame2Header">'.cfCaption('genDisplay').'</div>';

	// Items per page
	$output.='<div>'.cfCaption('explorerItemsPerPage');
	$output.='<select id="itemsPerPage" size="1" class="textInput" onChange="changeItemsPerPage(this.value)" style="margin-left:1em">';
	foreach ($itemsPerPageList as $value){
		$output.='<option value="'.$value.'" '.(($value==cfRGetVar('itemsPerPage'))?'selected="selected"':'').'>'.(($value==0)?cfCaption('genAll').' ':$value).'</option>';
	}
	$output.='</select></div><br/>';

	// Compact / Full / Cover display / Extended covers
	$output.='<div>'.cfCaption('genDisplay').'</div><center>';

	$displayTypes=array('displayCompact'=>'musicLayoutList.png','displayFull'=>'musicLayoutFull.png','displayCovers'=>'musicLayoutCovers.png','displayCoversExt'=>'musicLayoutCoversExt.png');

	$nb=0;
	foreach ($displayTypes as $dt=>$icon) {
		$output.=outBlockItem('displayType',outImage('/gfx/'.$icon),false,'padding:3px;margin:2px',array('onclick'=>'changeDisplayType(\''.$dt.'\')'),(cfRGetVar('displayType')==$dt)?'sel':'unsel');
		$nb++;
		//if($nb==2) $output.='<br>';
	}
	$output.='</center>';
	$output.="</div>\n\n";

	$output.="</div>\n\n";

	$output.="</div>\n\n";
	return $output;
}

/**
 * @desc refresh selection area content
 *
 */
function asyncRefreshDisplay(){
	switch (cfRGetVar('groupBy')) {
		// Search in Artists, Albums, Tracks
		case 'Search':
			cfRSetVar('searchNbResults',0);// Reset search counter
			// Exception for unknown artist or album : replace by empty string
			if(cfUTF8Decode(@$_POST['data2'])==cfCaption('explorerAudioUnknownArtist') || cfUTF8Decode(@$_POST['data2'])==cfCaption('explorerAudioUnknownAlbum')) $_POST['data2']='';
			// Clear results area
			echo cfAsyncXMLJSaction('if(dgi("selectionDiv")) dgi("selectionDiv").innerHTML="";');
			echo cfAsyncXMLJSaction('if(dgi("mainDiv")) dgi("mainDiv").innerHTML="";');

			// Search in artists
			$filter="WHERE lower(Artist) LIKE '".strtolower(dbEscapeString(((cfRGetVar('searchValue')==cfCaption('explorerAudioUnknownArtist'))?'':cfRGetVar('searchValue')),$db))."'";
			echo cfAsyncXMLJSaction('displayList('.groupByArtistData($filter).',0)');

			// Search in albums
			$filter="WHERE (lower(Artist) NOT LIKE '".strtolower(dbEscapeString(cfRGetVar('searchValue'),$db))."') AND (lower(Album) LIKE '".dbEscapeString(strtolower(((cfRGetVar('searchValue')==cfCaption('explorerAudioUnknownAlbum'))?'':cfRGetVar('searchValue'))),$db)."')";
			echo cfAsyncXMLJSaction('displayList('.groupByAlbumData($filter).',0)');

			// Search in tracks
			$filter="WHERE (lower(Artist) NOT LIKE '".dbEscapeString(strtolower(cfRGetVar('searchValue')),$db)."') AND (lower(Album) NOT LIKE '".dbEscapeString(strtolower(cfRGetVar('searchValue')),$db)."') AND (lower(Name) LIKE '".dbEscapeString(strtolower(cfRGetVar('searchValue')),$db)."')";
			echo cfAsyncXMLJSaction('displayList('.groupByTrackData($filter).',0)');

			if(cfRGetVar('searchNbResults')==0) echo cfAsyncXMLJSaction('displayNoResults()');

			break;
		case 'Artist':
			echo cfAsyncXMLJSaction('displayList('.groupByArtistData().',true)');
			break;
		case 'Album':
			echo cfAsyncXMLJSaction('displayList('.groupByAlbumData().',true)');
			break;
		case 'Year':
			break;
		case 'Playlist':
			echo cfAsyncXMLJSaction('displayList('.groupByPlaylistData().',true)');
			break;

		default:
			break;
	}

//echo microtime(true)-$a;
//dbDisplayTable('SELECT Album from tracks GROUP BY Album;');
//@dbExec('CREATE VIEW artAlb AS SELECT Artist, Album, count(id) AS nbTracks, (min(Artwork || PersistentID)) AS art FROM tracks GROUP BY Artist, Album;',$db);
//dbDisplayTable('SELECT * FROM artAlb');
//dbDisplayTable('SELECT Artist, sum(nbTracks) AS nbTracks2, count(Album) AS nbAlbums FROM artAlb WHERE lower(Artist) < \'benabar\' GROUP BY Artist ORDER BY lower(Artist) ASC;');
//dbExec('DROP VIEW artAlb',$db);
//dbDisplayTable('library');
//dbDisplayTable('playlistsTracks');
}


/**
 * @desc display Javascript functions
 * @return void
 *
 */
function displayJavascript(){
?>
<script type="text/javascript">
var seqId=0;
var supportsPlaylist=<?php echo (int)(cfBGetVar('playlistFormat'));?>;
var margins=<?php if(cfGetBrowser()=='ie') echo 20; else echo 25;?>;
var lPID="<?php echo cfRGetVar('Library Persistent ID');?>";
var coverTemplate="<?php echo cfExtImage('*resourceBasePath*/IMAGEPATH',cfRGetVar('fullviewCoverWidth'),cfRGetVar('fullviewCoverWidth'),true,0,array('utf8Decode'=>true),true);?>";
var coverTemplateLarge="<?php echo cfExtImage('*resourceBasePath*/IMAGEPATH',cfRGetVar('fullviewCoverExtWidth'),cfRGetVar('fullviewCoverExtWidth'),true,0,array('utf8Decode'=>true),true);?>";
var hmsCaption="<?php echo cfCaptionJS('javascriptHourMinSecFormat');?>";
var msCaption="<?php echo cfCaptionJS('javascriptMinSecFormat');?>";
var sourceType="<?php echo SOURCE_TYPE;?>";
var maxDragItems=<?php echo cfHiGetVar('maxDragItems',80);?>;
var searchValue;
var coverDefaultSrc='<?php echo dirname($_SERVER['PHP_SELF']);?>/../music2/cover.png';
var dAWW=<?php echo DEFAULT_AUDIO_WINDOW_WIDTH;?>, dAWH=<?php echo DEFAULT_AUDIO_WINDOW_HEIGHT;?>,fCW=<?php echo cfRGetVar('fullviewCoverWidth');?>;
</script>
<?php
	echo cfScriptLink('musicDB.js');
}
