<?php
/**
 * File viewer
 *
 * Images : view, slideshow, next, prev, fullSize
 * HTML : view webpage (and not source code)
 * text : inline view
 * other formats : download (Opera) or inline view if possible (IE)
 *
 * parameters : POST data1 (file directory),  data2 (file name),  data3 (="view"),  data4 (action (optional)),  data5 (parameters (optional))
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc insert flash mp3 player, and start playing (if autostart is set to true)
 *
 * @param string $completeFilename : complete file name of mp3 file
 * @param string $playerType : 'normal', 'compact', 'minimal', 'mobile'
 * @param string $autoplay : true : start playing when loaded, false don't
 * @param integer $width : player width
 * @param integer $height : player height
 * @param integer $format : if set, use this player (flash,qt,wmp). Note: flash inserts both flash and HTML 5 players. HTML 5 player if automatically used if flash is not installed
 * @param string $script: returned script for async initialisation of flash/html5 player. Cannot be returned into HTML as it doesn't work in asynchronous HTML insertion.
 */
function vfInsertAudioPlayer($completeFilename, $playerType='normal', $autoplay=false,$width=false,$height=false,$format=false, &$script=false){
	$html5Playlist=array(); // Array of external filenames to be used with HTML5 player (as it can't user playlists)

	if($format && ($format=='flash' || $format=='qt' || $format=='wmp')) {}
	// Detect format according to browser support
	elseif	(cfBGetVar('flash'))$format='flash';
	elseif	(cfBGetVar('qt'))	$format='qt';
	elseif	(cfBGetVar('wmp'))	$format='wmp';
	else	$format='flash';

	if($width===false)	$width=280; if($height===false)	$height=26;

 	// If file is a playlist, modify it's extension and/or name so /ext/audio.php script can detect flash player is being used and so convert m3u file into an XML format

 	// If file is not a playlist, convert it into single-file playlist
 	if(!$completeFilename) $externalFileName='/audio/emptyPlaylist.xml';
	elseif(cfFileExtension($completeFilename)!='m3u') {
		cfRSetVar('singleMP3', $completeFilename);
		$externalFileName='*singleMP3*.m3u'.(($format=='flash')?'.xml':'');
		// Get song data (label, duration)
		if(is_file($completeFilename)){ // From meta data if file
			require_once(INCLUDE_DIR.'explorerFunctions.php');
			$tags=efGetAudioInfo($completeFilename,false);
			cfRSetVar('singleMP3Label', cfUTF8Encode($tags['label']));
		}
		elseif (cfRGetVar('extAccessFunction')){ // Or from database
			$extAccessFunction=create_function('$completeFilename', cfRGetVar('extAccessFunction'));
			$tags=efGetAudioInfo($extAccessFunction($completeFilename),false);
		}
		$externalFileName=cfExtAudio($externalFileName, "audio/".cfRGetVar('outputBitRate')."/",'relative');

		// Store into HTML5 files playlist
		$html5Playlist[]=array(
			'url'=>cfExtAudio($completeFilename, "audio/".cfRGetVar('outputBitRate')."/",'relative'),
			'artist'=>@$tags['artist'],
			'album'=>@$tags['album'],
			'track'=>@$tags['track'],
			'duration'=>@$tags['length']/1000,
			'externalRef'=>cfBase64_url_encode(cfResourceRelativePath($completeFilename))
			);
	}
	// If it's a playlist, add a .xml to filename (if flash) so it's detected by /ext/audio.php and flash player as an xml playlist file
	else {
		// Build a list of audio files for HTML5 audio player
		if($format=='html5' || $format=='auto' || $format=='flash'){
			if($completeFilename=='*playlist*.m3u') $files=cfRGetVar('playlist');
			else if($completeFilename=='*tmp*playlist*.m3u') $files=cfRGetVar('playlistTmp');
			else if($completeFilename=='*empty*playlist*.m3u') $files=array();
			foreach ($files as $fileInfo){
				$externalFileName=cfExtAudio($fileInfo['completeFileName'], "audio/".cfRGetVar('outputBitRate')."/",'relative');

				// External reference, used to update song info in inline player frame
				if(@$fileInfo['externalRef']) $externalRef=$fileInfo['externalRef'];
				else $externalRef=cfBase64_url_encode(cfResourceRelativePath($fileInfo['completeFileName']));

				$html5Playlist[]=array('url'=>$externalFileName,'artist'=>$fileInfo['artist'],'album'=>$fileInfo['album'],'track'=>$fileInfo['track'],'duration'=>$fileInfo['length']/1000,'externalRef'=>$externalRef);
			}
		}
		$externalFileName=$completeFilename.(($format=='flash')?'.xml':'');
		// If file should be reencoded (downgraded bitrate), insert /audio/[bitrate] into external file name
		$externalFileName=cfExtAudio($externalFileName, "audio/".cfRGetVar('outputBitRate')."/",'relative','b64');
	}

	$playerCode='';

	/**
	 * Flash player
	 */
	if($format=='flash'){
		if($playerType=='normal') $player='player.swf';
		elseif($playerType=='compact') $player='playerSmall.swf';
		elseif($playerType=='mobile') $player='mAPlayer.swf';
		else $player='playerMin.swf';

		$args='&bufferSize='.((cfRGetVar('flashPlayerBufferLength'))?cfRGetVar('flashPlayerBufferLength'):'3').'&autoplay='.(($autoplay)?'1':'0').'&volume='.((cfRGetVar('flashPlayerDefaultVolume'))?cfRGetVar('flashPlayerDefaultVolume'):'70').'&browser='.cfGetBrowser().'&rnd='.rand();

		// Wrap flash player into a dedicated div. This will be used to async update flash player replacing its html code, and without touching the HTML5 audio player
		if(!cfIsAsync()) $playerCode.='<div id="flashPlayerAsyncOuterDiv">';

		// Audio Flash player code
		$playerCode.='<object type="application/x-shockwave-flash" data="/js/'.$player.'?file='.$externalFileName.$args.'" width="'.$width.'" height="'.$height.'" id="playerAudio">';
		$playerCode.='<param name="movie" value="/js/'.$player.'?file='.$externalFileName.$args.'">';
		$playerCode.='<param name="wmode" value="transparent">';
		$playerCode.='<embed type="application/x-shockwave-flash" ';
			$playerCode.='name="playerAudio" ';
			$playerCode.='id="playerAudioEmbed" ';
			$playerCode.='src="/js/'.$player.'?file='.$externalFileName.$args.'" ';
			$playerCode.='allowScriptAccess="sameDomain" ';
			$playerCode.='type="application/x-shockwave-flash" ';
			$playerCode.='wmode="transparent" ';
			$playerCode.='pluginspage="http'.((isset($_SERVER['HTTPS']))?'s':'').'://www.macromedia.com/go/getflashplayer"/>';
		$playerCode.='</object>';

		if(!cfIsAsync()) $playerCode.='</div>';

	/**
	 * HTML5 audio player (not visible, displayed only if flash is not supported), only sync display (async update is done by setting update javascript into $script)
	 */
		$forceHTML5AudioPlayer=cfHGetVar('forceHTML5AudioPlayer');
		// Utf8 encode fields
		foreach ($html5Playlist as $k=>$v) foreach ($v as $k2=>$v2)	$html5Playlist[$k][$k2]=cfUTF8Encode($v2);

		if(!cfIsAsync()){
			if(cfIsMobile()){
				$playerCode.='<style type="text/css">'.file_get_contents(cfAppDocRoot().'/themes/html5AudioPlayer.css').'</style>';
			}
			$playerCode.='<div id="playerAudioHtml5Div" style="width:'.$width.';height:'.$height.';display:none" class="'.((cfIsMobile())?'mobile':'').'">';
			$playerCode.='<audio id="playerAudioHtml5"></audio>';
			$playerCode.='<a href="javascript:html5AP.back()" id="html5AP_back"></a>';
			$playerCode.='<a href="javascript:html5AP.play()" id="html5AP_play"></a>';
			$playerCode.='<a href="javascript:html5AP.pause()" id="html5AP_pause"></a>';
			$playerCode.='<a href="javascript:html5AP.next()" id="html5AP_next"></a>';
			$playerCode.='<div id="html5AP_pbo" onclick="html5AP.pbClick(event)">';
			$playerCode.='<div id="html5AP_pload"></div>';
			$playerCode.='<div id="html5AP_ppos"></div>';
			$playerCode.='</div>';
			$playerCode.='<div id="html5AP_npos"></div>';
			$playerCode.='<div id="html5AP_ntot"></div>';
			$playerCode.='<div id="html5AP_text"></div>';
			$playerCode.='<div id="html5AP_vol" onmouseover="html5AP.volShow(1)" onmouseout="html5AP.volShow(0)" onclick="html5AP.volToggle()" class="aniF">';
			$playerCode.='<div id="html5AP_vol_po" onclick="html5AP.volClick(event)"><div id="html5AP_vol_progress" class="aniF"></div></div>';
			$playerCode.='</div>';
			$playerCode.='</div>';

			// Insert scripts managing flash/html5 players
			$playerCode.=cfScriptLink('html5AudioPlayer.js',true);
			if(!cfUGetVar('audioVolume')) cfUSetVar('audioVolume',70);
			$playerCode.='<script type="text/javascript">html5AP.init('.json_encode($html5Playlist).','.((int)$forceHTML5AudioPlayer).','.((int)cfUGetVar('audioVolume')).')</script>';
		}
		// Async update HTML5 audio player playlist
		else {
			$script='html5AP.init('.json_encode($html5Playlist).','.((int)$forceHTML5AudioPlayer).',-1);';
		}
	}
	/**
	 * Quicktime player
	 */
	elseif($format=='qt'){
		$playerCode ='<object id="playerVideo" classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" ';
   		$playerCode.='width="'.$width.'" height="'.$height.'" codebase="http'.((isset($_SERVER['HTTPS']))?'s':'').'://www.apple.com/qtactivex/qtplugin.cab#version=6,0,2,0"';
   		$playerCode.='align="middle" >';
		$playerCode.='<param name="src" value="'.$externalFileName.'"/>';
		$playerCode.='<param name="autoplay" value="true" /><param name="CONTROLLER" VALUE="true" /><param name="LOOP" VALUE="false" />';
		// Opera & Gecko parameters
		$playerCode.='<embed name="playerVideo" id="playerVideoEmbed" src="'.$externalFileName.'" width="'.$width.'" height="'.$height.'"';
		$playerCode.='pluginspage="http'.((isset($_SERVER['HTTPS']))?'s':'').'://www.apple.com/quicktime/download/" align="middle" autoplay="true"> </embed>';
		$playerCode.='</object>';
		if(!$externalFileName) $playerCode='';

	}
	/**
	 * Windows Media Player
	 */
	elseif($format=='wmp'){
		$playerCode='<object name="NSPlayPlayer " classid="CLSID:22D6F312-B0F6-11D0-94AB-0080C74C7E95" width="'.$width.'" height="'.$height.'"';
		$playerCode.='codebase="http'.((isset($_SERVER['HTTPS']))?'s':'').'://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=6,1,5,217"';
		$playerCode.='standby="'.cfUTF8Encode('Microsoft Windows Media Player components...').'" type="application/x-oleobject">';
		$playerCode.='<param name="AutoStart" value="1">';
		$playerCode.='<param name="filename" value="'.$externalFileName.'">';
		$playerCode.='<param name="WindowlessVideo" value="1">';
		$playerCode.='<param name="EnableContextMenu" value="1">';
		$playerCode.='<param name="ShowStatusBar" value="1">';
		$playerCode.='<param name="howAudioControls" value="1">';
		$playerCode.='<param name="ShowControls" value="1">';
		$playerCode.='<param name="ShowPositionControls" value="1">';
		$playerCode.='<param name="ShowDisplay" value="0">';
		$playerCode.='<param name="ShowTracker" value="0">';
		$playerCode.='<param name="ShowGotoBar" value="0">';
		$playerCode.='<param name="EnableTracker" value="1">';
		// Opera & Gecko parameters
		$playerCode.='<embed type="application/x-mplayer2" src="'.$externalFileName.'" width="'.$width.'" height="'.$height.'"';
		$playerCode.='ShowAudiocontrols="1" ShowStatusbar="1" ShowControls="1" Showdisplay="0" ShowGotoBar="0" AutoSize="0" ShowCaptioning="0" EnableTracker="0" ShowTracker="0" EnableContextMenu="0" ShowPositionControls="1">';
		$playerCode.='</embed></object>';
	}

	//cfDbg($playerCode,1);
	return $playerCode;
}

/**
 * Enter description here...
 *
 * @param string $completeFilename : path and file name of audio file to play
 * @param string $command : passed command :
 *				- firstLoad : display audio file info and player iframe. This command should only be received by main player frame
 *				- playFile : display audio player or send m3u file. This command should only be received by previously created iframe
 *				- change : change parameters (bitrate, player). This command should only be received by main player frame
 * @param string $extraData :
 * @return unknown
 */
function vfViewFileAudio($completeFilename, $command, $extraData=false){
	// set default parameters
	if(!cfRGetVar('audioPlayer')) {
		if(cfBGetVar('flash'))	cfRSetVar('audioPlayer','flash');
		elseif(cfBGetVar('wmp'))cfRSetVar('audioPlayer','wmp');
		elseif(cfBGetVar('qt')) cfRSetVar('audioPlayer','qt');
		else cfRSetVar('audioPlayer','external');
	}
	if(cfRGetVar('audioPlayer')=='embeded') cfRSetVar('audioPlayer','wmp'); // Backward compatibility
	if(!cfRGetVar('outputBitRate')) cfRSetVar('outputBitRate','original');
	if(!cfRGetVar('maxOutputBitRate')) cfRSetVar('maxOutputBitRate','original');

	/**
	 * Quicktime specific playlists (so far iPhone only...), display embed quicktime with qtnext parameters acting as playlist
	 */
	if(($completeFilename=='*playlist*.m3u' || $completeFilename=='*tmp*playlist*.m3u') && cfBGetVar('playlistFormat')=='qt'){
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		require_once(INCLUDE_DIR.'mobileExplorerFunctions.php');

		// iPhone rotation
		$_ENV['meta']='<meta name="viewport" id="viewport" content="width=device-width, height=device-height, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=no">';

		// Top controls: if sharedMode RVar is defined then it's an explorer resource => display explorer controls, else it's an audio database => display back button
		if(cfRGetVar('sharedMode')) $topControlsExtra=mefTopControls(); else $topControlsExtra=mfButton(false,'javascript:D.location.href=\''.(WEnv::res()->resourceBaseURL()).'\'',outIcon('back'));

		// Insert header and scripts
		mfInsertHead(array('topControlsExtra'=>$topControlsExtra));
		mefInsertScripts();
?>
<script type="text/javascript">
function updateOrientation(){
	if(W.orientation==undefined) return;
	if(Math.abs(orientation)==90) {
		dgn('viewport').setAttribute("content", "width=480, height=268, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='268px';
	}
	else {
		dgn('viewport').setAttribute("content", "width=320, height=416, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='416px';
	}
}
</script>
<div id="mainDiv">
<embed style="position:absolute;width:90%;height:40%;left:5%;top:20%" width="90%" height="40%" codebase="http://www.apple.com/qtactivex/qtplugin.cab" id="iPhonePlayer" autoplay="true" controller="false" loop="false"
<?php
		if($completeFilename=='*tmp*playlist*.m3u') $playlist=cfRGetVar('playlistTmp');
		elseif(cfRGetVar('shufflePlaylist')) $playlist=cfRGetVar('playlistShuffled');
		else $playlist=cfRGetVar('playlist');

		$nb=0;
		foreach ($playlist as $song){
			// external ref (returned by flash player to browser every time a new song is played ???)
			if(!isset($song['externalRef'])) $song['externalRef']= base64_encode(cfResourceRelativePath($song['completeFileName']));
			// URL to file - note : no bitrate calculation, as it will be done later by audio.php
			$externalFileName=cfExtAudio($song['completeFileName'],'','relative');
			// Create XML node
			if(!$nb) echo ' src="'.$externalFileName.'" ';
			else echo ' qtnext'.$nb.'="<'.$externalFileName.'> T<myself>"';
			$nb++;
		}
		cfLog('extAccess.php: audio playlist generated from internal playlist. '.$nb.' records generated',LOG_DBG);

?>></div>
<?php
		exit;
	}

	/**
	 * If player must be opened in a standalone window (so far iPhone only, single tracks)
	 */
	if(cfBGetVar('standaloneMediaPlayer')){
		// Byte-range request unsupported by iPhone => no reencoding
		/*
		if(cfRGetVar('maxOutputBitRate')!='original'){
			require_once(INCLUDE_DIR.'outputFunctions.php');
			outDisplayErrorPage('Cannot play reencoded mp3s on iPhone');
		}
		*/
		// Update time before client disconnection as no ping will be sent during file play
		cfPingUpdate('force',60*10);
		header('Location: '.cfExtAudio($completeFilename));
		exit;
	}

/*
 ***************************************************************************************************************************
 * Player IFRAME
 ***************************************************************************************************************************
 */

	// if display audio player command is received
	if($command=='playFile') {

		// Check file's existence, access rights and format
		if((($completeFilename=='*playlist*.m3u' || $completeFilename=='*tmp*playlist*.m3u') || cfFileRights($completeFilename,'download')) && cfRGetVar('fileViewAllowed') && substr(mimeType(cfFileExtension($completeFilename)),0,5)=='audio'){
			//if(cfRGetVar('audioPlayer')=='flash' && cfFileExtension($completeFilename)!='m3u' && cfFileExtension($completeFilename)!='mp3' && cfFileExtension($completeFilename)!='xml') cfRSetVar('audioPlayer','wmp');
			switch (cfRGetVar('audioPlayer')){
				case 'wmp':
				case 'flash':
				case 'qt':
					$_ENV['winNoBar']=true;
					cfInsertHEAD(true);
					echo '<body class="frame1" style="border:0;margin:0;padding:0;text-align:center;min-width:0px">';
					echo vfInsertAudioPlayer($completeFilename,'normal',true,false,false,cfRGetVar('audioPlayer'));
					if(cfRGetVar('audioPlayer')=='flash') echo '<script type="text/javascript">function flashCommand(cmd,args){}</script>';
					echo '</body></html>';
					break;
				case 'external':
					efSendM3UFile($completeFilename);
					break;
			}
			exit;
		}
		exit;
	}

/*
 ***************************************************************************************************************************
 * Change player / output bit rate command
 ***************************************************************************************************************************
 */

	// if a player-parameters update command is passed
	if($command=='change' && $extraData) {

		// Change audio player
		$audioPlayer=substr($extraData,0,strpos($extraData,';'));
		if($audioPlayer=='qt' || $audioPlayer=='wmp' || $audioPlayer=='flash' || $audioPlayer=='external') cfRSetVar('audioPlayer',$audioPlayer);
		// Change output bitrate
		$outputBitRate=substr($extraData,strpos($extraData,';')+1);
		if($outputBitRate=='original' || $outputBitRate=='128' || $outputBitRate=='64' || $outputBitRate=='32'){
		if(cfRGetVar('maxOutputBitRate')=='original' || ($outputBitRate!='original' && cfRGetVar('maxOutputBitRate')>=$outputBitRate))
			cfRSetVar('outputBitRate',$outputBitRate);
		}
	}

/*
 ***************************************************************************************************************************
 * Display song/playlist information page
 ***************************************************************************************************************************
 */

	/*
	 * HEAD and JS
 	 */
	cfInsertHEAD(false);
	?>
	<script type="text/javascript">
function change(){
var rs,cf=document.comForm;
	if(dgi('wmp').checked==true) rs='wmp;';
	if(dgi('qt').checked==true) rs='qt;';
	if(dgi('flash').checked==true) rs='flash;';
	if(dgi('external').checked==true) rs='external;';
	if(dgi('brOriginal') && dgi('brOriginal').selected==true) rs=rs+'original';
	if(dgi('br128') && dgi('br128').selected==true) rs=rs+'128';
	if(dgi('br64') && dgi('br64').selected==true) rs=rs+'64';
	if(dgi('br32').selected==true) rs=rs+'32';
	cf.data4.value='change';
	cf.data5.value=rs;
	cf.target='_self';
	cf.submit();
}
function init(){
	D.comForm.target="playerIFrame";
	D.comForm.submit();
<?php
	if($command=='firstLoad' && !cfHGetVar('noPopup')) echo "W.resizeTo(dgi('topDiv').offsetWidth+20,85+dgi('topDiv').offsetHeight);\n";// resize and center window
	elseif($command=='firstLoad') echo "winMe.resizeInnerTo(dgi('topDiv').offsetWidth+8,dgi('topDiv').offsetHeight+10);\n";// resize and center window
	elseif($command=='change' && !cfHGetVar('noPopup')) echo "W.resizeTo(dgi('topDiv').offsetWidth+20,50+dgi('topDiv').offsetHeight);";// resize height of window
	elseif($command=='change') echo "winMe.resizeInnerTo(dgi('topDiv').offsetWidth,dgi('topDiv').offsetHeight);";// resize height of window
?>
}
</script>
</head>
<?php
	/*
	 * BODY
 	 */
	echo '<body class="iframeBody" onload="init()" id="audioFrameBody" style="text-align:center;overflow:hidden">'."\n";

	// Standard com form
	outInsertStandardComForm("",cfUTF8Encode(cfDirName(cfResourceRelativePath($completeFilename))),cfUTF8Encode(basename($completeFilename)),'view','playFile');

	echo '<div class="frame1" id="topDiv" style="width:300px" width="300">';
	if($completeFilename!='*playlist*.m3u' && $completeFilename!='*tmp*playlist*.m3u') echo '<div class="frame1Header">'.cfUTF8Encode(cfFileWithoutExtension(basename($completeFilename)))."</div>\n";
	//
	// ID3 TAG display
	//
	if(cfFileExtension($completeFilename)!='m3u' && $id3=efGetAudioInfo($completeFilename)){
		echo '<div class="frame2">'."\n";
		echo "<div class=\"frame2Header\">".cfCaption('explorerAudioFileInfo')."</div>";

		echo "<b>".cfCaption('explorerAudioArtist').cfCaption('genSeparator')."</b>".((isset($id3['artist']))?cfUTF8Encode($id3['artist']):'?')."<br>\n";
		echo "<b>".cfCaption('explorerAudioAlbum').cfCaption('genSeparator')."</b>".((isset($id3['album']))?cfUTF8Encode($id3['album']):'?')."<br>\n";
		echo "<b>".cfCaption('explorerAudioTrack').cfCaption('genSeparator')."</b>".((isset($id3['track']))?cfUTF8Encode($id3['track']):'?')."<br>\n";
		echo "<b>".cfCaption('explorerAudioTitle').cfCaption('genSeparator')."</b>".((isset($id3['title']))?cfUTF8Encode($id3['title']):'?')."<br>\n";
		if(isset($id3['genre'])){
			if(is_int($id3['genre']) && $id3['genre']>=0 && $id3['genre']<148) echo "<b>".cfCaption('explorerAudioGenre').cfCaption('genSeparator')."</b>". cfUTF8Encode(id3_get_genre_name($id3['genre']))."<br>\n";
			else  echo "<b>".cfCaption('explorerAudioGenre').cfCaption('genSeparator')."</b>".cfUTF8Encode($id3['genre'])."<br>\n";
		}
		else echo "<b>".cfCaption('explorerAudioGenre').cfCaption('genSeparator')."</b> ?<br>\n";
		echo "<b>".cfCaption('explorerAudioYear').cfCaption('genSeparator')."</b>".((isset($id3['year']))?cfUTF8Encode($id3['year']):'?')."<br>\n";
		echo "<b>".cfCaption('genComments').cfCaption('genSeparator')."</b>".((isset($id3['comment']))?cfUTF8Encode($id3['comment']):'?')."<br>\n";
		echo "</div>";
	}

	//
	// PLAYLISTS (m3u files)
	//
	if(cfFileExtension($completeFilename)=='m3u'){
		echo '<div class="frame2">';
		echo '<div class="frame2Header">'.cfCaption('explorerAudioPlaylist')."</div><br/>\n";
		echo '<div style="overflow:scroll;height:15em;">';
		// Internal playlist
		if($completeFilename=='*playlist*.m3u' || $completeFilename=='*tmp*playlist*.m3u'){
			$i=1;
			// Shuffle playlist if needed
			if(cfRGetVar('shufflePlaylist')){
				$shuffle=cfRGetVar('playlist');
				shuffle($shuffle);
				cfRSetVar('playlistShuffled',$shuffle);
				unset($shuffle);
				foreach (cfRGetVar('playlistShuffled') as $value) echo cfUTF8Encode($i++).') '.cfUTF8Encode(cfStrTruncate($value['label'],42))."<br>\n";
			}
			else{
				foreach (cfRGetVar('playlist') as $value) echo cfUTF8Encode($i++).') '.cfUTF8Encode(cfStrTruncate($value['label'],42))."<br>\n";
			}
		}
		// existing m3u file : open and read playlist content
		elseif(($handle = fopen ($completeFilename, "r"))){
			$nbFilesInM3U=0;
			while (!feof ($handle)) {
				$m3uFile = trim(fgets($handle, 4096));
				if(substr($m3uFile,0,1)!="#"){
					$audioCompleteFileName=cfJoinPathFile(cfDirName($completeFilename),$m3uFile);
					if(file_exists($audioCompleteFileName) && efFileType($audioCompleteFileName)=='audio' && cfFileRights($audioCompleteFileName,'download')){
						$nbFilesInM3U++; echo cfUTF8Encode($nbFilesInM3U).') '.cfUTF8Encode(cfStrTruncate($m3uFile,42))."<br>\n";
					}
				}
  			}
  			fclose ($handle);
		}
		echo "</div></div>\n";
	}

	//
	// Display player and encoding options
	//
	if(!(cfRGetVar('audioHidePlayerOptions'))){
		echo '<div class="frame2">';
		// Audio player type (wmp or external)
		echo '<div class="frame2Header">'.cfCaption('explorerAudioPlayerOptions')."</div>\n";
		echo "<div style=\"font-weight:bold;\">".cfCaption('explorerAudioPlayer')."</div>";
		echo '<form action="">';
		echo '<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';

		if(cfBGetVar('flash')) echo '<input type="radio" id="flash" onclick="change()" name="player"'.((cfRGetVar('audioPlayer')=='flash')?(" checked"):("")).">\n". cfCaption('musicCfgFlash')."<br>\n";
		if(cfBGetVar('qt')) echo '<input type="radio" id="qt" onclick="change()" name="player"'.((cfRGetVar('audioPlayer')=='qt')?(" checked"):("")).">\nQuicktime<br>\n";
		if(cfBGetVar('wmp')) echo '<input type="radio" id="wmp" onclick="change()" name="player"'.((cfRGetVar('audioPlayer')=='wmp')?(" checked"):("")).">\n". cfCaption('musicCfgEmbeded')."<br>\n";
		if(cfBGetVar('externalM3U')) echo '<input type="radio" id="external" onclick="change()" name="player"'.((cfRGetVar('audioPlayer')=='external')?(" checked"):("")).">\n". cfCaption('musicCfgExternal')."<br>\n";
		echo '</form>';
		// Output encoding kbps
		echo '<span style="margin-right:1em; font-weight:bold;">'.cfCaption('explorerAudioEncode')."</span>\n";

		echo '<select name="encode" size="1" onChange="change()">';
		if(cfRGetVar('maxOutputBitRate')=='original') echo '<option id="brOriginal" value="original"'.((cfRGetVar('outputBitRate')=='original')?' selected':'').'>'.cfCaption('explorerAudioOriginal')."</option>\n";
		if(cfRGetVar('maxOutputBitRate')=='original' || cfRGetVar('maxOutputBitRate')>='128') echo '<option id="br128" value="128"'.((cfRGetVar('outputBitRate')=='128')?' selected':'').'>'.cfCaption('explorerAudio128')."</option>\n";
		if(cfRGetVar('maxOutputBitRate')=='original' || cfRGetVar('maxOutputBitRate')>='64') echo '<option id="br64" value="64"'.((cfRGetVar('outputBitRate')=='64')?' selected':'').'>'.cfCaption('explorerAudio64')."</option>\n";
		echo '<option id="br32" value="32"'.((cfRGetVar('outputBitRate')=='32')?' selected':'').'>'.cfCaption('explorerAudio32')."</option>\n";
		echo '</select>';
		echo "<br><br></div>\n";
	}
	// Player IFRAME
	$ph=array('wmp'=>50,'flash'=>28,'qt'=>28,'external'=>0);
	echo '<iframe name="playerIFrame" width="280px" height="'.$ph[cfRGetVar('audioPlayer')].'px" frameborder=0 scrolling="no"></iframe>';
	outKeysManager();
?>
</body>
</html>
<?php
	return true;
}

/**
 * @desc View images
 *
 * @param string $completeFilename : path and filename
 * @param string $command : action (optional) 'firstLoad', "next", "previous", "fullsize", "normalsize", "slideshow", "stopSlideshow","rotateX" where X is angle
 * @param string $extraData : screen size if "firstLoad", else window size. Format:widthxheight
 */
function vfViewFileImage($completeFilename, $command=false, $extraData=false){
	require(INCLUDE_DIR.'viewImageFunctions.php');
	exit;
}

/**
 * @desc view video files
 *
 * @param string $completeFilename : path and filename
 * @param string $command : action (optional) 'firstLoad', 'forceDivxPlugin', 'forceMediaPlayer'
 * @param string $extraData : screen size if "firstLoad", else window size. Format:widthxheight
 */
function vfViewFileVideo($completeFilename, $command=false, $extraData=false){
	require(INCLUDE_DIR.'viewVideoFunctions.php');
	vfvViewVideo($completeFilename,$command,$extraData);
	exit;
}

/**
 * @desc view text files
 *
 * @param string $completeFilename
 * @param string $command : "firstLoad" or false
 * @param mixed $extraData : unused
 */
function vfViewFileText($completeFilename, $command=false, $extraData=false){
	// First load of script : resize window and re-call script for displaying file
	if(false && $command=='firstLoad'){
		cfInsertHEAD(false);
		echo '<script language="javascript" type="text/javascript">';
		echo 'function resize(){';
		if(cfHGetVar('noPopup')) echo 'winMe.resizeTo(parent.dd.getWndW()*3/4,parent.dd.getWndH()*3/4); winMe.moveTo(parent.dd.getWndW()/8,parent.dd.getWndH()/8);'."\n";
		else echo 'window.resizeTo(screen.width*3/4,screen.height*3/4); window.moveTo(screen.width/8,screen.height/8);'."\n";
		echo 'document.comForm.submit();}';
		echo '</script>';
		echo '<body onload="javascript:resize()">';
		outInsertStandardComForm($_SERVER['PHP_SELF'],cfUTF8Encode(cfResourceRelativePath($completeFilename)),'','view','reload');
		outKeysManager();
		echo '</body></html>';
		exit;
	}


	// HTML : display page
	if(substr(cfFileExtension($completeFilename),0,3)=='htm' || (cfFileExtension($completeFilename)=='mht' && cfGetBrowser()=='ie')){
		header('Content-Type:'.efFileMimeType($completeFilename));
		header('Content-Disposition:inline; filename="'.cfUTF8Encode(basename($completeFilename))).'"';
		header('Content-Length: '.cfFileSize($completeFilename));		// Close session
		wSession_write_close();
		echo file_get_contents($completeFilename);
		exit;
	}
	// Insert head
	cfInsertHEAD(false);

	// Highlighted file types
	global $highlighted;
	if (isset($highlighted[cfFileExtension($completeFilename)])){
		require_once(INCLUDE_DIR.'Text/Highlighter.php');
	}
?>
<script type="text/javascript" language="javascript">
function keyPressed(e) {
	if (e.keyCode) keycode=e.keyCode; else keycode=e.which;
	if(keycode==27) winMe.closeMe()
}
//connect(W,'load',function(){D.body.style.overflow="auto"});

</script>
</head>
<body onKeyPress="keyPressed(event)" class="viewText" style="overflow:visible !important">
<div style="min-width:100%;min-height:100%;overflow:visible"><div style="padding:0px 5px 5px 5px">
<?php

	// Close session
	wSession_write_close();

	// Hightlighted language
	if (isset($highlighted[cfFileExtension($completeFilename)])){
		$hl =& Text_Highlighter::factory($highlighted[cfFileExtension($completeFilename)]);
		echo cfUTF8Encode($hl->highlight(file_get_contents($completeFilename)),false,false);
	}
	// Other : display as plain text
	elseif($fp=@fopen($completeFilename,'r')) {
		fclose($fp);
		echo '<pre>'.str_replace('<','&lt;',cfUTF8Encode(file_get_contents($completeFilename))).'</pre>';
	}
	echo '</div></div></body></html>';
	exit;
}

/**
 * @desc VIEW FILE IN A DEDICATED WINDOW
 * 		specific file types :
 * 		- audio (vfViewFileAudio)
 * 		- video (vfViewFileVideo)
 * 		- image (vfViewFileImage)
 * 		- text (vfViewFileText)
 * Note : parameters have previously been utf-8 decoded
 *
 * @param string $completeFilename : path and filename of file to view
 * @param string $data3
 * @param string $data4
 * @param string $data5
 */
function vfViewFile($completeFilename, $data3=false, $data4=false, $data5=false){
	$cfFileRightsOK=false;

	if(substr($completeFilename,0,14)=='*resSpecific*/' && cfRGetVar('extAccessFunction')) {
		$extAccessFunction = create_function('$completeFilename', cfRGetVar('extAccessFunction'));
		$externalFilename=$completeFilename;
		$completeFilename=$extAccessFunction($completeFilename);
		if($completeFilename) $cfFileRightsOK=true;
	}

	// Set close URL back to resource for frameless display
	$_ENV['winCloseURL']='/res/'.cfRGetVar('type').'/'.cfRGetVar('subType').'/'.cfRGetVar('baseFile');

	// if download/view not authorized
	if(($data3!='view' && $data3!='viewFullscreen') || !cfRGetVar('fileViewAllowed') || ($completeFilename!='*playlist*.m3u' && $completeFilename!='*tmp*playlist*.m3u' && (!$cfFileRightsOK && !cfFileRights($completeFilename,'download')))) {

		// Hide actual filepath
		if(isset($externalFilename)) $completeFilename=$externalFilename;
		else $completeFilename=str_replace('*resourceBasePath*','',cfResourceRelativePath($completeFilename));
		// Error page
		outDisplayErrorPage(cfCaption('explorerErrorView').'<br/>'.cfUTF8Encode($completeFilename),false,false);
		return false;
	}
	// Get file mime type
	$fileType=efFileType($completeFilename);

	// Correct mime type for text_highlighted files
	global $highlighted;
	$highlighted=array("c"=>"CPP","cpp"=>"CPP","css"=>"CSS","dtd"=>"DTD","java"=>"JAVA","js"=>"JAVASCRIPT","sql"=>"SQL","mysql"=>"MYSQL","perl"=>"PERL","php"=>"PHP","py"=>"PYTHON","ruby"=>"RUBY","xml"=>"XML","vbs"=>"VBSCRIPT","bas"=>"VBSCRIPT","ctl"=>"VBSCRIPT","cls"=>"VBSCRIPT","frm"=>"VBSCRIPT");
	if(isset($highlighted[cfFileExtension($completeFilename)])) $fileType='text';

	// Log
	//if($data4=='firstLoad') cfLog(cfUTF8Decode(cfCaption('logFileView',cfUTF8Encode($completeFilename),false,false,true)),LOG_DET);

	// Reset original name of *resSpecific* filenames
	if(isset($externalFilename)) $completeFilename=$externalFilename;

	/**
	 * Mobile devices: use specific scripts, except for some devices
	 */
	if(cfIsMobile()) require(INCLUDE_DIR.'mobileViewFunctions.php');
	if(cfIsMobile('iPhone','iPad','wii')){
		if($fileType=='image' && (cfBGetVar('OS')=='Android' /*|| cfBGetVar('OS')=='webOS'*/)){/* use regular script for webOS/Android images */}
		else return mvfViewFile($completeFilename, $data4, $data5, $fileType);
	}

	// Display file according to format
	switch($fileType){
		case 'audio':
			return vfViewFileAudio($completeFilename, $data4, $data5);
		break;
		case 'image':
			return vfViewFileImage($completeFilename, $data4, $data5);
		break;
		case 'video':
			if(cfIsMobile('wii')) return mvfViewFile($completeFilename, $data4, $data5, $fileType); // iPhone: go to mobile view
			return vfViewFileVideo($completeFilename, $data4, $data5);
		break;
		case 'text':
			cfLogContentAccess($completeFilename);
			return vfViewFileText($completeFilename, $data4, $data5);
		break;
		// Other files formats
		default:
			cfLogContentAccess($completeFilename);
			// JS file : treat as text file
			if(efFileMimeType($completeFilename)=='application/x-javascript') return vfViewFileText($completeFilename, $data4, $data5);
			if(cfFileExtension($completeFilename)=='url'){
				$c=file_get_contents($completeFilename);
				if(stripos($c,'url=')) {
					$c=substr($c,stripos($c,'url=')+4);
					if(strpos($c,"\n")) $c=trim(substr($c,0,strpos($c,"\n")));
					header('Location: '.$c);
					exit;
				}
			}

			header('Content-Type:'.efFileMimeType($completeFilename));
			header('Content-Disposition:inline; filename="'.cfUTF8Encode(basename($completeFilename))).'"';
			header('Content-Length: '.cfFileSize($completeFilename));
			header("Cache-Control: must-revalidate, post-check=0, pre-check=0, public");
			header("Expires: 0");
			wSession_write_close();
			cfStreamFile($completeFilename,false,'auto');
			if(cfIsWII()) outKeysManager();
			exit;
		}
	return false;
}
?>