<?php
/**
 * User Interface for new user creation
 *
 * Display a list of accounts type and once an account type is selected, display name & password form (except for Facebook friends account)
 * Result (account type, name, password) is sent back to uiUsers.php for actual user creation
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.0.0
 */

/**
 * @desc Return a list of all accounts types that may be created
 * @return array account type => title, caption, icon[, disabled]
 *
 */
function newUserGetAccountTypes(){
	require(INCLUDE_DIR.'facebookFunctions.php');

	$accounts=array(
		'singleUser'=>array(
			'title'=>cfCaption('userAccountTypeSingleUser'),
			'caption'=>cfCaption('userAccountTypeSingleUserDesc'),
			'icon'=>'/local/uiUsers/singleUser.png'
		),
		'anonymous'=>array(
			'title'=>cfCaption('userAccountTypeAnonymous'),
			'caption'=>cfCaption('userAccountTypeAnonymousDesc'),
			'icon'=>'/local/uiUsers/anonymous.png'
		),
		'list'=>array(
			'title'=>cfCaption('userAccountTypeList'),
			'caption'=>cfCaption('userAccountTypeListDesc'),
			'icon'=>'/local/uiUsers/list.png'
		),
		'named'=>array(
			'title'=>cfCaption('userAccountTypeNamed'),
			'caption'=>cfCaption('userAccountTypeNamedDesc'),
			'icon'=>'/local/uiUsers/named.png'
		),
		'facebookFriends'=>array(
			'title'=>cfCaption('userAccountTypeFacebook'),
			'caption'=>cfCaption('userAccountTypeFacebookDesc'),
			'icon'=>'/local/uiUsers/facebook.gif'
		),
	);

	// Search for an already existing facebookFriends account (only one is allowed)
	foreach (WEnv::userConfigs() as $user){
		if($user->getVar('accountType')=='facebookFriends'){$found=1;break;}
	}
	if(isset($found)) unset($accounts['facebookFriends']);

	// If facebook profile has not been defined, display warning and disable account
	elseif(!facebookAccountGet()){
		$accounts['facebookFriends']['disabled']=true;
		$accounts['facebookFriends']['caption'].='<div class="warning">'.outImageIcon('info2').cfCaption('facebookAccountConfigWarning').'</div>';
	}

	return $accounts;
}

if(cfRequireFunctionsEnd()) return;


require_once(INCLUDE_DIR.'outputFunctions.php');



/**
 ***************************************************************************************************************************
 * Go to Phase 2: name & password processing
 ***************************************************************************************************************************
 */
if(isset($_POST['accountType']) || cfIsAsync()){
	$newUserForm=true;
	require(dirname(__FILE__).'/password.php');
	exit;
}

// Retreive an array of accounts types
$accounts=newUserGetAccountTypes();


/**
 ***************************************************************************************************************************
 * Display user type form
 ***************************************************************************************************************************
 */
cfInsertHEAD(true,array('size'=>'500x455','title'=>cfCaption('userNewUser'),'icon'=>'new'));
?>
<body onload="init()" style="overflow:hidden">
<?php
echo outDivFrame('frame1','id="topDiv"');

foreach ($accounts as $type=>$account){
	$content=outImage($account['icon'],false,false,'float:left;margin-right:15px');
	$content.='<div class="bigFont" style="padding-bottom:10px;font-weight:bold">'.$account['title'].'</div>';
	$content.=$account['caption'];
	if(!isset($account['disabled']))
		echo outBlockItem('accountType',$content,'','display:block;padding:10px',array('onclick'=>'selectAccountType(\''.$type.'\')'));
	else
		echo outBlockItem('accountType',$content,'','display:block;padding:10px',array(),'disabled');
}

?>
<script type="text/javascript">
function init(){wl.webPopupAdjustHeight(0)}
function selectAccountType(at){wl.postData({'accountType':at})}
</script>
</div>
</body>
<?php
exit;