<?php
/**
 * New weezo version available message box
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

$updateHost=substr(DOWNLOAD_SITE,7);

$updateUri='/update.zip';

$_ENV['7zDecOutput']='';

// Initialize script data
require_once('localFunctions.php');
lfInit();

function updatePage($id,$text){
	if($id) echo '<script type="text/javascript">document.getElementById("'.$id.'").innerHTML="'.str_replace('"','\\"',$text).'"</script>';
	else echo '<script type="text/javascript">'.$text.'</script>';
	flush();
}

/**
 * @desc display update error page
 *
 */
function displayErrorPage($extraMessage=''){
	updatePage('mainCaption','<span class="warning">'.cfCaption('updateVersionError').$extraMessage.'</span>');
	updatePage('cancelButtonOuter',outButton(cfCaption('genCancel'),'javascript:wl.UICommand(\'close\')',outIcon('cancel'),false,false,'style="margin-bottom:5px"'));
	exit;
}



/**
 * Download update package into data/temp/update.zip/7z
 * @param $interactive: true to display all HTML stuff needed for package update web interface
 * @return downloaded package file name on success (exit with error message on failure)
 */
function downloadUpdatePackage($interactive){
	global $updateUri,$updateHost;

	set_time_limit(3600);

	// Destroy previous file
	if(file_exists(cfAppDataDir().'/temp/update.zip')) {
		if(!@unlink(cfAppDataDir().'/temp/update.zip')) {
			if(!$interactive) return false;
			displayErrorPage('(Cannot remove previous update package)');
		}
	}

	// Destroy previous file
	if(file_exists(cfAppDataDir().'/temp/update.7z')) {
		if(!@unlink(cfAppDataDir().'/temp/update.7z')) {
			if(!$interactive) return false;
			displayErrorPage('(Cannot remove previous update package)');
		}
	}

	// Check file's existence and size
	if(!isset($_POST['updateFile']) && isset($_GET['updateFile'])) $_POST['updateFile']=$_GET['updateFile'];
	if(isset($_POST['updateFile']) && eregi('^[0-9a-z?&=/\.]{1,100}$',$_POST['updateFile'])) $updateUri=$_POST['updateFile'];
	if($updateUri[0]!='/') $updateUri='/'.$updateUri;

	// Local downloaded file name
	$localUpdateFile=cfAppDataDir().'/temp/update.'.((strpos($updateUri,'zip'))?'zip':'7z');

	// Get HTTP response header (1024 first chars) to :
	// - follow redirections
	// - get file size from content-length header

	// Max 2 redirections
	$proceed=2;
	while($proceed){
		if(!($handle=cfSocketHTTPRequest('http://'.$updateHost.$updateUri,7,false,$foo1,$foo2,false,true))) displayErrorPage('(Cannot connect to download site)');
		$response='';
		while(strlen($response)<1023 && !feof($handle)) $response .= fread($handle, 1024);

		$pos=strpos($response,"\r\n\r\n");
		if($pos === false){
			fclose($handle);
			if(!$interactive) return false;
			displayErrorPage('(Cannot find download package)');
		}
		$header=substr($response,0,$pos);
		$response=substr($response,$pos+4);

		$headers = array();
		$lines = explode("\r\n", $header);
		$fSize=0;
		if(count($lines)==0) {
			if(!$interactive) return false;
			displayErrorPage('(Cannot find download package)');
		}

		// HTTP response code
		$responseCode=substr($lines[0],9,3);

		// Seek file size or redirection
		foreach($lines as $value){
			if(strtolower(substr($value,0,15))=='content-length:') $fSize=trim(substr($value,15));
			if(strtolower(substr($value,0,9))=='location:') $redir=trim(substr($value,9));
		}
		// If redirection
		if(isset($redir)) {
			$proceed--;
			if($proceed<0) displayErrorPage('(Too many redirections)');
			$updateUri=$redir;
			if(substr($redir,0,1)!='/') $updateHost='';
			if(substr($updateHost,0,7)=='http://') $updateHost=substr($updateHost,7);
			if(substr($updateUri,0,7)=='http://') $updateUri=substr($updateUri,7);
			unset($redir);
		}
		// ! 200 OK
		elseif($responseCode!='200') {
			if(!$interactive) return false;
			displayErrorPage('(Cannot find download package - '.$responseCode.')');
		}
		// No redirection and response code 200 OK
		else $proceed=0;
	}

	// Proceed Download
	$handlew = fopen ($localUpdateFile, "w");
	if(!$handlew) {fclose($handle);displayErrorPage('(Cannot open local temporary update file)');}
	$i=-1;
	fwrite($handlew,$response);
	$dlSize=strlen($response);
	while (!feof($handle)) {
		$response=fread($handle, 8192);
		fwrite($handlew,$response);
		$i++;$dlSize+=strlen($response);
		if($interactive && $i%9==0) updatePage(false,'uPB('.floor(100*$dlSize/$fSize).');');
	}
	fclose($handlew);
	fclose($handle);

	return $localUpdateFile;
}

/**
 * Decompress update package
 * @param $localUpdateFile: filename of compressed update package
 * @param $interactive: true to display all HTML stuff needed for package update web interface
 * @return string directory where files have been unpacked (or false on failure and non-interactive)
 */
function unpackUpdatePackage($localUpdateFile,$interactive){
	if($interactive){
		updatePage('mainCaption',cfCaption('updateVersionUnzip'));
		updatePage('progress','<br/>');
	}

	/**
	 * UNZIP / 7ZDec in /temp/update
	 */
	$i=0;
	$unzipOk=false;

	@mkdir(cfAppDataDir().'/temp/update/');
	$df=cfAppDataDir().'/temp/update/';

	// Try to open as zip archive
	if(cfFileExtension($localUpdateFile)=='zip' && ($zip = @zip_open($localUpdateFile))) {
		while ($zip_entry = zip_read($zip)) {
			$i++;
			if(!@$interactive) updatePage('progress',$i.' '.cfCaption('genFiles'));

			if (zip_entry_open($zip, $zip_entry, 'r')) {
				if(substr(zip_entry_name($zip_entry),-1)=='/') $dirname=$df.zip_entry_name($zip_entry);
				else $dirname=dirname($df.zip_entry_name($zip_entry)).'/';
				if(!is_dir($dirname)) cfMkdir($dirname);
				if(!is_dir($df.zip_entry_name($zip_entry)) && $handle=fopen($df.zip_entry_name($zip_entry),'w')){
					@fwrite($handle,zip_entry_read($zip_entry, zip_entry_filesize($zip_entry)));
					fclose($handle);
				}
				zip_entry_close($zip_entry);
			}
		}
		zip_close($zip);
		$unzipOk=true;
	}
	// If not zip archive, try to open with 7zDec
	elseif(cfFileExtension($localUpdateFile)=='7z'){
		$decExe=cfAppBinDir().'/7zDec.exe';
		// Download 7z decoder if not installed
		if(!file_exists($decExe)){
			if(function_exists('cfSocketHTTPRequest')) $decBin=cfSocketHTTPRequest('http://'.$updateHost.'/7zDec.exe'); else	$decBin=@file_get_contents('http://'.$updateHost.'/7zDec.exe');
			if(strlen($decBin)>20000 && strlen($decBin)<30000) {
				$decExe=cfAppDataDir().'/temp/7zDec.exe';
				@file_put_contents($decExe,$decBin);
				unset($decExe);
			}
			else {
				if($interactive) return false;
				displayErrorPage('(Cannot download decompacter)');
			}
		}
		// Launch 7z decoder
		if(file_exists($decExe)){
			/**
			 * @desc Update decompression progress from 7zDec output parsing
			 *
			 * @param string $buff
			 */
			function update7zProgress($buff){
				static $nb=0;
				str_replace("> Extracting",'',$buff,$rc);
				if($rc>0) {
					$nb+=$rc;
					if(@$_ENV['updateInteractive']) updatePage('progress',($nb).' '.cfCaption('genFiles'));
				}
				$_ENV['7zDecOutput'].=$buff;
			}
			chdir(cfAppDataDir().'/temp/update');

			// Unpack file and monitor through update7zProgress function
			$stdoutFunction=($interactive)?'update7zProgress':false;
			$_ENV['updateInteractive']=$interactive;

			cfStreamProc('"'.cfAppBinDir().'/7zDec.exe" e "'.$localUpdateFile.'"',array('stdout'=>$stdoutFunction),array('bufferSize'=>30,'haltOnSeq'=>'Ok'.chr(13).chr(10)));

			// Look in 7zDec output if decompression is successfull
			if(strpos($_ENV['7zDecOutput'],'Everything is Ok')!==false) $unzipOk=true;
		}
	}

	// destroy update package
	@unlink($localUpdateFile);

	// If file couldn't be unpacked
	if(!$unzipOk){
		if(!$interactive) return false;
		echo '<script type="text/javascript">dgi("mainCaption").innerHTML="'.cfCaption('updateVersionError').'"</script>';
		echo '<script type="text/javascript">wl.UICommand("close:")</script>';
		exit;
	}

	return cfAppDataDir().'/temp/update';
}

/**
 * @desc Install unpacked files
 *
 * @param string $unzipedPackageDirectory
 * @param bool $interactive:
 */
function installUpdatePackage($unzipedPackageDirectory, $interactive){
	// Extra update script
	if(file_exists($unzipedPackageDirectory.'/extraUpdate.php')) {
		require($unzipedPackageDirectory.'/extraUpdate.php');
		@unlink($unzipedPackageDirectory.'/extraUpdate.php');
	}

	/**
	 * Move unzipped files
	 */

	// If a new "Weezo Updater.exe" is in the downloaded package, replace old one
	if(file_exists($unzipedPackageDirectory.'/bin/Weezo Updater.exe')) {
		// Normal Weezo installation : Weezo Updater is located right into application data root dir (cfAppDataRootDir())
		if(file_exists(cfAppDataRootDir().'/Weezo Updater.exe')){
			if(!@unlink(cfAppDataRootDir().'/Weezo Updater.exe')) cfLog('Error updating "Weezo updater.exe"'); //die('Update failure: cannot delete file "'.cfAppDataRootDir().'/Weezo Updater.exe"<br>Download full installer to upgrade');
			@rename($unzipedPackageDirectory.'/bin/Weezo Updater.exe',cfAppDataRootDir().'/Weezo Updater.exe');
		}
		// Compact Weezo installation : Weezo Updater is located into bin directory
		else{
			if(!@unlink(cfAppBinDir().'/Weezo Updater.exe')) {
				// If update failed, retry in cfAppDataRootDir
				if(!@rename($unzipedPackageDirectory.'/bin/Weezo Updater.exe',cfAppDataRootDir().'/Weezo Updater.exe'))
					die('Update failure: cannot delete file "'.cfAppBinDir().'/Weezo Updater.exe"<br>Download full installer to upgrade');
			}
			else @rename($unzipedPackageDirectory.'/bin/Weezo Updater.exe',cfAppBinDir().'/Weezo Updater.exe');
		}
	}

	// Move www directory
	if(is_dir($unzipedPackageDirectory.'/www')) @cfCopyDir($unzipedPackageDirectory.'/www',cfAppDataRootDir().'/www',true);

	// Move data directory
	if(is_dir($unzipedPackageDirectory.'/data')) @cfCopyDir($unzipedPackageDirectory.'/data',cfAppDataRootDir().'/data',true);

	// If all files have been moved, just close window, don't reboot
	if(!count(glob($unzipedPackageDirectory.'/*'))){
		@unlink($unzipedPackageDirectory.'');
		if(!$interactive) return true;
		cfInsertHEAD();
		die('<body onload="wl.UICommand(\'close\')"></body>');
	}

	/**
	 * REBOOT
	 * If files must be written inside cfAppRootDir(),
	 * use Weezo Updater.exe (with admin rights) to complete installation
	 * (called by weezo.exe, as it couldn't work properly if lauched by apache)
	 */

	// Close window and relaunch Weezo
	updatePage('mainCaption',cfCaption('genRestarting'));
	updatePage('progress','<br/>');
	updatePage(false,'document.getElementById("cancelButton").className="buttonDisabled";');
	updatePage(false,'document.getElementById("cancelButton").href="#";');

	sleep(1);
	cfServerSendCommand('installUpdate'); // Note: cfExecFork(cfAppBinDir().'/weezoLauncher.exe','SW_HIDE','-installUpdate'); Doesn't work !!!!

	exit;
}

/**
 * @desc Display update web page, download, unpack and install update package
 *
 */
function startUpdateProcess(){
	// Display download page
	cfInsertHEAD(false,array('size'=>'440x165'));
	echo '<script type="text/javascript">function uPB(pc){progressSetPerc("pBar",pc);progressSetText("pBar",pc+"%")}</script></head>';
	echo '<body width="440" height="165" title="'.APPLICATION_NAME.'" oncontextmenu="return false;" oonload="document.proceedForm.submit()" style="overflow:hidden">';
	echo outDivFrame('frame2',false,'margin-top:10px');

	echo outImage(outIcon('installIcon'),false,false,'float:left; margin-right:1em');
	echo '<b>'.cfCaption('newVersionAvailable').'</b>';
	echo '<br/><br/><div id="mainCaption">'.cfCaption('updateVersionDownloading').'</div><br/>';
	echo '<div id="progress" style="text-align:center">';
	echo outProgressBar(0,'200px','0%','pBar');
	echo '</div><br/>';
	echo '<center id="cancelButtonOuter">'.outButton(cfCaption('genCancel'),'javascript:wl.UICommand(\'close\')',outIcon('cancel'),false,'cancelButton','style="margin-bottom:5px"').'</center><br/>';
	echo '<form name="proceedForm" method="POST" action="" style="display:none"><input name="doUpdate" value="download"></form>';
	echo '</div>';
	flush();


	/**
	 * Update update script
	 */
	if(isset($_GET['updateScript']) && $_GET['updateScript']!=substr(md5_file($_SERVER['SCRIPT_FILENAME']),0,3)){
		updatePage('mainCaption','Updating script...');
		// Download script
		if(function_exists('cfSocketHTTPRequest')) $scriptContent=cfSocketHTTPRequest('http://'.$updateHost.'/updateVersion.phps'); else	$scriptContent=@file_get_contents('http://'.$updateHost.'/updateVersion.phps');
		if(!strpos($scriptContent,'?>')) displayErrorPage('(Update script couldn\'t be downloaded)');
		// Save script
		file_put_contents(cfAppDocRoot().'/local/updateVersion.php',$scriptContent);
		updatePage('mainCaption','Script successfully updated');

		// Reload page (to use downloaded script)
		updatePage(false,'var nl=document.location.href;document.location=nl.replace(/updateScript/,"updateScriptOK")+"&noConfirmation=1"');
		exit;
	}

	// Download update package
	$updatePackageFile=downloadUpdatePackage(true);

	// Decompress update package
	$updatePackageDirectory=unpackUpdatePackage($updatePackageFile,true);

	// Move updated files and launch Weezo Updater.exe
	installUpdatePackage($updatePackageDirectory,true);
}


/**
 * Save preferences
 */
if(isset($_POST['savePrefs']) && ($_POST['savePrefs']=='auto' || $_POST['savePrefs']=='disabled')) cfServerSendCommand('changeParameter name="updateApplication" value="'.$_POST['savePrefs'].'"');


// Proceed to download and install
if(isset($_GET['noConfirmation']) || cfGGetVar('updateApplication')=='auto' || (isset($_POST['doUpdate']) && $_POST['doUpdate']=='download')) startUpdateProcess();


// Udate disabled : quit
if(cfGGetVar('updateApplication')=='disabled'){
	echo '<body onload="wl.UICommand(\'close\')"></body>';
	exit;
}
if(!cfGGetVar('updateApplication')) cfGSetVar('updateApplication','confirm');



/**
 * Download confirmation page
 */


// If other file is passed
$updateFile=false;
if(isset($_GET['updateFile']) && eregi('^[0-9a-z/.]*$',$_GET['updateFile'])) $updateFile=$_GET['updateFile'];

cfInsertHEAD(true,array('size'=>'440x165'));
echo '<body title="'.APPLICATION_NAME.'" oncontextmenu="return false;">';
echo outDivFrame('frame2',false,'margin-top:10px');

echo outImage(outIcon('info2'),false,false,'float:left; margin-right:1em');

// If new version is beta version, display beta icon
if(isset($_GET['beta'])) echo outImageIcon('beta');

// Changelog: link to beta site or main site changelog
if(isset($_GET['beta'])) $newsLink='http://dev.weezo.net'; else $newsLink='http://www.weezo.net/redirect.php?item=changelog';

// Update captions
echo cfCaption('newVersionAvailable');
echo '<br/><br/>'.cfCaption('updateVersionUpdate').'<br><br><br>';
echo '<form name="dontAskForm" method="POST" action="">';

// Save / Cancel buttons
	echo '<div style="position:relative; top:-4px; text-align:center; width:100%">'.outBt(cfCaption('genOK'),outIcon('ok16'),'javascript:dl()',false,false,false,'up',80);
	echo '<span style="padding-left:2em;"></span>'.outBt(cfCaption('genCancel'),outIcon('cancel'),'javascript:doCancel()',false,false,false,'up',80);
	echo '</div></div>';

	echo '<input type="hidden" name="savePrefs" style="display:none;" />';
	echo '<input type="hidden" name="doUpdate" style="display:none;" value=""/>';
	if($updateFile) echo '<input type="hidden" name="updateFile" style="display:none;" value="'.$updateFile.'"/>';
	// "Don't ask" Checkbox
	echo '<input type="checkbox" name="dontAsk" style="margin-top:3px; margin-right:1em;" /><span>'.cfCaption('dontAskAnymore').'</span>';

echo '<span style="position:absolute; right:1em"><a class="link" href="'.$newsLink.'" target="_blank">'.cfCaption('updateNews').'</a></span>';
echo '</form>';

?>
<script type="text/javascript" language="javascript">
function dl(){
	if(dgn("dontAsk").checked) dgn("savePrefs").value="auto";
	dgn('doUpdate').value="download";
	D.dontAskForm.submit();
}
function doCancel(){
	if(!dgn("dontAsk").checked) wl.UICommand('close')
	else {
		dgn("savePrefs").value="disabled";
		D.dontAskForm.submit();
	}
}
</script>
</body></html>