<?php
/**
 * LOGIN
 *
 * login controls : verify password, log user
 * DOESN'T DISPLAY LOGIN FORM:
 * theme login form is included by login.php
 *
 *
 * List of parameters sent to loginForm.php script
 * - $_ENV['users'][] : arrays of user accounts to be displayed (array key is user id)
 *		user data :
 * 			['authenticationMethod'] : user authentication method : 'noAuthentication' or 'password'
 * 			['name'] (optional) : user name. Set to false if both name and user Id must be entered
 * 			['icon'] : user icon file basename (add /gfx/icons/ to have full path)
 * 			['hint'] (optional) : password hint, to be displayed
 * - $userToDisplay () optional : id of selected user (set if an user name has been passed in get args or if a wrong password has been typed),
 * 			(set to 0 (zero) if login+password) input must be selected)
 * - $userAndPasswordMethod : true if login+password form has been used
 * - $wrongPasswordSent: true if a wrong password has been entered
 * - $wrongUserSent: requested user name if a non-existing user name has been entered
 * - $hiddenUsers: true if at least one hidden user account exists => user name+password form must be displayed
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */
require_once(INCLUDE_DIR.'initFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

define('LOGIN_TOKEN_LIFETIME', 60); // Login token lifetime, in seconds
define('LOGIN_TOKEN_CHECK_IP', false); // Set to true to check client connects with same IP that the IP used for connection to central authentication server (disabled)
$_ENV['subUserLogged']=false; // Set to subUserId if a subUser has successfully logged-in



$nbWrongPasswordTyped=0; // Global var, number of wrong password typed by an IP address in passwordCheckBanPeriod time interval
$userToDisplaySeqNumber=0; // Sequential user number on login page

/**
 * @desc function called on successful user login: load parameters and display left panel and 1st resource
 * 			if $userConfigFileName is provided, load user from filename
 * @param string $userId: user id
 * @param string $userConfigFileName (optional): user configuration file
 * @param bool $apiMode: set to true to just create user and session, and remove all HTTP/HTML wrapping
 * @return void
*/
function userLoad($userId,$userConfigFileName=false,$apiMode=false){
	if(!$userId && !$userConfigFileName) die('User ID not found');

	// Temp store session data as $_SESSION will be erased by wSession_start,
	// and some usefull data may already have been stored into (cfGSetVar)
	if(@$_SESSION) $tmp=$_SESSION;

	// Destroy previous session if existing (might have been started by /index.php)
	if(isset($_ENV['wSession'])) {
		wSession_write_close();
		unset($_ENV['wSession']);
	}
	if(isset($_COOKIE['WSESSID'])) unset($_COOKIE['WSESSID']);

	// Start session
	wSession_start(false,$apiMode);
	if(isset($tmp)) $_SESSION=$tmp;

	// Get configuration (to ensure user exists and retreive actual id)
	// Note: if $userId is 'standalone', $uc is properly generated for a standalone user
	$uc=new WUserConfig(($userId)?$userId:$userConfigFileName);
	if(!$uc->isValid()) die('User ID not found');

	// Set logged user id

	// If user id not passed, find user from filename
	$_SESSION['user']=$uc->getData();
	$userId=$_SESSION['userId']=$_SESSION['user']['id']=$uc->id();

	/**
	 * Last security check: check referer (XSRF protection)
	 */
	$headers=getallheaders();
	if(cfIPFilterIsset($_SESSION['user']) && isset($headers['Referer'])) {
		$refererURL=parse_url($headers['Referer']);

		if(isset($refererURL['host'])){
			// Allow redirection from "DNS" server
			if('http://'.$refererURL['host']!=DNS_SITE && $refererURL['host']!=cfGGetVar("externalIP")){
				// Else, get IP from host name
				if($refererURL['host']=='localhost') $refererURL['host']='127.0.0.1'; // prod
				if(!cfIsIP($refererURL['host'])) $refererURL['host']=cfGethostbyname($refererURL['host']);

				// If referer IP could not be retreived from referer hostname or referer IP is not allowed to connect this account, abort
				if(!cfIsIP($refererURL['host']) || !cfIPFilterCheck($_SESSION['user'],$refererURL['host'])){
					wSession_destroy();
					$message='Unauthorized redirection from '.$refererURL['host'];
					if(cfIsAsync())	die(cfAsyncHeader().cfAsyncJSaction("alert('".$message."')").cfAsyncFooter(false));
					die($message);
				}
			}
		}
	}

	// (re) set browser as it might have been erased during centralized authentication process
	if(!cfGGetVar('browserCaps')) cfSetBrowserCaps();

	// Mobile login
	if(@$_POST['mobile']) cfGSetVar('forceMobile',1);


	// Server port (override general.ini value as alternatePort may have been used)
	if(@$_SERVER['SERVER_PORT']){
		cfGSetVar('serverPort',$_SERVER['SERVER_PORT']);
		cfGSetVar('protocol',($_SERVER['SERVER_PORT']==443)?'https':'http');
	}

	// Standalone resource (published resource or direct resource access) integrate publishToken
	if($userId==='standalone') {
		if(isset($_GET['directResourceAccessToken'])){
			$_SESSION['user']['directResourceAccessToken']=$_GET['directResourceAccessToken'];
			// Also set in a cookie so if (weezo) session ends, user automatically re-logs
			if(!$apiMode) setcookie('weezoDirectResourceAccessToken',$_GET['directResourceAccessToken'],(time()+3600));
		}
		else{
			$_SESSION['user']['publishToken']=$_GET['publishToken'];
			// Also set in a cookie so if (weezo) session ends, user automatically re-logs
			if(!$apiMode) setcookie('weezoPublishToken',$_GET['publishToken'],(time()+3600));
		}
	}

	// Load user resource info
	$nbResourcesLoaded=ifLoadUserResources();

	$_SESSION['accountIP']=$_SERVER['REMOTE_ADDR'];
	$_SESSION['activeResourceId']=0;

	// If account type is "list", store subUser Id
	if($_ENV['subUserLogged']) cfUSetVar('subUserId',$_ENV['subUserLogged']);

	// Misc commands processing (not appliable for API)
	if(!$apiMode) {
		// If user has requested a specific theme, replace user theme
		if(@$_POST['userSelectedTheme'] && cfGGetVar('allowUserThemeChange'))
			cfUSetVar('theme',cfUTF8Decode($_POST['userSelectedTheme']));

		// Get client screen size
		if (isset($_POST['userScreenWidth'])) cfGSetVar('userScreenWidth',$_POST['userScreenWidth']);
		if (isset($_POST['userScreenHeight'])) cfGSetVar('userScreenHeight',$_POST['userScreenHeight']);

		// Get client flash support
		if(cfBGetVar('flash')==='auto' && isset($_POST['userHasFlash'])) cfBSetVar('flash',$_POST['userHasFlash']);

		// If a resource is requested as 1st page startup
		if(isset($_GET['res'])){
			foreach (cfResourcesGetUser() as $k=>$v) if(isset($v['name']) && $v['name']==urldecode($_GET['res'])){
				cfGSetVar('resourceDisplayAtStartup',$k);
				break;
			}
		}

		// else if mobile device, use mobile theme
		if (cfIsMobile()) {
			if(!cfGGetVar('mobileTheme') || !file_exists(cfAppDocRoot().'/themes/mobile/'.cfGGetVar('mobileTheme'))) cfGSetVar('mobileTheme','b'); // Default
			cfGSetVar('theme','mobile/'.cfGGetVar('mobileTheme'));
			if(cfBGetVar('theme')) cfGSetVar('mobileBrowserTheme',cfBGetVar('theme'));
			cfUSetVar('theme',false); // Reset user-defined theme so it doesn't overrides mobile theme
		}
		// If browser has a dedicated theme, apply it and forbid further changes
		elseif(cfBGetVar('theme')) {
			cfGSetVar('theme',cfBGetVar('theme'));
			cfGSetVar('allowUserThemeChange',false);
		}

		if(!cfBGetVar('frames')) cfGSetVar('noPopup',true);

		// Get server (this) host name
		if (isset($_POST['serverExternalHost'])) cfGSetVar('hostName',((isset($_SERVER['HTTPS']))?'https':'http').'://'.$_POST['serverExternalHost']);

		// Disable IP <-> Session control for mobile devices
		if(cfIsMobile()) cfGSetVar('disableClientSessionIPControl',true);
	}

	// Get visitor's host name
	if(cfGGetVar('getVisitorHostName')){if($host=@gethostbyaddr($_SERVER['REMOTE_ADDR'])==$_SERVER['REMOTE_ADDR']) $host='';}	else $host='';
	cfUSetVar('host',$host);

	// If single user account type, verify that this user is not already logged
	$nbConnected=0;
	$maxConnected=cfUGetVar('maxSimultaneousUsers');
	if(cfUGetVar('accountType')=='singleUser') $maxConnected=1;
	if($maxConnected){
		$sessionFiles=glob(cfAppDataDir().'/sessiondata/sess_*');
		// Browse sessions
		foreach ($sessionFiles as $filename) if(substr($filename,5)!=wSession_id()){
			if(cfGGetVar('sessionHandler')=='memory') $sessData=cfUnserializeSession(@cfMGetVar('weezoS'.substr(basename($filename),1),true));
			else $sessData=cfUnserializeSession(file_get_contents($filename));
			if(isset($sessData['user']['id']) && $sessData['user']['id']==cfUGetVar('id')) {
				$nbConnected++;
				// If single user and loginDisconnectPreviousUser, destroy previous connection
				if(cfUGetVar('accountType')=='singleUser' && cfGGetVar('loginDisconnectPreviousOnConflict')){
					if(cfGGetVar('sessionHandler')=='memory') @cfMUnsetVar('weezoS'.substr(basename($filename),1));
					else @unlink($filename);
					break;
				}
				// If more than max connected users, abort
				if($nbConnected>=$maxConnected){
					if($apiMode) return false;
					cfAsyncHeader();
					if($maxConnected==1) echo cfAsyncXMLJSaction('alert("'.cfCaption('loginAlreadyConnected',cfUTF8Encode(cfUGetVar('name'))).'")');
					else echo cfAsyncXMLJSaction('alert("'.cfCaptionJS('loginMaxUsersReached').'")');
					echo cfAsyncXMLJSaction('W.location.reload()');
					echo cfAsyncFooter(false);
					wSession_destroy();
					exit;
				}
			}
		}
	}

	// Set user language
	// First use preference's language from single users
	if(cfUGetVar('accountType')=='singleUser' && cfMIssetVar('weezoLng'.cfUGetVar('language'))) cfGSetVar('language',cfUGetVar('language'));

	// Facebook login: build pseudo from profile. Note: Facebook userPseudo includes "FBID"+id+":"+name
	if(cfUGetVar('authenticationMethod')=='facebook' && @$_ENV['userPseudo']){
		cfUSetVar('pseudo',cfUTF8Decode($_ENV['userPseudo']));
	}

	// Check if user name/pseudo must be requested
	if(cfUGetVar('accountType')=='named'){
		// If pseudo passed to API
		if(isset($_ENV['userPseudo']) && $_ENV['userPseudo']) cfUSetVar('pseudo',cfUTF8Decode($_ENV['userPseudo']));
		// If pseudo passed as POST var
		if(isset($_POST['userPseudo']) && $_POST['userPseudo'])	cfUSetVar('pseudo',cfUTF8Decode($_POST['userPseudo']));
		// If pseudo passed as GET var
		elseif(isset($_GET['userPseudo']) && $_GET['userPseudo']) cfUSetVar('pseudo',cfUTF8Decode($_GET['userPseudo']));
		else{
			cfGSetVar('requestUserPseudo',true);
			// Redirect to login page
			if(cfIsAsync()){
				echo cfAsyncHeader();
				echo cfAsyncXMLJSaction('if(wl.mask) wl.mask.hide()');
				echo cfAsyncXMLJSaction('loginPseudoGet("'.addslashes(cfUTF8Encode(cfUGetVar('name'))).'")');
				echo cfAsyncFooter(false);
			}
			else {
?>
<body onload="document.connectForm.submit()">
<form name="connectForm" action="/index.php" method="GET" enctype="multipart/form-data" style="display:none"></form>
</body>
<?php
			}
			exit;
		}
	}
	// Set user as logged
	$_SESSION['userLogged']=true;

	// send info to application and display main page
	loginFinish($apiMode);

	// API mode: return session id
	if($apiMode) return wSession_id();
}

/**
 * @desc finish login process
 * 		- Send info to application
 * 		- Log connection to database
 * 		- Load main page
 *
 * @param boolean $apiMode
 *
 */
function loginFinish($apiMode=false){
	// Set user as logged
	$_SESSION['userLogged']=true;
	$_SESSION['connectionTime']=time();
	cfGSetVar('connectionTime',time());

	// If direct resource access, set resource name as user name, for display in activity panel
	if(cfUGetVar('id')=='standalone' && !cfUGetVar('name')){
		$res=new WResConfig(@$_SESSION['res'][0]['id']);
		cfUSetVar('name',@$res->name());
		cfUSetVar('icon','blankIcon.gif');
	}

	// Write connection in log and activity panel
	if(cfUGetVar('id')=='standalone')
		cfLog(cfUTF8Decode(cfCaption('msgNewConnection').cfCaption('genSeparator').cfUTF8Encode(cfUGetVar('name'))),LOG_INF,true,LOG_EVENT_LOGIN);
	else if(cfUGetVar('authenticationMethod')=='list')
		cfLog(cfUTF8Decode(cfCaption('loginNewConnection',cfUTF8Encode(cfUGetVar('name')).cfCaption('genSeparator').cfUTF8Encode(WEnv::user()->subUser()->name()),false,false,true)),LOG_INF,true,LOG_EVENT_LOGIN);
	else if(cfUGetVar('authenticationMethod')=='facebook')
		cfLog(cfUTF8Decode(cfCaption('loginNewConnection',cfUTF8Encode(cfUGetVar('name')).' - '.substr(cfUGetVar('pseudo'),strpos(cfUGetVar('pseudo'),':')+1),false,false,true)),LOG_INF,true,LOG_EVENT_LOGIN);
	else
		cfLog(cfUTF8Decode(cfCaption('loginNewConnection',cfUTF8Encode(cfUGetVar('name').((cfUGetVar('pseudo'))?' - '.cfUGetVar('pseudo'):'')),false,false,true)),LOG_INF,true,LOG_EVENT_LOGIN);


	// List user's chat resources, and check if user has webcam
	$chat=''; $webcam=false;
	foreach (WEnv::user()->getResources() as $rid=>$rfilename){
		if($res=cfMGetVar('weezoResData'.$rid)){
			if($res['type']=='webcam' && $res['subType']=='chat') $chat.=(($chat)?',':'').$rid.'/'.	$rfilename;
			if($res['type']=='webcam' && $res['subType']=='std') $webcam=true;
		}
	}


	// Log information in statistics database (note: this step must be done before calling usersLogged()->addCurrentUser() because db row id is required
	if(!cfUGetVar('invisible')) {
		require_once(INCLUDE_DIR.'databaseFunctions.php');
		dbLogNewConnection();
	}

	// Restore user customized configuration
	loginRestoreCustomConfig();

	/**
	 * Add current user to logged users list,
	 */
	WEnv::usersLogged()->addCurrentUser();
	if(cfUGetVar('accountType')=='list') $subUserName=str_replace('"','',WEnv::user()->subUser()->name()); else $subUserName='';

	// inform application that a new user logged in
	$logString='connect sessionId="'.wSession_id().
		'" name="'.str_replace('"','',cfUGetVar('name')).
		'" id="'.cfUGetVar('id').
		'" pseudo="'.str_replace('"','',((cfCmpLeft(cfUGetVar('pseudo'),'FBID'))?substr(cfUGetVar('pseudo'),strpos(cfUGetVar('pseudo'),':')+1):cfUGetVar('pseudo'))).
		'" subUserName="'.$subUserName.
		'" icon="'.cfUGetVar('icon').
		'" IP="'.$_SESSION['accountIP'].
		'" filename="'.basename(cfUGetVar('configFilename')).
		'" host="'.cfUGetVar('host').
		'" audioProfile="'.cfUGetVar('audioProfile').
		'" browser="'.cfBGetVar('name').'"';
	if($chat) $logString.=' chatResourceFilenames="'.$chat.'"';
	if($webcam) $logString.=' needWebcam="true"';

	cfServerSendCommand($logString);

	// Monitor event
	if(!cfUGetVar('invisible')) cfLogEvent(cfCaption('loginNewConnection',cfUTF8Encode(cfUGetVar('name')),false,false,true),EVENT_CONNECTION,S_EVENT_LOGIN,cfUGetVar('id'));

	// API mode: we're done!
	if($apiMode) return;




	/**
	 * Redirect to main page
	 */

	// Async redirect
	if(cfIsAsync()){
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('if(wl.mask) wl.mask.hide()');
		// Send login cookie to Wii
		if(cfIsWII()){
			echo cfAsyncXMLJSaction('var exp=new Date();exp.setTime(exp.getTime() + (365*24*3600000));document.cookie="weezoUser='.cfUGetVar('id').'; expires="+exp.toGMTString();');
			if(cfUGetVar('authenticationMethod')=='password')
				echo cfAsyncXMLJSaction('var exp=new Date();exp.setTime(exp.getTime() + (365*24*3600000));document.cookie="weezoAuth='.sha1(cfUGetVar('password')).'; expires="+exp.toGMTString();');
		}

		// Manually set the cookie to prevent issues in some cases where cookie is not received by browser when displayed in a frame
		foreach (headers_list() as $v) if(cfCmpLeft($v,'Set-Cookie:')){
			$cookie=trim(substr($v,11));
			echo cfAsyncXMLJSaction('D.cookie="'.$cookie.'"');
		}
		//echo cfAsyncXMLJSaction('wl.cookieSet("WSESSID","'.wSession_id().'")');

		// Do connect
		echo cfAsyncXMLJSaction('loginConnect("'.wSession_id().'")');
		echo cfAsyncFooter(false);
	}
	// Sync redirect (when???)
	else {
		// Set URL where user will be redirected
		$connectedURL='/index.php';

		// If user is re-login to a published which session has terminated, redirect to page he was
		if(isset($_ENV['publishTokenBeforeLogoutURI'])) $connectedURL=$_ENV['publishTokenBeforeLogoutURI'];
		if(isset($_ENV['directAccessTokenBeforeLogoutURI'])) $connectedURL=$_ENV['directAccessTokenBeforeLogoutURI'];
?>
<head>
<script type="text/javascript">
function doLog(){
	var D=document,f=D.connectForm;
<?php
if(!cfGGetVar('userScreenWidth')) {?>f.userScreenWidth.value=screen.width;f.userScreenHeight.value=screen.height; <?php }
if(!cfGGetVar('hostName')) {?>f.serverExternalHost.value=D.location.host; <?php }
?>
	f.submit();
}
</script>
</head><body onload="doLog()">
<form name="connectForm" action="<?php echo $connectedURL; ?>" method="GET" enctype="multipart/form-data" style="display:none">
<?php
		$out='';
// IE Workaround: cookie sometimes cannot be set at this step (when using login token)
// => pass session id in GET so cookie is set at next page display (by security.php)
		if(!isset($_COOKIE) || !count($_COOKIE)) $out.='<input name="WSESSID" value="'.wSession_id().'">';
		if(!cfIsWII()) $out.='<input name="loginOK" value="true">';
		if(!cfGGetVar('userScreenWidth')) $out.='<input name="userScreenWidth" value=""><input name="userScreenHeight">';
		if(!cfGGetVar('hostName')) $out.='<input type="text" name="serverExternalHost" value="">';
		$out.='<input type="submit" value="go">';
		echo $out.'</form></body>';
	}
	wSession_write_close();

	exit;
}

/**
 * @desc finish loggin users who need name/pseudo
 *
 */
function userLoadNamedUser(){
	// User canceled
	if($_POST['userPseudo']=='*weezoCancel*'||$_POST['userPseudo']=='undefined') {
		wSession_destroy();
		if(isset($_POST['asyncRequest'])){
			echo cfAsyncHeader();
			echo cfAsyncXMLJSaction('D.connectForm.method="POST";loginConnect()');
			cfAsyncFooter();

		}
		unset($_POST);
		return;
	}
	cfUSetVar('pseudo',cfUTF8Decode($_POST['userPseudo']));
	cfGSetVar('requestUserPseudo',false);
	// send info to application and display main page
	loginFinish();
}

/**
 * @return string ID or false if not found
 * @param string $name : user name
 * @desc Return user id (number) from user name. Return -1 if not found
*/
function userIdFromName($name){
	foreach ($_ENV['users'] as $key=>$value) if($value['name']==$name) return $key;
	return false;
}

/**
 * @return bool : true if OK
 * @param string $userId : user id
 * @param string $password : password
 * @desc verify $userId and $password. Return true if OK, false if not OK (in this case, wait for a few seconds)
*/
function verifyUserPassword($userId, $password){
	global $wrongPasswordSent;
	global $securityEvent;

	$aM=$_ENV['users'][$userId]['authenticationMethod'];

	//if user's authentication method is password, user list or facebook (facebook access_token is passed as a password)
	if($aM=='password'||$aM=='facebook'||$aM=='list'){
		// Decrypt RSA crypted password

		$decrypted=cfRSADecrypt($password,$wasCiphered);
		// Parse return string
		// Return string (decrypt) format : ip=XXX/password/ts=YYY/
		// where XXX is iptolong ip address and YYY is timestamp
		if($wasCiphered){
			$timestamp=substr($decrypted,strrpos($decrypted,'/ts=')+4);$timestamp=substr($timestamp,0,strpos($timestamp,'/'));
			$longip=substr($decrypted,strpos($decrypted,'ip=')+3);$longip=substr($longip,0,strpos($longip,'/'));
			$password=substr($decrypted,strpos($decrypted,'/',strpos($decrypted,'ip='))+1);$password=substr($password,0,strrpos($password,'/ts='));

			// Check client ip address matches with ciphered IP
			if($_SERVER['REMOTE_ADDR']!=long2ip($longip) && !cfGGetVar('disableClientSessionIPControl')){
				$securityEvent.='Password check: incorrect client IP address error: '.$_SERVER['REMOTE_ADDR'].'<>'.long2ip($longip);
				setWrongPasswordTyped();
				return false;
			}
			// Check timestamp (allow a 60 seconds difference (default) to cover transmission and ciphering time
			if(abs(((int)microtime(true))-$timestamp)>((cfGGetVar('RSAAuthorizedDelay'))?cfGGetVar('RSAAuthorizedDelay'):20)){
				$securityEvent.='Password check: excessive time shift error<br>Modify "RSAAuthorizedDelay" value in data/general.ini';
				return false;
			}
		}


		// If password is sent unencrypted but encryption was required, refuse
		// Note: facebook access_token may come unciphered when redirected from weezo.net (mobile facebook login)
		if (0 /* !$wasCiphered && cfGGetVar('protocol')=='http' &&
				cfGGetVar('encryptPassword') && (!cfGGetVar('disablePasswordCipheringForSlowJSDevices') || !cfBGetVar('slowJS'))*/){
			cfLog('login.php: unencrypted password sent. Connection refused',LOG_ERROR);
			$securityEvent.='Password check: encrypted password not found';
			return false;
		}

		/**
		 * Regular password authentication: check password's md5
		 */
		if($aM=='password' || $aM=='list'){
			// List login: split password into subUser name / password, check for user then for password
			if($aM=='list'){
				if(strpos($password,'*wSep*')){
					// Extract subUser name / password
					list($subUserName,$password)=explode('*wSep*',$password,2);
					$subUserName=cfUTF8Decode($subUserName,true,true,false,128);

					// Search for subUser into group
					$uc=new WUserConfig($userId);

					foreach ($uc->getSubUserNames() as $sUId=>$sUName) if($subUserName===$sUName){
						$subUserNameFound=1;
						// Check subUser password
						$subUser=new WSubUser($sUId);
						// Password OK

						if($subUser->checkPassword($password)){
							// Log in subUser!
							setRightPasswordTyped();
							$_ENV['subUserLogged']=$sUId;
							return true;
						}
						break;
					}
				}

				// Wrong password or wrong subUser name
				setWrongPasswordTyped();
				sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));
				if(isset($subUserNameFound))
					cfLog('login.php: wrong password typed for '.$_ENV['users'][$userId]['name'].' group',LOG_DBG);
				else
					cfLog('login.php: wrong user name typed for '.$_ENV['users'][$userId]['name'].' group',LOG_DBG);

				$wrongPasswordSent=true;
				return false;
			}
			// Correct password typed
			if(cfHashedPasswordCheck($password,$_ENV['users'][$userId]['password'])){
				setRightPasswordTyped();
				return true;
			}
			// Wrong password typed
			setWrongPasswordTyped();
			sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));
			cfLog('login.php: wrong password typed for '.$_ENV['users'][$userId]['name'].' account',LOG_DBG);
			$wrongPasswordSent=true;
			return false;
		}

		/**
		 * Facebook access_token authentication: connect to facebook to check wether it's a friend or not
		 */
		if($aM=='facebook'){
			require_once(INCLUDE_DIR.'facebookFunctions.php');
			$wrongPasswordSent=true; // Default behaviour for processing incident (that should theorically not happend): treat as wrong password

			// If Facebook profile is not defined, reject Facebook logins (should not happend)
			if(!($profile=facebookAccountGet())) return false;

			// Retreive connected user's Facebook profile using transmitted access_token ($password)
			$userProfile=facebookAPIRequest('me',false,false,false,$password);

			// If an error occured, reject connection
			if(!isset($userProfile['id'])) return false;

			// If same user, it's OK
			if($profile['id']===$userProfile['id']){
				$userProfile=$profile;
			}
			// Check if user and registered are friends
			else{
				$res=facebookOLDRESTAPIRequest('friends.areFriends',array('uids1'=>$profile['id'],'uids2'=>$userProfile['id']));

				// They are not friends: reject and set wrongPasswordSent to 'facebookNotFriends' so message indicate the real cause
				if(!@$res[0]['are_friends']) {
					$wrongPasswordSent='facebookNotFriends';
					return false;
				}
			}

			// Set user name (and ID) as pseudo
			$_ENV['userPseudo']='FBID'.$userProfile['id'].':'.$userProfile['name'];

			return true;
		}
	}
	elseif($aM=='noAuthentication') return true; // Password typed on a passwordless account
	cfLog('login.php: unknown authentification method in.usr file, cannot log in'.$userId.' account',LOG_ERROR);
	$securityEvent.='Password check: unknown error';
	$wrongPasswordSent=true;
	return false;
}

/**
 * @return bool : true if OK
 * @param string $userId : user id
 * @param string $password : password
 * @desc verify $userId and $password. Return true if OK, false if not OK (in this case, wait for a few seconds)
*/
function verifyDirectResourceAccessPassword($rId, $password){
	global $wrongPasswordSent;
	global $securityEvent;

	// Decrypt RSA crypted password
	$decrypted=cfRSADecrypt($password,$wasCiphered);

	// Parse return string
	// Return string (decrypt) format : ip=XXX/password/ts=YYY/
	// where XXX is iptolong ip address and YYY is timestamp
	if($wasCiphered){
		$timestamp=substr($decrypted,strrpos($decrypted,'/ts=')+4);$timestamp=substr($timestamp,0,strpos($timestamp,'/'));
		$longip=substr($decrypted,strpos($decrypted,'ip=')+3);$longip=substr($longip,0,strpos($longip,'/'));
		$password=substr($decrypted,strpos($decrypted,'/',strpos($decrypted,'ip='))+1);$password=substr($password,0,strrpos($password,'/ts='));
	}

	// Check client ip address
	if($wasCiphered && $_SERVER['REMOTE_ADDR']!=long2ip($longip) && !cfGGetVar('disableClientSessionIPControl')){
		$securityEvent.='Password check: incorrect client IP address error: '.$_SERVER['REMOTE_ADDR'].'<>'.long2ip($longip);
		setWrongPasswordTyped();
		return false;
	}

	// Check timestamp (allow a 60 seconds difference (default) to cover transmission and ciphering time
	if($wasCiphered && abs(((int)microtime(true))-$timestamp)>((cfGGetVar('RSAAuthorizedDelay'))?cfGGetVar('RSAAuthorizedDelay'):20)){
		$securityEvent.='Password check: excessive time shift error<br>Modify "RSAAuthorizedDelay" value in data/general.ini';
		return false;
	}

	// If password is sent unencrypted but encryption was required, refuse
	if (0 /* !$wasCiphered && cfGGetVar('protocol')=='http' &&
			cfGGetVar('encryptPassword') && (!cfGGetVar('disablePasswordCipheringForSlowJSDevices') || !cfBGetVar('slowJS'))*/){
		cfLog('login.php: unencrypted password sent. Connection refused',LOG_ERROR);
		$securityEvent.='Password check: encrypted password not found';
		return false;
	}

	// Correct password typed
	if($password==cfStoredPasswordDecrypt(wResConfigNew($rId)->getVar('directResourceAccessPassword'))) {
		setRightPasswordTyped();
		return true;
	}
	// Wrong password typed
	else {
		setWrongPasswordTyped();
		sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));
		cfLog('login.php: wrong password typed for direct resource access',LOG_DBG);
		$wrongPasswordSent=true;
		return false;
	}
}

/**
 * @desc Validate an external authentication request, and deliver an authentication token if all OK
 *
 * @param string $authenticationToken: encrypted userName(or direct resource access token prefixed with dRA:) & pwd
 * @param string $restrictSource: true to restrict external authentication to weezo.net & weezo.info... domains
 */
function requestLoginToken($authenticationToken, $restrictSource=true){
	ignore_user_abort(true);

	// Restrict source to weezo.net/weezo.info/... servers
	if($restrictSource && !cfIsWeezoServer($_SERVER['REMOTE_ADDR'])) die('Forbidden Remote Host ('.$_SERVER['REMOTE_ADDR'].')');

	// Verify user is registered, and retreive shared 3DES key
	if(!($regInfo=cfRegistrationData())) die();
	$key=base64_decode($regInfo['regKey']);

	/**
	 * Public key request
	 */
	if($authenticationToken=='getRSAPK') {
		$RSAKeyPair = cfRSALoadKeyPair();
		// If RSA keypair successfully loaded, prepare response (but send it later, as we may 1st want to check if a password-less user name or access token is passed)
		if(isset($RSAKeyPair['maxDigits']) && isset($RSAKeyPair['modulo']) && isset($RSAKeyPair['publicExponent']))
			$getRSAPKResponse='RSAPK/'.$RSAKeyPair['maxDigits'].'/'.$RSAKeyPair['modulo'].'/'.$RSAKeyPair['publicExponent'].'/'.$RSAKeyPair['keyLength'];
		// Problem/failure
		else
			$getRSAPKResponse='';

		// If an user name (possibly containing a direct resource access token) is passed, 1st check if it exists and has no password, and if so, directly generate an access token
		if(isset($_GET['ip']) && (isset($_GET['checkToken']) || isset($_GET['checkUserPassword']))){
			if(isset($_GET['checkUserPassword'])){
				$args['user']=$_GET['checkUserPassword'];
			}
			else{
				$args['user']=$_GET['checkToken'];
			}
			$args['ip']=$_GET['ip'];
			$args['password']=false;
		}
		// If no user or DRA token passed, just return public key
		else {
			$subUserGroups=array();
			// Check for groups with subUsers in order to return a list of this groups to centralized authentication server
			foreach (WEnv::userConfigs() as $uc){
				if($uc->getVar('accountType')=='list' && $uc->getVar('activated')) $subUserGroups[]=str_replace('|','',cfUTF8Encode($uc->getVar('name')));
			}
			if(count($subUserGroups)>0) $getRSAPKResponse='SUBGROUPS'.implode('|',$subUserGroups).'|'.$getRSAPKResponse;
			die($getRSAPKResponse);
		}
	}

	// Decrypt token ciphered with shared 3DES key, and parse URL-encoded result into $args array
	else{
		$token=base64_decode(str_replace(' ','+',$authenticationToken));
		$iv=substr($token,-8); $token=substr($token,0,strlen($token)-8);
		$clear=@mcrypt_decrypt(MCRYPT_TRIPLEDES,$key,$token,MCRYPT_MODE_CBC,$iv);
		@parse_str($clear,$args);
	}


	/**
	 * Check decrypted token format
	 */
	if((!isset($args['RSAToken']) && (!isset($args['user'])||!isset($args['password'])))||!isset($args['ip'])) die();

	// Client browser encrypted password with RSA public key: decrypt
	if(isset($args['RSAToken'])){
		$decrypted=cfRSADecrypt('rsa:'.$args['RSAToken']);
		@parse_str($decrypted,$args2);
		// Verify it contains user and password
		if(!isset($args2['user']) || !isset($args2['password'])) die();
		$user=$args2['user'];
		$password=$args2['password'];
	}
	// If RSA encoding not used, retreive user name and password from 3DES ciphered arguments
	else{
		$user=cfUTF8Decode(base64_decode($args['user']),true,false,false);
		$password=(base64_decode($args['password']));
	}

	// Check if client's IP is not banned for excessive attempts
	if(cfIPBanned($args['ip'])){
		die('maxAttempts');
	}



	/**
	 * Direct Resource Access
	 */
	if(cfCmpLeft($user,'dRA:')){
		$directResourceAccessToken=substr($user,4);

		// Browse resources to find passed token
		foreach (WEnv::resConfigs() as $rid=>$resConfig){
			if($resConfig->getVar('directResourceAccessToken')!=$directResourceAccessToken) continue; // Token not matching
			if(!$resConfig->getVar('directResourceAccessEnabled')) continue; // Direct access not enabled
			if($resConfig->getVar('directResourceAccessLanOnly')) continue;  // Access from LAN only (and a centralized authentication request cannot come from an on-LAN computer)

			// If it's a public key request, we just want to check if there is a password.
			// If so, then just return RSA PK so remote user can enter a password
			if($authenticationToken=='getRSAPK' && $resConfig->getVar('directResourceAccessPasswordHas')) die($getRSAPKResponse);

			// Check password (if set)
			if(!$resConfig->getVar('directResourceAccessPasswordHas') || cfStoredPasswordDecrypt($resConfig->getVar('directResourceAccessPassword'))==$password){
				// Send resource ID in stored token, prefixed with dRA: (direct resource access)
				// to inform checkLoginToken() function that passed ID matches with a resource
				$userId='dRA:'.$rid;
				break;
			}
		}

		// Check if it's a short link
		if(!isset($userId) && cfMightBeShortLink($directResourceAccessToken) && cfShortLinkEnabled()){
			require_once(cfAppDocRoot().'/local/directLink.php');

			if($link=dlResolveShortLink($directResourceAccessToken,true)) {

				// Check if access is restricted to LAN (a centralized authentication request cannot come from an on-LAN computer)
				if(@$link['type']=='resource'){
					$resConfig=new WResConfig($link['resourceId']);
					if($resConfig->getVar('directResourceAccessLanOnly')) die('loginFailed');
				}

				// If it's a public key request, we just want to check if there is a password
				// If so, then just return RSA PK so remote user can enter a password
				if($authenticationToken=='getRSAPK' && @$link['passwordHas']) die($getRSAPKResponse);

				$userId='dRA:'.$directResourceAccessToken;
			}
		}
		if(!isset($userId)) die('loginFailed');
	}

	/**
	 * Regular user account access
	 */
	else {
		if(cfCmpLeft($user,'dUA:')) $user=substr($user,4);
		$directResourceAccessToken=$user;

		// Verify user's existence
		if(!($userId=userIdFromName($user))) {
			if($authenticationToken=='getRSAPK') die($getRSAPKResponse); else die('loginFailed');
		}

		// Verify IP is allowed to connect to this user
		if(!cfIPFilterCheck($_ENV['users'][$userId],$args['ip'])) die('loginFailed');

		// Check subUser password
		if($_ENV['users'][$userId]['authenticationMethod']=='list'){
			// If it's a public key request, we just want to check if there is a password.
			// If so, then just return RSA PK so remote user can enter a password
			if($authenticationToken=='getRSAPK') die($getRSAPKResponse);

			// Check password is ok
			if(strpos($password,'*wSep*')){
				// Extract subUser name / password
				list($subUserName,$password)=explode('*wSep*',$password,2);
				$subUserName=cfUTF8Decode($subUserName,true,true,false,128);

				// Search for subUser into group
				$uc=new WUserConfig($userId);

				foreach ($uc->getSubUserNames() as $sUId=>$sUName) if($subUserName===$sUName){
					$subUserNameFound=1;
					// Check subUser password
					$subUser=new WSubUser($sUId);
					// Password OK
					if($subUser->checkPassword($password)){
						// Indicate password is ok
						$subUserPasswordOk=1;
						$subUserId=$sUId;
						break;
					}
					break;
				}
			}

			if(!@$subUserPasswordOk) die('loginFailed');
		}

		// Check group password
		elseif($_ENV['users'][$userId]['authenticationMethod']=='password'){
			// If it's a public key request, we just want to check if there is a password.
			// If so, then just return RSA PK so remote user can enter a password
			if($authenticationToken=='getRSAPK') die($getRSAPKResponse);
			// Check password is ok
			if(!cfHashedPasswordCheck($password,$_ENV['users'][$userId]['password'])) die('loginFailed');
		}
		// If it's a public key request and user has a pseudo, return public key
		if($authenticationToken=='getRSAPK' && $_ENV['users'][$userId]['accountType']=='named') die($getRSAPKResponse);

		// prefix dUA: (direct user acces) to inform checkLoginToken() function that passed ID matches with an user
		$rawUserId=$userId;
		$userId='dUA:'.$userId;
	}

	// Generate random login token
	$loginToken=''; for($i=0;$i<16;$i++) $loginToken.=chr(rand(0,255)); $loginToken=base64_encode($loginToken);

	// Save login token to memory
	$loginTokens=cfMGetVar('loginTokens');
	$loginTokens[$loginToken]=$userId.'/'.@$subUserId.'/'.$args['ip'].'/'.time();
	cfMSetVar('loginTokens',$loginTokens);

	// All OK: generate login token
	if(!isset($rawUserId)) $rawUserId=$userId;

	die('loginToken='.$loginToken.((@$_ENV['users'][$rawUserId]['accountType']==='named')?'&requestPseudo=1':''));
}

/**
 * @desc Centralized authentification last step: user has been redirected here with login token : check login token validity, and if OK, log user in
 *
 * @param string $loginToken
 */
function checkLoginToken($loginToken){
	$loginToken=str_replace(' ','+',$loginToken);

	if(isset($_GET['preLogin'])){
		cfInsertHead();
		echo '<script type="text/javascript">';
		echo 'wl.goURL("'.str_replace('&preLogin=1','',$_SERVER['REQUEST_URI']).'")';
		echo '</script>';
		exit;
	}

	// Check for passed token in valid tokens list
	$loginTokens=cfMGetVar('loginTokens');
	if(!$loginTokens) {
		//cfVarDump(cfMGetVar('loginTokens',true));
		exit;
	}
	if(!isset($loginTokens[$loginToken])) outDisplayErrorPage(cfCaption('loginForbiddenAccess').' (IT)');

	// Extract access type (dRA for resource or dUA for user), user id and ip
	list($userId,$subUserId,$ip,$timestamp)=explode('/',$loginTokens[$loginToken]);
	list($accessType,$userId)=explode(':',$userId);

	// Remove from valid tokens list
	unset($loginTokens[$loginToken]);
	cfMSetVar('loginTokens',$loginTokens);

	// Check token is still valid
	if(time()>$timestamp+LOGIN_TOKEN_LIFETIME) die(cfCaption('loginForbiddenAccess').' lt');

	// Check token IP matches with client IP
	if(LOGIN_TOKEN_CHECK_IP && $ip!=$_SERVER['REMOTE_ADDR']) {
		if($ip!='0') die(cfCaption('loginForbiddenAccess').' ip nok:'.$ip.'<>'.$_SERVER['REMOTE_ADDR']);
	}


	// Check user / resource id is valid
	if($accessType=='dUA' && !isset($_ENV['users'][$userId])) die(cfCaption('loginForbiddenAccess').' no user');
	if($accessType=='dRA'){
		// Check if it's a resource link
		if(!wResConfigNew($userId)->isValid()) {
			require_once(INCLUDE_DIR.'explorerFunctions.php');
			require_once(cfAppDocRoot().'/local/directLink.php');

			// Else, check if it's a short link
			if(cfShortLinkEnabled() && $link=dlResolveShortLink($userId,true)){
				// Resource direct link
				if($link['type']=='resource'){
					$userId=$link['resourceId'];
					if(!wResConfigNew($userId)->isValid()) $userId=false;
				}
				// Download/streaming direct link
				elseif($link['type']=='directLink'){
					header('Location: /downloads/dlToken'.$link['id'].'/'.basename($link['filename']));
					exit;
				}
				else $userId=false;
			}
			else $userId=false;
		}
		if(!$userId)  outDisplayErrorPage(cfCaption('loginForbiddenAccess').' (NR)');
	}

	// Destroy previous session info (might have some if re-login from a different directAccessToken)
	$_SESSION=array();

	/**
	 * User login
	 */
	if($accessType=='dUA'){
		// Check user pseudo is OK
		if($_ENV['users'][$userId]['accountType']=='named' && !isset($_GET['userPseudo']) && !isset($_POST['userPseudo'])) die(cfCaption('loginForbiddenAccess').' no pseudo');

		// If it's a subUser, pass the info with an ugly ENV var
		if($subUserId) $_ENV['subUserLogged']=$subUserId;

		// All OK, proceed to login
		userLoad($userId);
	}

	/**
	 * Resource login
	 */
	else{
		$_GET['directResourceAccessToken']=wResConfigNew($userId)->getVar('directResourceAccessToken');

		/**
		 * Load standalone user.
		 * Resource info will be later retreived from $_GET['directResourceAccessToken'],
		 * put into $_SESSION['user']['directResourceAccessToken'] (userLoad() function),
		 * and then converted to actual linked resource by ifLoadUserResources() function
		 */

		userLoad('standalone');
	}
}

/**
 * @desc Resource-only login through publishToken
 *
 */
function publishTokenLogin(){
	// If publishToken is retreived from cookie (session timed out), restore in $_GET for further processing
	if(!isset($_GET['publishToken'])) $_GET['publishToken']=$_COOKIE['weezoPublishToken'];

	// Browse resources to find passed token
	foreach (WEnv::resConfigs() as $id=>$res){
		if($res->getVar('publishToken')==$_GET['publishToken']) {
			$found=1;
			break;
		}
	}

	// If no matching resource found, die
	if(!isset($found)){
		// Remove publish token if set (so error page won't return to login)
		if(isset($_COOKIE['weezoPublishToken'])){
			unset($_COOKIE['weezoPublishToken']);
			@setcookie('weezoPublishToken','',(time()-3600));
		}

		require_once(INCLUDE_DIR.'outputFunctions.php');
		outDisplayErrorPage(cfCaption('keyError'));
	}

	// Set theme
	if(isset($_GET['theme'])) $_POST['userSelectedTheme']=$_GET['theme'];

	// Load standalone user.
	// Resource info will be later retreived from $_GET['publishToken'], put into $_SESSION['user']['publishToken'] (userLoad() function)

	// And then converted to actual linked resource by ifLoadUserResources() function
	userLoad('standalone');
}

/**
 * @desc Direct resource acces login
 * 		 Check passed directResourceAccessToken exists (browse resources)
 * 		 If no passed password and no direct resource access password, proceed to userLoad (login OK)
 * 		 If passed password check and proceed to userLoad if OK
 * 		 Else display password login page
 * @param $token: direct resource access token. If set, overrides data passed into post, get or cookie (used by API)
 * @param $password: password. If set, overrides data passed into post, get or cookie (used by API)
 * @param $apiMode: set this parameter to true to just return result of token check
 * @return boolean result of acces token check (only if apiMode set to true)
 */
function directResourceAccessTokenLogin($token=false,$password=false,$apiMode=false){
	/**
	 * @desc Display direct resource access login page
	 * @param string $directResourceAccessToken: token
	 * @param boolean $wrongPasswordTyped: true if a wrong password has been typed
	 */
	function directResourceAccessLoginPage($directResourceAccessToken,$wrongPasswordTyped=false){
		$_ENV['directResourceAccessToken']=$directResourceAccessToken;
		$_ENV['users']=array();
		displayLoginForm(true);
		exit;
	}

	// If publishToken is retreived from cookie (session timed out) or POST, restore in $_GET for further processing (userLoad function uses it)
	if($token){}
	elseif(isset($_POST['directResourceAccessToken'])) $token=$_POST['directResourceAccessToken'];
	elseif(isset($_GET['directResourceAccessToken'])) $token=$_GET['directResourceAccessToken'];
	else $token=@$_COOKIE['weezoDirectResourceAccessToken'];
	$_GET['directResourceAccessToken']=$token;

	// Browse resources to find passed token
	foreach (WEnv::resConfigs() as $rid=>$res){
		if($res->getVar('directResourceAccessToken')==$token) {
			$found=1;
			break;
		}
	}

	// If no matching resource found, die
	if(!isset($found)){
		if($apiMode) return false;
		// Remove direct resource access token if set (so error page won't return to login)
		if(isset($_COOKIE['weezoDirectResourceAccessToken'])){
			unset($_COOKIE['weezoDirectResourceAccessToken']);
			@setcookie('weezoDirectResourceAccessToken','',(time()-3600));
		}
		// Async die (should not happend)
		if(cfIsAsync())	die(cfAsyncHeader().cfAsyncFooter(false));
		// Sync error page
		require_once(INCLUDE_DIR.'outputFunctions.php');
		outDisplayErrorPage(cfCaption('keyError'));
	}

	// Access not enabled or LAN only access
	if((!$res->getVar('directResourceAccessEnabled')) || ($res->getVar('directResourceAccessLanOnly') && !cfIsOnLAN())){
		if(cfIsAsync()){
			echo cfAsyncHeader();
			echo cfAsyncXMLJSaction('if(W.resetInputs) resetInputs();loginSetError("'.cfCaption('loginBadPassword').'");if(wl.mask) wl.mask.hide()');
			die(cfAsyncFooter());
		}
		outDisplayErrorPage(cfCaption('keyError'));
	}

	// If direct access is not password protected, proceed to userLoad (see last userLoad call in this function for more info)
	if(!$res->getVar('directResourceAccessPasswordHas')) {
		if($apiMode) return true;
		return userLoad('standalone');
	}

	// Retreive sent password
	if($password){} // Password passed from API
	elseif(isset($_POST['password'])) $password=$_POST['password']; // Password sent from login form
	elseif(isset($_GET['password'])) {// Password included in URL
		$password=$_GET['password'];
		if(cfCmpLeft($password,'b64.')) $password=cfBase64_url_decode(substr($password,4)); // If password is base 64 encoded, decode
	}

	// Password not (yet sent), directly display direct resource access login page
	if(!$password && $password!=="") directResourceAccessLoginPage($token);

	// Check password, and if not OK, display direct resource access login page
	if(!verifyDirectResourceAccessPassword($rid,$password)) {
		if($apiMode) return false;
		if(cfIsAsync()){
			echo cfAsyncHeader();
			echo cfAsyncXMLJSaction('if(W.resetInputs) resetInputs();loginSetError("'.cfCaption('loginBadPassword').'");if(wl.mask) wl.mask.hide()');
			die(cfAsyncFooter());
		}
		else directResourceAccessLoginPage($token,true);
	}

	// Password OK

	if($apiMode) return false;

	// Load standalone user
	// Resource info will be later retreived from $_GET['publishToken'], put into $_SESSION['user']['publishToken'] (userLoad() function)
	// And then converted to actual linked resource by ifLoadUserResources() function
	userLoad('standalone');
}

/**
 * @desc add entry in data/connectionAttempts.txt file
 * @return void
 *
 */
function setWrongPasswordTyped(){
	global $nbWrongPasswordTyped;
	if(!cfGGetVar('passwordCheckMaxAttempts')) return;
	cfAppendTextToFile(microtime(true).' = '.$_SERVER['REMOTE_ADDR'],cfAppDataDir().'/connectionAttempts.txt',false);
	$nbWrongPasswordTyped++;
	if($nbWrongPasswordTyped>=cfGGetVar('passwordCheckMaxAttempts')){
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('wl.goURL()');
		die(cfAsyncFooter());
	}
}

/**
 * @desc add right password entry in data/connectionAttempts.txt file
 * @return void
 *
 */
function setRightPasswordTyped(){
	if(!cfGGetVar('passwordCheckMaxAttempts')) return;
	cfAppendTextToFile(microtime(true).' = OK'.$_SERVER['REMOTE_ADDR'],cfAppDataDir().'/connectionAttempts.txt',false);
}

/**
 * @desc echo hidden login form and javascript function used for RSA password encryption
 * @param boolean $encryptionAnimation : true if lock icon animation should be played on encryption
 * @param boolean $waitCursor : true if a wait cursor is displayed while loginForm is being submitted
 * @return  void
 *
 * 		"startEncryption" script must always be called by loginFrom.php scripts
 * 		It must be called within <body> node as it includes HTML code (unless $encryptionAnimation is set to false)
 *		JS function encryptPassword must be called on form submit, with password node as parameter
 *
 * 		JS Script description : cipher password, and submit login form and optionnaly play animation
 *
 */
function includeLoginFormAndScripts($waitCursor=true){
	/**
	 * Page reload form
	 */
	?>
	<form name="connectForm" action="/index.php" method="GET" enctype="multipart/form-data" style="display:none">
	<input name="loginOK" value="true">
	</form>
	<?php

	/**
	 * Login Form
	 */
	echo '<form id="loginForm" name="loginForm" action="/login.php" method="post" enctype="multipart/form-data" style="display:none">';
	// User Name
	echo '<input name="user" id="user">';
	// Password
	echo '<input name="password" id="password" maxlength="1500">';
	// User Id
	echo '<input name="userId" id="userId">';
	// User pdeudo/name
	echo '<input name="userPseudo" id="userPseudo">';
	// User screen size
	echo '<input name="userScreenWidth"><input name="userScreenHeight">';
	// User screen size
	echo '<input name="userHasFlash">';
	// Server host name
	echo '<input name="serverExternalHost">';
	// Mobile login
	echo '<input name="mobile">';
	// Direct resource access token
	if(isset($_ENV['directResourceAccessToken']))
		echo '<input name="directResourceAccessToken" value="'.$_ENV['directResourceAccessToken'].'">';

	// Theme
	echo '<input type="text" name="userSelectedTheme" value="'.(cfGGetVar('userSelectedTheme')?cfUTF8Encode(cfGGetVar('userSelectedTheme')):'').'">';
	//echo '<input type="submit" value="s" style="position:absolute; top:-100px;">';
	echo "</form>\n";

// User pseudo input box
?>
<div id="userPseudoDiv" style="display:none;position:absolute;left:50%;top:50%;z-index:1001">
	<div class="popup" style="position:relative;left:-50%;top:-100px;width:30em">
	<div class="popupHeader" id="userPseudoHeader"><?php echo cfCaption('userTypeName');?></div>
	<?php echo outDivFrame('frame2');?><br>
	<?php echo cfCaption('userTypeName');?><br>
	<input type="text" size="20" style="width:100%" id="loginPseudoInput"><br><br><center>
	<?php echo outButton('','javascript:loginPseudoSubmit(dgi(\'loginPseudoInput\').value);',outIcon('ok16'),false,false,'style="margin-right:2em"');?>
	<?php echo outButton('','javascript:loginPseudoSubmit(\'*weezoCancel*\');',outIcon('cancel'));?>
	</center><br></div></div>
</div>

<?php

/**
 * Facebook login procedure:
 * - user clicks on button => call to login() function
 * - as user password type is "facebook", call loginFacebook() instead of regular logUserById/logUserByName call
 * - WFacebook.login() opens a popup on facebook authentication page. If user successfully log in and accepts application,
 * 		loginFacebookProceed is called with access_token as parameter. access_token is then passed as the password,
 * 		and server-side checked.
 */
require_once(INCLUDE_DIR.'facebookFunctions.php');
facebookConnectScript('login','loginFacebookProceed');
?>
<script type="text/javascript">
var loginFacebookUserId;
function loginFacebook(userId){loginFacebookUserId=userId;WFacebook.connect()}
function loginFacebookProceed(access_token){logUserById(users[loginFacebookUserId]['id'],access_token)}
<?php
/**
 * User's pseudo entry functions
 */
?>
function loginPseudoGet(userName){
	if(W.promptUserPseudo) return promptUserPseudo(userName);
	dgi("userPseudoHeader").innerHTML=userName;
	if(W.fade && wl.mask){
		fade(dgi("userPseudoDiv"),0,1,10,'dgi("loginPseudoInput").focus()');
		wl.mask.show(0,1)
		wl.mask.moveAbove(dgi("userPseudoDiv"))
		wl.setKeycodeListener(D,loginPseudoKD)
	}
	// Mobile
	else {
		var p=prompt("<?php echo cfCaption('userTypeName')?>","");
		if(p) loginPseudoSubmit(p); else wl.liUnselAll();
	}
}
function loginPseudoSubmit(userPseudo){
	if(userPseudo=='') return;
	dgi('userPseudo').value=userPseudo;
	asyncSubmitForm("loginForm");
	fade(dgi("userPseudoDiv"),1,0);
	if(wl.mask) wl.mask.hide(1)
}
function loginPseudoKD(kc){
	if(kc==13) loginPseudoSubmit(dgi('loginPseudoInput').value);
	if(kc==27) loginPseudoSubmit('*weezoCancel*');
}

<?php


/**
 * Main login functions: encrypt data if necessary, then send to server for login
 * - logUserById: use user unique id
 * - logUserByName: use user name
 */
if(!cfIsMobile()) {?>
function logUserById(id,pwd){
	dgi('userId').value=id;
	dgi('user').value='';
	dgi('password').value=(pwd)?pwd:'';
	startEncryption();
}
function logUserByName(name,pwd){
	dgi('user').value=name;
	dgi('userId').value='';
	dgi('password').value=(pwd)?pwd:'';
	startEncryption();
}
function forceMobile(){D.location.href=D.location+((D.location.href.indexOf('?')!=-1)?'&':'?')+'forceMobile=1';}
<?php }?>

</script>

<?php
// RSA Scripts
	echo outRSAScripts('auto',$doesCipher);
?>
<script language="JavaScript" type="text/javascript">
<?php
echo 'var li=new Image; li.src="'.outIcon('lockB').'";'; // Preload image
echo 'var lsi=new Image; lsi.src="'.outIcon('lockSend').'";'; // Preload image
echo 'var ts='.(int)microtime(true).";\n";
echo 'var ip='.sprintf("%u", ip2long($_SERVER['REMOTE_ADDR'])).";\n";

?>
var loadTs=Date.parse((new Date()).toGMTString());
function loginConnect(c){D.connectForm.submit()}
function startEncryption(){
<?php if(cfIsMobile()) {?>
	dgn("userScreenWidth").value=Math.max(screen.width,W.outerWidth,dgn("userScreenWidth").value);
	dgn("userScreenHeight").value=Math.max(screen.height,W.outerHeight,dgn("userScreenHeight").value);
	dgn("mobile").value=1;
<?php
	}
	else {
?>
	dgn("userScreenWidth").value=screen.width; dgn("userScreenHeight").value=screen.height;
<?php
	}
	if(cfBGetVar('flash')==='auto') echo 'dgn("userHasFlash").value=(wl.hasFlash())?1:0;';
?>
	dgn("serverExternalHost").value=D.location.host;
	if(wl.mask) wl.mask.show(1);
	if(D.loginForm.password.value.length==0) {
		asyncSubmitForm("loginForm");
		return;
	}
<?php
	// Disabled encryption: just sumit form
	if(!$doesCipher) {
?>
	dgi('password').value=dgi('password').value.replace(/\+/,'*weezoPlus*')
	return asyncSubmitForm("loginForm");
}
<?php
		echo '</script>';
		return;
	}
	// Encryption enabled: show encryption lock, and proceed
	else{
?>
	dgi("encryptionLock").style.display="block";
	setTimeout("proceedEncryption()",10);
}
<?php } ?>
function proceedEncryption(){
	var msg=D.loginForm.elements["password"].value
	var now=new Date();
	RSACrypt("ip="+ip+"/"+msg+"/ts="+(ts+(Date.parse(now.toGMTString())-loadTs)/1000+2)+"/",proceedEncryption2,(W.encryptPercNode)?encryptPercNode:dgi("encryptPerc"))
}

function proceedEncryption2(ciphered){
	D.loginForm.password.value=ciphered;
	if(dgi("encryptionLock")) dgi("encryptionLock").style.display="none";
	if(dgi("encryptionLockSend")) dgi("encryptionLockSend").style.display="block";
	if(W.postEncodingMessage) postEncodingMessage();
	asyncSubmitForm("loginForm");
}

</script>
<?php
// encryption indicator
?>
<div style="z-index:995" id="encryptionLock"><div style="position:relative; left:-50%;">
	<center><?php echo outImage(outIcon('lockB'),false,false, 'position:relative');?><br><div class="loginEncrypt"><?php echo cfCaption('passwordCrypt');?> <span id="encryptPerc">0%</span></div></center>
</div></div>
<div style="z-index:996" id="encryptionLockSend"><div style="position:relative; left:-50%;">
	<center><?php echo outImage(outIcon('lockSend'),false,false, 'position:relative');?><br><div class="loginEncrypt"><?php echo cfCaption('loginConnectButton');?></div></center>
</div></div>
<?php
}

/**
 * @desc Display login form
 *
 * @param boolean $passwordOnly: true to display a password only form (so far used only for direct resource access login)
 */
function displayLoginForm($passwordOnly=false){
	global $userToDisplay, $wrongUserSent, $userNameSent;

	/*
	 ***************************************************************************************************************************
	 * Select login form
	 ***************************************************************************************************************************
	 */

	// Password only form
	if($passwordOnly){
		// Mobile
		if(cfIsMobile()) $loginThemeForm='mobile/loginForm.php';
		// Browser
		else $loginThemeForm='common/loginFormClassic.php';
	}

	// regular user + password form
	else{
		// If browser detected as mobile or user forced mobile view, use common mobile login form
		if(cfIsMobile()) $loginThemeForm='mobile/loginForm.php';

		// Else, select classic or coverflow theme
		elseif(cfGGetVar('loginForm')==='coverflow') $loginThemeForm='common/loginFormCoverflow.php';

		// Else use common login form
		else $loginThemeForm='common/loginFormClassic.php';
	}



	/*
	 ***************************************************************************************************************************
	 * Page header (and progress animation)
	 ***************************************************************************************************************************
	 */
	loginPageHeader($loginThemeForm);



	/*
	 ***************************************************************************************************************************
	 * Simple mode: display a list of shared resources with a direct access
	 ***************************************************************************************************************************
	 */
	if(cfSimpleMode() && !isset($_GET['directResourceAccessToken'])){
		if(cfIsMobile()) require_once(cfAppDocRoot().'/themes/mobile/resourcesLoginForm.php');
		else require_once(cfAppDocRoot().'/themes/common/resourcesLoginForm.php');
	}

	/*
	 ***************************************************************************************************************************
	 * Display Login Form
	 ***************************************************************************************************************************
	 */
	else require_once(W_THEMES_DIR.$loginThemeForm);

	/*
	 ***************************************************************************************************************************
	 * Finish login Animation
	 ***************************************************************************************************************************
	 */
	loginPageFooter();

	cfLog(cfCaption('logIncomingConnection',false,false,false,true),LOG_DBG);
}

/**
 * Echo page header, including loading screen, body, scripts & other stuff
 *
 */
function loginPageHeader($loginThemeForm){
	if(cfIsAsync() || cfIsMobile()) return;

	// Estimate total page size
	$pageSize=800; // Header
	$pageSize+=$cssSize=cfFileSize(cfAppDocRoot().'/themes/common.css')+@filesize(cfAppDocRoot().'/themes/'.cfGGetVar('theme').'/theme.css'); // CSS
	$pageSize+=($commonSize=cfFileSize(cfAppDocRoot().'/js/common.js'));// common.js
	if(cfGGetVar('loginForm')==='coverflow'){
		$pageSize+=($dragdropSize=cfFileSize(cfAppDocRoot().'/js/wz_dragdrop.js'));
		$pageSize+=($htmlSize=13000); // Estimated HTML size
		$pageSize+=count($_ENV['users'])*2500; // User icons
	}
	else {
		$pageSize+=($htmlSize=15000); // Estimated HTML size
		$pageSize+=2000; // Estimated window icons size
		$dragdropSize=0;
	}
	$pageSize+=2200+481; // Icons

	/*
	 ***************************************************************************************************************************
	 * Insert HTML Head and Javascript
	 ***************************************************************************************************************************
	 */
	cfInsertHEAD(true,array('noCommon.js'=>1));
	/**
	 * Insert body with delayed background image loading
	 */
	list($wallpaperSrc,$wallpaperPosition)=loginGetBackgroundWallpaper();

	if(strtolower(cfFileWithoutExtension(basename($loginThemeForm)))=='loginformcoverflow') $bodyClass='coverflow'; else $bodyClass='classic';

?>
<body class="mainFrameBody loginBody <?php echo $bodyClass;?>" style="background-image:none" onload="setTimeout('lPreload()',100)"><NOSCRIPT><div class="textShadow" style="font-size:200%;margin-top:30%;width:100%;text-align:center"><?php echo outImage(outIcon('alertBig'),false,false,'vertical-align:middle;margin-right:1em').cfCaption('loginErrorNoJavascript');?></div></NOSCRIPT>
<table id="lTable" style="height:100%;width:100%"><tr><td style="vertical-align:middle;text-align:center">
<div id="lTxt" class="textShadow" style="font-size:80px;font-family:Times New Roman">...</div>
<br><br>
<b class="textShadow loginPreCSS"><?php echo str_replace('...',' ('.floor(100*800/$pageSize).'%)',cfCaption('loading'));?></b>
<b class="textShadow loginPostCSS" style=";position:relative;overflow:hidden;width:100%;display:block"><div style="margin-left:-9999px"><?php echo str_replace('...','',cfCaption('loading')).' (<span id="lProg">'.floor(100*$cssSize/$pageSize).'</span>%)';?></div></b>
</td></tr></table>
<script type="text/javascript">
var D=document,lProg=<?php echo floor(100*$cssSize/$pageSize); ?>;lStep=-1,lInt=setInterval(function(){var i=(++lStep)%6;document.getElementById('lTxt').innerHTML=((!i||i>3)?'.':'&nbsp;')+((i<2||i>4)?'.':'&nbsp;')+((i<3)?'.':'&nbsp;')},250);
function lFunc(s,p){lProg+=((p)?p/100:1)*((s=='common')?<?php echo floor(100*$commonSize/$pageSize);?>:((s=='wz_dragdrop')?<?php echo floor(100*$dragdropSize/$pageSize);?>:<?php echo floor(100*$htmlSize/$pageSize);?>));if(dgi('lProg')) dgi('lProg').innerHTML=Math.ceil(lProg)}
<?php
	/**
	 * Wallpaper
	 */

	// Remove all wallpaper for coverflow login
	if(basename(cfFileWithoutExtension($loginThemeForm))=='loginFormCoverflow' && !cfSimpleMode()){
		echo 'D.body.style.backgroundImage=""';
	}
	// Tiled wallpaper: set as body background image
	elseif($wallpaperPosition=='tiled')
		echo 'D.body.style.backgroundImage="url(\''.$wallpaperSrc.'\')"';

	// Stretched wallpaper
	elseif($wallpaperPosition=='stretched'){
?>
var i=D.createElement('img'),s=i.style;s.width='100%';s.height='100%';s.position='absolute';s.top=s.left='0px';i.id='wpImg';s.visibility='hidden';
i.onload=function(){if(window.fade) fade(this); else this.style.visibility=''};
D.body.appendChild(i);
setTimeout(function(){if(!D.getElementById('wpImg')) return; D.getElementById('wpImg').src='<?php echo $wallpaperSrc?>'},10);
<?php
	}
	// Centered wallpaper
	elseif($wallpaperPosition=='centered'){
		list($width, $height, $t, $a)=@getimagesize(cfAppDocRoot().$wallpaperSrc);
		echo "var i=D.createElement('img'),s=i.style;s.marginLeft='-".($width/2)."px';s.marginTop='-".($height/2)."px';s.position='absolute';s.top='50%';s.left='50%';i.id='wpImg';setTimeout(\"if(!D.getElementById('wpImg')) return; D.getElementById('wpImg').src=\\'".$wallpaperSrc."\\'\",10);s.visibility='hidden';i.onload=function(){if(window.fade) fade(this); else this.style.visibility=''};D.body.appendChild(i);";
	}
// Restore CSS background image that was temporary removed for faster loading, except if stretched or tiled (=fullscreen) wallpaper
else echo 'document.body.style.backgroundImage=""';

?>
</script>
<div id="lContent" style="display:none">
<?php

	/**
	 * Webkit cookies policy "workaround": if cannot set cookie, reload in top frame
	 */
	if(!isset($_POST['cookieReload'])){
?>
<form method="POST" action="<?php echo $_SERVER['REQUEST_URI'];?>" name="cookieReloadForm" target="_top" style="display:none"><input name="cookieReload" value="1"></form>
<script type="text/javascript">
if(parent && window!=parent){
	D.cookie='cookieTest=1;path=/;domain=http://192.168.0.10';
	if(!D.cookie||D.cookie.indexOf('cookieTest')==-1) D.cookieReloadForm.submit();
	var cd=new Date();cd.setTime(cd.getTime()-1);D.cookie="cookieTest=; expires="+cd.toGMTString();
}
</script>
<?php
	}
	flush();
	// Include scripts
	echo cfScriptLink('common.js');
}

/**
 * @desc Insert login page footer (finish login animation)
 *
 */
function loginPageFooter(){
	if(cfIsAsync() || cfIsMobile()) return;
	?>
</div>
<div id="preload"></div>
<script type="text/javascript">
function lPreload(){
	var i=3,n;
	for(;i>=0;i--) {
		n=D.createElement('SCRIPT');
		n.type="text/javascript";
		n.src='/js/'+['win','treeView','explorer','winClient'][i]+'.js';
		D.body.appendChild(n)
	}
}
clearInterval(lInt)
removeNode(dgi("lTable"));
dgi("lContent").style.display="";
if(W.init) init();
</script>
</body>
<?php
}

/**
 * Get background Wallpaper and wallpaper position (centered / tiled / streched)
 *
 * @param bool $theme: true to return theme's wallpaper
 * @return array (path to wallpaper, wallpaper type)
 */
function loginGetBackgroundWallpaper($theme=false){
	// Wallpapers located into theme directory
	$wps=glob(cfAppThemeDir(false).'/{wallpaperBoth,themeBoth,wallpaperLogin,themeLogin}-{centered,stretched,tiled}.{jpg,gif,png}',GLOB_BRACE);
	if(count($wps)){
		foreach ($wps as $key=>$wp) {
			list($type,$position)=explode('-',cfFileWithoutExtension(basename($wp)));
			$wps[$key]=array('cfn'=>$wp,'type'=>$type,'position'=>$position);
		}

		// Look for user-set login wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='wallpaperLogin') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);

		// Look for user-set login+desktop wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='wallpaperBoth') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);

		// Look for theme login wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='themeLogin') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);

		// Look for theme login+desktop wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='themeBoth') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);
	}
	return array(false,false);
}

/**
 * @desc Facebook crossdomain communication security restrictions workaround:
 * 	site (login.php)->facebook connect (authenticate)->weezo.net(.info) (turn code into access_token)->login.php?access_token=... ->insert hashtag into opener!
 *
 */
function loginRouteFacebookAccessToken(){
?>
<script type="text/javascript">
var W=window,l=W.location.href.replace(/&access_token/,'#access_token').replace(/\?access_token/,'#access_token')
if((i=l.indexOf('&redirectOpener'))>0) l=l.substr(0,i);
W.opener.location.href=l; W.close();
</script>
<?php
	exit;
}

/**
 * @desc Restore singleUser-group-level or subUser-level custom data
 *
 */
function loginRestoreCustomConfig(){
	// Retrieve raw data
	if(WEnv::user()->getVar('accountType')=='singleUser') $vars=WEnv::user()->getGVar();
	if(WEnv::user()->getVar('accountType')=='list') $vars=WEnv::user()->subUser()->getGVar();
	if(!isset($vars) || !is_array($vars)) $vars=array();

	// Apply to GVar
	foreach ($vars as $varName=>$value) {
		if($varName=='theme' && cfIsMobile()) continue; // Mobile login: don't apply desktop theme
		if($varName=='mobileTheme' && cfIsMobile()) $varName='theme'; // Mobile login: re-apply mobile theme
		if($varName=='language' && $value=='default') continue;
		cfGSetVar($varName,$value);
	}
}

// login.php may be included by api.php or webDAV.php for functions
if(defined('API_LOADED')) return;

/**
 * Detect browser and set capabilities
 */
cfSetBrowserCaps();

// If a pseudo is sent, start session (as user is already logged-in, so connection info should be retreived)
if(@$_POST['userPseudo']) wSession_start();

// Pseudo received from authenticated user
if(cfGGetVar('requestUserPseudo') && cfUGetVar('name') && isset($_POST['userPseudo']) && $_POST['userPseudo']!='') userLoadNamedUser();

//Facebook crossdomain communication security restrictions workaround: pass access_token to opener
if(isset($_GET['access_token']) && isset($_GET['redirectOpener'])) loginRouteFacebookAccessToken();


/*
 ***************************************************************************************************************************
 * AUTHENTICATION
 ***************************************************************************************************************************
 */
if(!isset($securityEvent)) $securityEvent=false;

// Load ini, usr and lng files (general parameters, users data, language), except if a login token is requested
ifInitializeSession(!isset($_GET['requestLoginToken']));

// Add a "fake" user with no name to indicate than an input box with both name and password must be set.
// This user will be unset if no hidden users are set
$_ENV['users'][0]=array('id'=>0, 'name'=>false, 'authenticationMethod'=>'password', 'icon'=>(is_file(cfTGetVar('dir').'/groups2.gif'))?'../..'.cfTGetVar('path').'/groups2.gif':'_groups2.gif');

/**
 * External authentication: login token sent by redirection server -> check crypted user name & password, and if OK, respond with a login token
 * that will be transmitted to browser before being redirected here with loginToken as GET parameter
 */
if(isset($_GET['requestLoginToken'])) requestLoginToken($_GET['requestLoginToken'],true);

// Check banned IP
if (cfIPBanned())  {sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));outDisplayErrorPage(cfCaption('loginForbiddenAccess').'...',APPLICATION_NAME);}

// External authentication login token (for regular group login and direct resource access)
if(isset($_GET['loginToken'])) checkLoginToken($_GET['loginToken']);

// Forbid direct web connections if registered user has set this option allong with externalAuth
if(!cfDirectWebConnectionsAllowed() && !cfIsOnLAN()){
	if(isset($regData['offlineURL']) && $regData['offlineURL']) {header('Location: http://'.$regData['offlineURL']);exit;}
	sleep(3600);cfHTTPError(404);
}

// Wii autologin
if(cfIsWII() && cfIsOnLAN() && isset($_COOKIE['weezoUser'])){
	if(isset($_ENV['users'][$_COOKIE['weezoUser']])){
		if($_ENV['users'][$_COOKIE['weezoUser']]['authenticationMethod']=='noAuthentication' ||
		(isset($_COOKIE['weezoAuth']) && $_COOKIE['weezoAuth']==sha1($_ENV['users'][$_COOKIE['weezoUser']]['password']))){
			userLoad($_COOKIE['weezoUser']);
		}
	}
}

// Published resource login
if(isset($_GET['publishToken']) || isset($_COOKIE['weezoPublishToken'])) publishTokenLogin();

// Direct resource access login
if(isset($_POST['directResourceAccessToken']) || isset($_GET['directResourceAccessToken']) || isset($_COOKIE['directResourceAccessToken'])) directResourceAccessTokenLogin();

$userNameSent=false;$wrongPasswordSent=false; $wrongUserSent=false;
$password=false; $userId=false; $userName=false; $userAndPasswordMethod=false;



/*
 ***************************************************************************************************************************
 * Get user to load from GET/POST parameters
 ***************************************************************************************************************************
 */

// If "autolog" GET parameter passed (from Weezo site), and site has a single user with no authentication, proceed to log
if(isset($_GET['autolog']) && count($_ENV['users'])==1){
	foreach($_ENV['users'] as $key=>$value)
		if($value['authenticationMethod']=='noAuthentication' && $value['hidden']=='false' && cfIPFilterCheck($value)) userLoad($key);
}

// if user name sent
if(@$_GET['user']) {
	$userNameSent=$_GET['user'];
	$userName=cfUTF8Decode(urldecode($_GET['user']),true,true,false);
	$userId=userIdFromName($userName);
	if($userId){
		// If password is sent unencoded into GET parameters, retreive
		if(isset($_GET['password'])) {
			$password=cfUTF8Decode(urldecode($_GET['password']),true,true,false);
			$password=str_replace('*weezoPlus*','+',$password);
		}
	}
	else {
		$userName=false;
		$userNameSent=false;
		$wrongUserSent=$_GET['user'];
		unset($_GET['user']); // Unset user GET so all non-matching groups are not removed
	}
}

// if login user name is sent using POST, and no userId sent, get Matching userId (user + password form used)
if(!$userId && @$_POST['user'] && !@$_POST['userId']){
	$userNameSent=$_POST['user'];
	$userName=cfUTF8Decode($_POST['user'],true,true,false);
	$userId=userIdFromName($userName);

	$userAndPasswordMethod=true;
	if($userId) {
		if(isset($_POST['password'])) $password=cfUTF8Decode($_POST['password']);
	}
	else{
		$userName=false;
		$wrongUserSent=$_POST['user'];
		sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));
	}
}
// if userId is sent
if(!$userId && isset($_POST['userId']) && $_POST['userId']!='0'){
	$userId=$_POST['userId'];
	if(isset($_POST['password'])) $password=cfUTF8Decode($_POST['password']);
	$userAndPasswordMethod=false;
}

/**
 * Facebook synchronous login (mobile version): we've been redirected from facebook.com->weezo.net->here
 * with facebook "access_token" as GET parameter
 * => find facebook group (should be one), and put access_token in $password
 */
if(isset($_GET['access_token'])) {
	foreach ($_ENV['users'] as $key=>$value) if(@$value['accountType']=='facebookFriends'){
		$userId=$key;
		$password=$_GET['access_token'];
		break;
	}
}

if($password=='') $password=false;

/*
 ***************************************************************************************************************************
 * Check user and password
 ***************************************************************************************************************************
  */

// If an user has been found
if($userId && isset($_ENV['users'][$userId])){
	$aM=$_ENV['users'][$userId]['authenticationMethod'];

	// No direct login to standalone-resource dedicated user
	if($aM=='publishToken') die('Unauthorized login');

	// If IP is not banned for this user
	if(cfIPFilterCheck($_ENV['users'][$userId])){
		// If this user doesn't require password, log in
		if($aM=='noAuthentication')
		 	userLoad($userId);
		// If this user requires a password or a facebook access_token, check
		elseif(($aM=='password' || $aM=='facebook' || $aM=='list') && $password){
			//if(strpos($password,'/shn=')) $password=substr($password,0,strpos($password,'/shn='));
			if(verifyUserPassword($userId, $password)) userLoad($userId);
		}
	}
	else $securityEvent.=' Banned IP';
}
elseif($userId) $wrongUserSent=$userId;

// If correct user name sent but password sent, set user name as wrong so error message doesn't indicate user name is correct
if($userNameSent && $wrongPasswordSent) $wrongUserSent=$userNameSent;

// Set user to display
if($userAndPasswordMethod) $userToDisplay=0;
elseif ($userId) $userToDisplay=$userId;


// Check if user has selected a theme on login page
if(isset($_POST['themeUpdate']) && $_POST['themeUpdate']!='' && isset($_POST['themeLevel']) && cfGGetVar('allowUserThemeChange')){
	cfGSetVar('userSelectedTheme',cfUTF8Decode($_POST['themeUpdate']));
	cfGSetVar('theme',cfUTF8Decode($_POST['themeUpdate']));
}
// if POST login parameters include a previously selected theme

if(isset($_POST['userSelectedTheme']) && $_POST['userSelectedTheme']!='' && cfGGetVar('allowUserThemeChange')){
	cfGSetVar('userSelectedTheme',cfUTF8Decode($_POST['userSelectedTheme']));
	cfGSetVar('theme',cfUTF8Decode($_POST['userSelectedTheme']));
}



/**
 * "Clean" users list
 */
$hiddenUsers=false;
require_once(INCLUDE_DIR.'facebookFunctions.php');

foreach ($_ENV['users'] as $key=>$value){
	// Remove hint from accounts with no password
	if(@$value['hint'] && $value['authenticationMethod']!=='password') unset($_ENV['users'][$key]['hint']);

	// If single account is requested, hide others
	if(isset($_GET['user']) && !isset($_GET['showOthers'])){
		 if($_GET['user']!==$value['name']) unset($_ENV['users'][$key]);
	}
	// Remove hidden users from user list before sending list to login page script
	elseif(isset($value['hidden']) && $value['hidden']==true) {
		unset($_ENV['users'][$key]);
		$hiddenUsers=true;
	}
	// If facebook account is not active, remove facebook group
	elseif($value['authenticationMethod']=='facebook' && !facebookAccountGet()) unset($_ENV['users'][$key]);
}
// If no hidden user, unset "fake" login+password user
if(!$hiddenUsers) unset($_ENV['users'][0]);



// If a specific user account is requested in url, verify this account exists
if(isset($_GET['user'])){
	$found=false;
	foreach ($_ENV['users'] as $key=>$value) if($_GET['user']===$value['name']){$found=true;break;}
	if(!$found) {
		$wrongUserSent=$_GET['user'];
		unset($_GET['user']);
	}
}



/*
 ***************************************************************************************************************************
 * Async Response processing
 ***************************************************************************************************************************
 */
if(cfIsAsync() && isset($_POST['user'])){
	echo cfAsyncHeader();

	// If security issue detected, display error message
	if($securityEvent)echo cfAsyncXMLJSaction('dgi("securityEvent").innerHTML="'.str_replace('"',"''",($securityEvent)).'";dgi("securityEvent").style.display="";');
	echo cfAsyncXMLJSaction('if(W.resetInputs) resetInputs();');

	// Facebook login error: user is not friend with defined facebook profile
	if($wrongPasswordSent==='facebookNotFriends') {
		$errorText=str_replace('"',"''",cfCaption('loginForbiddenAccess'));
		echo cfAsyncXMLJSaction('loginSetError("'.$errorText.'")');
	}
	// Wrong user name or wrong password sent: display "wong user or password" in the 1st case or "wrong password" in the second.
	else if($wrongPasswordSent || $wrongUserSent){
		$errorText=str_replace('"',"''",(($wrongUserSent)?cfCaption('loginBadPasswordOrAccount'):cfCaption('loginBadPassword')));
		echo cfAsyncXMLJSaction('loginSetError("'.$errorText.'")');
	}
	die(cfAsyncXMLJSaction('if(wl.mask) wl.mask.hide()').cfAsyncFooter(false));
}

/*
 ***************************************************************************************************************************
 * Sync display login form
 ***************************************************************************************************************************
 */

displayLoginForm();