<?php
/**
 * RSS feed viewer
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.4.0
 */

// width & height of thumbnails. Note: if modified, .rssImg must also be modified in /themes/mobile/theme.css
define('RSS_MOBILE_THUMBNAIL_WIDTH',48);
define('RSS_MOBILE_THUMBNAIL_HEIGHT',32);
define('RSS_MOBILE_THUMBNAIL_RESAMPLE_QUALITY',4);
define('RSS_MOBILE_THUMBNAIL_JPG_QUALITY',65);

require(INCLUDE_DIR.'mobileFunctions.php');

/**
 * @desc Download and resize feed item's thumbnail
 *
 * @param string $thumbnailId: feedId_itemId
 */
function mobileRSSDisplayResizedThumbnail($thumbnailId){
	global $rssFeeds;
	wSession_write_close(false);

	// Display error image in case of problem
	function tnError(){
		WHeaders::contentType('png');
		die(file_get_contents(cfAppDocRoot().outIcon('broken')));
	}

	WHeaders::cache();

	// Parse get parameter : feedId_itemSequentialNumber
	list($rssFeedId,$itemSeqNumber)=@explode('_',$thumbnailId);
	if(!isset($itemSeqNumber)||!is_numeric($itemSeqNumber) || !isset($rssFeeds[$rssFeedId])) tnError();

	// Parse XML to retreive image's URL
	$feed=$rssFeeds[$rssFeedId];
	$rss=new XML_RSS($feed->xmlValue);
	$rss->parse();
	if(!($item=cfArrayItem($rss->getItems(),$itemSeqNumber))) tnError();
	$url=$feed->getDescriptionImage($item);

	// Download image from site
	if(!($image=cfSocketHTTPRequest($url,5))) tnError();

	WHeaders::contentType('jpg');
//die($image);
	// Resize image
	if(!($sourceImage=imagecreatefromstring($image))) tnError();
	$sourceWidth=imagesx($sourceImage); $sourceHeight=imagesy($sourceImage);
	if(!$sourceHeight) tnError();
	$r=RSS_MOBILE_THUMBNAIL_WIDTH/RSS_MOBILE_THUMBNAIL_HEIGHT;
	if($sourceWidth/$sourceHeight>$r){
		$srcAreaWidth=$sourceHeight*$r;
		$srcAreaHeight=$sourceHeight;
		$srcX=$sourceWidth/2-$srcAreaWidth*$r/2;
		$srcY=0;
	}
	else{
		$srcAreaWidth=$sourceWidth;
		$srcAreaHeight=$sourceWidth/$r;
		$srcX=0;
		$srcY=$sourceHeight/2-$srcAreaHeight/2;
	}
	$destImage = imagecreatetruecolor(RSS_MOBILE_THUMBNAIL_WIDTH,RSS_MOBILE_THUMBNAIL_HEIGHT);
	fastimagecopyresampled($destImage, $sourceImage, 0, 0, $srcX, $srcY, RSS_MOBILE_THUMBNAIL_WIDTH,RSS_MOBILE_THUMBNAIL_HEIGHT, $srcAreaWidth, $srcAreaHeight, RSS_MOBILE_THUMBNAIL_RESAMPLE_QUALITY);

	// Send to browser
	imagejpeg($destImage,null,RSS_MOBILE_THUMBNAIL_JPG_QUALITY);
	exit;
}

/**
 * @desc Init mobile specific data
 *
 */
function mobileRSSInitResourceData(){
	global $tabs;
	if(cfRGetVar('mobileParametersLoaded')) return;

	// Default selected view
	if(count($tabs)>1) cfRSetVar('view','tabs'); else cfRSetVar('view','feeds');

	cfRSetVar('mobileParametersLoaded',1);
}

/**
 * @desc Top controls
 *
 * @return string HTML code
 */
function mobileRSSTopControls(){
	global $tabs;
	$out=mfButton(false,'javascript:setView(\'tabs\')',outIcon('rssTab'),'id="tabsBt"',(cfRGetVar('view')=='tabs')?'on':false).'&nbsp;';
	if(count($tabs)<2) $out=''; // Do not display tab selection icon if only one tab
	//foreach ($tabs as $tab)	$out.=mfButton('',"javascript:setTab('".$tab->id."')",(($tab->iconURL)?$tab->iconURL:outIcon('rss')));

	// Options drop down menu
	$ico=outIcon('ok16');
	$space=outIcon('w0-16');
	$feedsDisplay=cfRGetVar('feedsDisplay');
	$out.=mfDropDownMenu(outIcon('gearsM'),false,array(
		array('selected'=>($feedsDisplay=='number'),'caption'=>cfRGetVar('maxItems'),'onclick'=>"setFeedsDisplay('number')"),
		array('selected'=>($feedsDisplay=='day'),'caption'=>cfCaption('genToday'),'onclick'=>"setFeedsDisplay('day')"),
		array('selected'=>($feedsDisplay=='last'),'caption'=>cfCaption('genNotRead'),'onclick'=>"setFeedsDisplay('last')"),
		array('selected'=>($feedsDisplay=='all'),'caption'=>cfCaption('genAll'),'onclick'=>"setFeedsDisplay('all')")
	));
	return $out;
}

/**
 * @desc Insert javascript
 */
function mobileRSSInsertScripts(){
	?>
<script type="text/javascript">
var mIconSrc="<?php echo outIcon('minus');?>",pIconSrc="<?php echo outIcon('plus');?>",lIconSrc="<?php echo outIcon('loading');?>",sDet={},detId;
function setView(v){wl.liSel(dgi(v+'Bt'));fAS('','setView',v)}
function setTab(n){
	if(typeof n=="object"){
		wl.liSel(n); fAS('','tabSelect',n.id)
	} else fAS('','tabSelect',n)
}
function ol(l){W.open(l,"_blank")}
function ufl(id,fromBt){
	dgi('detTitle').innerHTML=dgi('det'+id).previousSibling.innerHTML;
	fAS(id,'getFeedInfo',1,0,'async');
}
function itemSetDetail(id,t){
	wl.scrollSave('feeds')
	detId=id;
	sDet[id]=t;
	dgi('detTxt').innerHTML=t.extra.desc.c
	dgi('detDiv').style.display='';
	dgi('detDiv').style.left='0';
	dgi('feedsDiv').style.left='-100%';
	wl.scrollRestore('det')
}
function detGo(){dgi("go"+detId).onclick()}
function showFeeds(){
	dgi('loading').style.display='none';
	dgi('detDiv').style.display='none';
	dgi('feedsDiv').style.display='';
	dgi('detDiv').style.left='100%';
	dgi('feedsDiv').style.left='0';
	wl.scrollRestore('feeds')
}
function setFeedsDisplay(d){
	fAS(d,'setFeedsDisplay');
	feedsDisplay=d;
}

</script>
<?php
}

// Init mobile specific data
mobileRSSInitResourceData();


/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */

if(isset($_POST['data3'])) switch ($_POST['data3']){
	case 'setView':
		cfRSetVar('view',($_POST['data4']=='tabs')?'tabs':'feeds');
	break;
	case 'tabSelect';
		if(isset($tabs[$_POST['data4']])){
			cfRSetVar('selectedTab', $_POST['data4']);
			cfRSetVar('view', 'feeds');
		}
	break;
	// Async get feed details
	case 'getFeedInfo':
		cfAsyncHeader();
		list($feed,$itemNb)=@explode('_',$_POST['data2']);
		if(isset($rssFeeds[$feed]) && is_numeric($itemNb)) {
			if($info=$rssFeeds[$feed]->tooltipArray($itemNb,array('rawText'=>1))){
				require_once(INCLUDE_DIR.'fileInfoFunctions.php');
				echo cfAsyncXMLJSaction('itemSetDetail("'.$_POST['data2'].'",'.fiPHPArrayToJSArray($info).')');
			}
		}
		die(cfAsyncFooter());
	// Change feed's max displayed items
	case 'setFeedsDisplay':
		if(in_array($_POST['data2'],array('all','day','last','number'))) cfRSetVar('feedsDisplay',$_POST['data2']);
	break;
}

/*
 ***************************************************************************************************************************
 * Display RSS feeds images (load them from site and resize)
 ***************************************************************************************************************************
 */
if(isset($_GET['tn'])) mobileRSSDisplayResizedThumbnail($_GET['tn']);


/*
 ***************************************************************************************************************************
 * Insert HEAD, javascripts and comForm
 ***************************************************************************************************************************
 */
mfInsertHead(array('topControlsExtra'=>mobileRSSTopControls()));
outInsertStandardComForm($_SERVER['PHP_SELF']);
mobileRSSInsertScripts();

/*
 ***************************************************************************************************************************
 * Tabs view
 ***************************************************************************************************************************
 */
if(cfRGetVar('view')=='tabs'){
	echo mfListTable();
	foreach ($tabs as $tab){
		echo mfListItem('onclick="setTab(this)" id="'.$tab->id.'"','<img src="'.(($tab->iconURL)?$tab->iconURL:outIcon('rss')).'" class="frW ico">',false,'<div class="lits">&nbsp;'.($tab->name).'</div>',false,'go',false);
	}
	echo '</table>';
	//cfVarDump($tabs);
	exit;
}

/*
 ***************************************************************************************************************************
 * Feeds view
 ***************************************************************************************************************************
 */
$tabId=cfRGetVar('selectedTab');
// Loading indicator
?>
<img id="loading" src="<?php echo outIcon('loading');?>" style="position:absolute;top:50%;left:50%">
<?php
flush();

// Overflow div
?>
<div id="ovDiv" style="width:100%;overflow:hidden;display:block;height:auto">
<?php

// Feed detail div
?>
<div id="detDiv" style="position:absolute;left:100%;width:100%;display:none">
<?php
// Header
echo mfListTable(false,'h1');
echo mfListItem(false,outImage(outIcon('back'),false,false,'margin-right:16px'),'onclick="showFeeds()"','<div class="lit" id="detTitle"></div>','onclick="detGo()"',outImage(outIcon('go')));
echo '</table>';
// Feed detail
echo mfListTable(' style="table-layout:fixed"');
echo mfListItem(false,false,false,'<div id="detTxt" style="font-size:130%;padding:1em"></div>',false);
echo '</table>';
?>
</div>
<?php

// Feed detail div
?>
<div id="feedsDiv" style="display:none;position:absolute;width:100%">
<?php

/**
 * Update feeds
 */
$urls=array();
foreach ($rssFeeds as $id=>$feed) {if($feed->tab==$tabId && $feed->nextUpdate()==0) $urls[$id]=$feed->url;}

// Retreive XML
$xmls=cfCURLMultiRequests($urls);

// Apply retreived XML to feeds
$needSave=false;
foreach ($rssFeeds as $id=>$feed){
	foreach ($xmls as $k=>$v) if($k==$id){
		if($feed->updateXML($v)=='OK') {
			$rssFeeds[$id]=$feed;
			$needSave=true;
		}
	}
}
if($needSave) saveFeeds($rssFeeds);

/**
 * Display feeds
 */

// Display feeds
foreach ($rssFeeds as $feed) if($feed->tab==$tabId){
	echo $feed->mobileHTML(false);
}
?>
</div>
</div>
<script type="text/javascript">showFeeds()</script>