<?php
/**
 * DEFAULT THEME LOGIN FORM
 *
 * Included by login.php
 * Display login form
 *
 * See /login.php for used variables
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/*
 * List of parameters sent to loginForm.php script
 * - $_ENV['users'][] : arrays of user accounts to be displayed (array key is user id)
 *		user data :
 * 			['authenticationMethod'] : user authentication method : 'noAuthentication' or 'password'
 * 			['name'] (optional) : user name. Set to false if both name and user Id must be entered
 * 			['icon'] : user icon file basename (add /gfx/icons/ to have full path)
 * 			['hint'] (optional) : password hint, to be displayed
 * - $userToDisplay () optional : sequential id of selected user (set if an user name has been passed in get args or if a wrong password has been typed),
 * 			(set to 0 (zero) if login+password) input must be selected)
 * - $userAndPasswordMethod : true if login+password form has been used
 * - $wrongPasswordSent : true if a wrong password has been entered
 * - $wrongUserSent : true if a non-existing user name has been entered
 * - $hiddenUsers : true if at least one hidden user account exists => user name+password form must be displayed
*/


// number of forms to be displayed
$nbForms=count($_ENV['users']);

// sequential number of pre-selected form
if(isset($userToDisplay)){
	$i=0; foreach ($_ENV['users'] as $key=>$value) {if($key===$userToDisplay) $userToDisplaySeqNumber=$i;	$i++;}
}
else $userToDisplaySeqNumber=round($nbForms/2)-1;



/*
 ***************************************************************************************************************************
 * Async Response
 ***************************************************************************************************************************
 */
if(cfIsAsync() && isset($_POST['user'])){
	echo cfAsyncHeader();

	// If security issue detected, display error message
	if($securityEvent)echo cfAsyncXMLJSaction('dgi("securityEvent").innerHTML="'.str_replace('"',"''",cfUTF8Encode($securityEvent)).'";dgi("securityEvent").style.display="";');
	echo cfAsyncXMLJSaction('if(W.resetInputs) resetInputs();');
	if($wrongPasswordSent || $wrongUserSent){
		$errorText=str_replace('"',"''",(($wrongUserSent)?cfCaption('loginBadPasswordOrAccount'):cfCaption('loginBadPassword')));
		echo cfAsyncXMLJSaction('loginSetError("'.$errorText.'")');
	}
	echo cfAsyncXMLJSaction('wl.mask.hide()');
	cfAsyncFooter();
}



/*
 ***************************************************************************************************************************
 * Display page (sync Response)
 ***************************************************************************************************************************
 */

// Insert HTML Head and Javascript
cfInsertHEAD(false);
echo cfScriptLink('loginCoverflow.js');
?>
<script type="text/javascript">
var nb=<?php echo count($_ENV['users']);?>,udSN=<?php echo $userToDisplaySeqNumber;?>;
</script>
</head>
<body class="loginBody">
<input id="noFocus" style="position:absolute;top:-20px">
<?php


// Background image
$bgImage='/themes/common/loginBg.png';
foreach (array('jpg','gif','png') as $ext){
	if(file_exists(cfAppThemeDir().'/loginBg.'.$ext)) {
		$bgImage=cfAppThemeDir(true).'/loginBg.'.$ext;
		break;
	}
}


//cfVarDump($_ENV['users']);
echo cfScriptLink('wz_dragdrop.js');
includeLoginFormAndScripts();
?>
<img ig="bgImage" style="position:absolute;top:50px;left:0px;width:100%;height:100%;z-index:0" src="<?php echo $bgImage; ?>">
<div id="debug"></div>
<div style="position:absolute;left:50%;top:50%" id="loginCenterPos">
<?php


/****************************************************************************************************************
 * Display Users
 ***************************************************************************************************************/
$i=0;$jsArray='';

foreach ($_ENV['users'] as $id=>$user){
	echo '<div id="l'.$i.'" class="loginItem">';
	echo outImage('/themes/common/loginFrame.png',false,false,'width:133px;height:166px;display:block'.((cfGetBrowser()=='ie')?';visibility:visible':''),1,false,true);
	echo '<img onload="fade(this)" src="/gfx/icons/'.$user['icon'].'">';
	echo '<div class="loginName" id="capt'.$i.'">';
	if($user['name']) echo cfUTF8Encode($user['name']);
	elseif(count($_ENV['users'])>1) echo cfCaption('loginOtherGroup');
	echo '</div>';
	echo '</div>';
	$jsArray.=',"'.(int)$i.'":{"id":"'.@$user['id'].'","password":'.(int)($user['authenticationMethod']!=='noAuthentication').',"hint":"'.str_replace('"','\\"',cfUTF8Encode(@$user['hint'])).'"'.(($user['name'])?'':',"reqUser":1').'}';
	$i++;
}

/**
 * Display additional controls (group input, password input, go button, error & hint info
 */
?>
	<div id="loginBox">
		<div id="loginBoxStuff">
		<div id="loginUserGroup">
			<div id="loginUserLb" style="width:140px"><?php echo cfCaption('genGroup');?></div>
			<input id="loginUserInput" class="textInput" >
		</div>
		<div id="loginPasswordGroup">
			<div id="loginPasswordLb" style="width:100%"><?php echo cfCaption('genPassword');?></div>
			<input id="loginPasswordInput" type="password" class="textInput" >
		</div>
		<div style="text-align:center"><?php echo outButton(cfCaption('loginConnectButton'),'javascript:go()',outIcon('go'));?></div>
		<div id="loginHintGroup" style="position:absolute;left:50%;margin-top:5px">
			<div class="loginBox" style="position:relative;left:-50%">
				<?php echo outImageIcon('bulb');?>
				<span id="loginHint" style="width:50%;overflow:hidden;width:auto">&nbsp;</span>
			</div>
		</div>
		<div id="loginError" class="loginBox warning" style="visibility:hidden;width:133px;margin-top:40px;text-align:center"></div>
		</div>


	</div>
</div>
<div id="loginDragMask" style="position:absolute;width:50px;height:50px;background:white;<?php echo outCssOpacity(1);?>;z-index:2000"></div>
<?php


/**
 * Display security messages
 */
echo '<span id="securityEvent" class="loginBox warning" style="'.((@$securityEvent)?'':'display:none').'">'.@$securityEvent.'</span>';

/**
 * Display Weezo frame
 */
?>
<div class="loginBox loginAppFrame">
<img src="<?php echo outIcon('appIcon');?>" style="float:left;margin-right:1em;visibility:hidden" onload="fade(this)">
<b><?php echo APPLICATION_NAME;?></b><br>
<?php echo '<a href="'.APP_SITE.'" target="_blank" class="link" >'.APP_SITE.'</a>';?><br>
<?php echo cfGGetVar('appVersion');?>
</div>
<?php

/**
 * Theme selection control
 */
if(cfGGetVar('allowUserThemeChange')) echo '<div class="loginBox loginThemeSelection">'.outThemeSelectionControl('general',false,'POST').'</div>';

/**
 * Display User-defined login message
 */
if($message=cfGGetVar('loginMessage')) {
	if(strpos(str_replace('<br />','',$message),'<')!==false)
		echo '<div style="z-index:999;position:absolute;top:5px;left:5px">'.$message.'</div>';
	else
		echo '<span class="loginBox loginMessage">'.cfUTF8Encode($message,false,false).'</span>';
}

/**
 * Mobile access control
 */
?>
<div id="loginForceMobile" class="loginBox" onclick="forceMobile()"><?php echo outImage(outIcon('mobile32'));?></div>

<?php
/**
 * SSL access (if main server port is non-SSL and alternateServerPort is 443
 */
if(cfGGetVar('alternateServerPort')==443 && cfGGetVar('serverPort')!=443){
?>
<div id="loginAlternateServerPort" class="loginBox" onclick="setAlternateServerPort()"><?php echo outImage(outIcon('ssl32'));?></div>
<?php } ?>

<script type="text/javascript">
var users={<?php echo substr($jsArray,1);?>};
D.onmousemove=function(e){
	if(!dMask) {
		dMask=dd.elements['loginDragMask'];
		dMask.setPickFunc(dMaskPick)
		dMask.setDragFunc(dMaskDrag)
	}
	if(getItemBelowMouse()!=-1) dMask.setCursor('hand'); else dMask.setCursor('default');
	dMask.div.style.cursor=(getItemBelowMouse()!=-1)?'pointer':'default';
	if(mouseY>actualOffsetTop(dgi('loginBoxStuff')) || mouseY<actualOffsetTop(dgi('loginCenterPos'))-133/2) dMask.moveTo(-100,-100);
	else dMask.moveTo(mouseX-25,mouseY-25)
}
<?php
cfDragAddItem('loginDragMask');
echo cfDragRegisterItems('',false);
?>
init();
</script>
</body>